//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "StdAfx.h"
#include "preferences.h"
#include <time.h>
#include "opcodes.h"
#include "otherfunctions.h"
#include "ini2.h"
#include "stdlib.h"
#include "stdio.h"
#include <locale.h>
#include "resource.h"
#include "emule.h"
#include "memcpy_amd.h"
#include <io.h>

// For SecuredVars
const DWORD CCriticalSection_INL::s_dwSafeValue = 0x159D37BF;


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define ELEMENT_COUNT(X) (sizeof(X) / sizeof(X[0]))

bool CPreferences::bWinVerAlreadyDetected = false;

CPreferences::CPreferences(){
	srand((uint32)time(0)); // we need random numbers sometimes
	m_bIsNTBased = false;
	prefs = new Preferences_Struct_DONTUSE;
//	memset(prefs,0,sizeof(Preferences_Struct));
	prefsExt=new Preferences_Ext_Struct;
	memset(prefsExt,0,sizeof(Preferences_Ext_Struct));

	//get application start directory
	TCHAR path[MAX_PATH];
	::GetModuleFileName(NULL, path, MAX_PATH);
	
	TCHAR drive[_MAX_DRIVE];
	TCHAR dir[_MAX_DIR];
	_tsplitpath(path, drive, dir, NULL, NULL); 
	
	appdir = CString(drive) + dir;	

	CreateUserHash();
	#ifndef AMD
	md4cpy(&prefs->userhash,&userhash);
	#else
	memcpy_amd(&prefs->userhash,&userhash,16);
	#endif

	// load preferences.dat or set standart values	
	CString fullpath;
	fullpath.Format(_T("%spreferences.dat"),appdir);
	FILE* preffile = _tfopen(fullpath,_T("rb"));

	LoadPreferences();
	if (!preffile){
		SetStandartValues();
		//if (Ask4RegFix(true)) Ask4RegFix(false);
	}
	else{
		fread(prefsExt,sizeof(Preferences_Ext_Struct),1,preffile);
		if (ferror(preffile))
			SetStandartValues();

		// import old pref-files
		if (prefsExt->version<20) {


			if (prefsExt->version>17) {// v0.20b+
				prefsImport20b=new Preferences_Import20b_Struct;
				memset(prefsImport20b,0,sizeof(Preferences_Import20b_Struct));
				fseek(preffile,0,0);
				fread(prefsImport20b,sizeof(Preferences_Import20b_Struct),1,preffile);

				#ifndef AMD
				md4cpy(&prefs->userhash,&prefsImport20b->userhash);
				#else
				memcpy_amd(&prefs->userhash,&prefsImport20b->userhash,16);
				#endif
				m_sIncomingDir.Put(prefsImport20b->incomingdir);
				m_sTempDir.Put(prefsImport20b->tempdir);
				m_sNick.Put(prefsImport20b->nick);

				m_nTotalDownloadedBytes.Put(prefsImport20b->totalDownloadedBytes);
				m_nTotalUploadedBytes.Put(prefsImport20b->totalUploadedBytes);

			} else if (prefsExt->version>7) { // v0.20a
				prefsImport20a=new Preferences_Import20a_Struct;
				memset(prefsImport20a,0,sizeof(Preferences_Import20a_Struct));
				fseek(preffile,0,0);
				fread(prefsImport20a,sizeof(Preferences_Import20a_Struct),1,preffile);

				#ifndef AMD
				md4cpy(&prefs->userhash,&prefsImport20a->userhash);
				#else
				memcpy_amd(&prefs->userhash,&prefsImport20a->userhash,16);
				#endif
				m_sIncomingDir.Put(prefsImport20a->incomingdir);
				m_sTempDir.Put(prefsImport20a->tempdir);
				m_sNick.Put(prefsImport20a->nick);

				m_nTotalDownloadedBytes.Put(prefsImport20a->totalDownloaded);
				m_nTotalUploadedBytes.Put(prefsImport20a->totalUploaded);

			} else {	//v0.19c-
				prefsImport19c=new Preferences_Import19c_Struct;
				memset(prefsImport19c,0,sizeof(Preferences_Import19c_Struct));

				fseek(preffile,0,0);
				fread(prefsImport19c,sizeof(Preferences_Import19c_Struct),1,preffile);

				if (prefsExt->version<3) {
					CreateUserHash();
					#ifndef AMD
					md4cpy(&prefs->userhash,&userhash);
					#else
					memcpy_amd(&prefs->userhash,&userhash,16);
					#endif
				} 
				else 
					#ifndef AMD
					md4cpy(&prefs->userhash,&prefsImport19c->userhash);
					#else
					memcpy(&prefs->userhash,&prefsImport19c->userhash,16);
					#endif
				m_sIncomingDir.Put(prefsImport19c->incomingdir);
				m_sTempDir.Put(prefsImport19c->tempdir);
				m_sNick.Put(prefsImport19c->nick);
			}
 		} else {
 			#ifndef AMD
			md4cpy(&prefs->userhash,&prefsExt->userhash);
 			#else
 			memcpy_amd(&prefs->userhash,&prefsExt->userhash,16);
 			#endif
			prefs->EmuleWindowPlacement=prefsExt->EmuleWindowPlacement;
		}
		fclose(preffile);
		#ifndef AMD
		md4cpy(&userhash,&prefs->userhash);
		#else
		memcpy_amd(&userhash,&prefs->userhash,16);
		#endif
		m_nSmartIdState = 0;
	}

	// shared directories	
	fullpath.Format(_T("%sshareddir.dat"),appdir);
	CStdioFile sdirfile;
	if(sdirfile.Open(fullpath,CFile::modeRead))
	{
		CString toadd;
		while (sdirfile.ReadString(toadd))
			shareddir_list.AddHead(toadd);
		sdirfile.Close();
	}

	// start added by InterCeptor (multiple tempdirs) 19.11.02
	// temp directories	
	fullpath.Format(_T("%stempdir.dat"),appdir);
	CStdioFile tdirfile;
	if (tdirfile.Open(fullpath,CFile::modeRead)){
		CString toadd;
		while (tdirfile.ReadString(toadd)){
			if(tempdir_list.Find(toadd) == NULL)
			{
				tempdir_list.AddHead(toadd);
				::CreateDirectory(toadd,0);
			}
		}
		tdirfile.Close();
	}
	
	// end added by InterCeptor (multiple tempdirs) 19.11.02

	
	
	//serverlist adresses
	LoadServerlistAddresses() ;

	userhash[5] = 14;
	userhash[14] = 111;

	//Maverick
    m_bUseProxyListenPort = false;
	m_nListenPort = 0;

	CreateDirectory(GetIncomingDir(),0);
	CreateDirectory(GetTempDir(),0);
}

// Added by Tarod
void CPreferences::LoadServerlistAddresses()
{
	// First we must free list
	adresses_list.RemoveAll() ;

	CString toadd ;
	CString fullpath = CString(appdir) + _T("adresses.dat") ;
	CStdioFile sdirfile;
	if (sdirfile.Open(fullpath, CFile::modeRead)) {
		adresses_list.RemoveAll() ;
		while (sdirfile.ReadString(toadd)) {
        	// [i_a] allow commented-out entries in the addresses.dat file.
                toadd.TrimLeft();
                if (toadd.GetLength() > 0 && toadd[0] != '#')
			adresses_list.AddHead(toadd) ;
		}
		sdirfile.Close() ;
	}	
}

void CPreferences::SetStandartValues(){
	// CreateUserHash();
	// Saving HashID 
	CString fullpath;
	fullpath.Format(_T("%suserhash.dat"), appdir) ;

	FILE* hashfile = _tfopen(fullpath, _T("rb")) ;
	
	if (hashfile) { 
		if (!fread(userhash, sizeof(userhash), 1, hashfile)) {
            CreateUserHash();
		}
		fclose(hashfile) ;
	}
	// End of Saving HashID 
	#ifndef AMD
	md4cpy(&prefs->userhash,&userhash);
	#else
	memcpy_amd(&prefs->userhash,&userhash,16);
	#endif

	WINDOWPLACEMENT defaultWPM;
	defaultWPM.length = sizeof(WINDOWPLACEMENT);
	defaultWPM.rcNormalPosition.left=10;defaultWPM.rcNormalPosition.top=10;
	defaultWPM.rcNormalPosition.right=700;defaultWPM.rcNormalPosition.bottom=500;
	defaultWPM.showCmd=0;
	prefs->EmuleWindowPlacement=defaultWPM;

	Save();
}

uint16 CPreferences::GetMaxDownload(){
	//dont be a Lam3r :)
	uint16 nMaxUp = GetMaxUpload();
	uint16 nMaxDown = GetMaxDownloadValue();
	if(nMaxUp >= 10 && LimitlessDownload()) // DonGato
		return UNLIMITED;
	if(nMaxUp < 4)
		return ((nMaxUp * 3 < nMaxDown) ? nMaxUp * 3 : nMaxDown);
	return (((nMaxUp < 10) && (nMaxUp * 4 < nMaxDown)) ? nMaxUp * 4 : nMaxDown);
}

bool CPreferences::Save(){

	bool error = false;
	CString fullpath;
	fullpath.Format(_T("%spreferences.dat"),appdir);

	FILE* preffile = _tfopen(fullpath,_T("wb"));
	prefsExt->version = PREFFILE_VERSION;
	
	if (preffile){
		prefsExt->version=PREFFILE_VERSION;
		prefsExt->EmuleWindowPlacement=prefs->EmuleWindowPlacement;
		#ifndef AMD
		md4cpy(&prefsExt->userhash,&prefs->userhash);
		#else
		memcpy_amd(&prefsExt->userhash,&prefs->userhash,16);
		#endif

		error = fwrite(prefsExt,sizeof(Preferences_Ext_Struct),1,preffile);
		// v1:- eklmn: bugfix(19): by shutdown force OS flush data direct to disk [SlugFiller]
		if ( !theApp.emuledlg->IsRunning() ) _commit(_fileno(preffile));
		fclose(preffile);
	}
	else
		error = true;

	// Saving HashID 	
	fullpath.Format(_T("%suserhash.dat"), appdir) ;

	FILE* hashfile = _tfopen(fullpath, _T("wb")) ;
	
	if (hashfile) {
		error = fwrite(prefs->userhash, sizeof(prefs->userhash), 1, hashfile) ;
		// v1:- eklmn: bugfix(19): by shutdown force OS flush data direct to disk [SlugFiller]
		if ( !theApp.emuledlg->IsRunning() ) _commit(_fileno(hashfile));
		fclose(hashfile) ;
	}
	else
		error = true ;
	// End of Saving HashID

	SavePreferences();
	
	fullpath.Format(_T("%sshareddir.dat"),appdir);
	CStdioFile sdirfile;
	if (sdirfile.Open(fullpath,CFile::modeCreate|CFile::modeWrite)){

		for (POSITION pos = shareddir_list.GetHeadPosition();pos != 0;shareddir_list.GetNext(pos)){
			sdirfile.WriteString(shareddir_list.GetAt(pos).GetBuffer());
			sdirfile.Write("\n",1);
		}
		sdirfile.Close();
	}
	else
		error = true;

	// start added by InterCeptor (multiple tempdirs) 19.11.02	
	fullpath.Format(_T("%stempdir.dat"),appdir);
	CStdioFile tdirfile;
	if (tdirfile.Open(fullpath,CFile::modeCreate|CFile::modeWrite)){
		for (POSITION pos = tempdir_list.GetHeadPosition();pos != 0;tempdir_list.GetNext(pos)){
			tdirfile.WriteString(tempdir_list.GetAt(pos).GetBuffer());
			tdirfile.Write("\n",1);
		}
		tdirfile.Close();
	}
	else
		error = true;
	// end added by InterCeptor (multiple tempdirs) 19.11.02	
	CreateDirectory(GetIncomingDir(),0);
	CreateDirectory(GetTempDir(),0);
	return error;
}

void CPreferences::CreateUserHash(){
	for (int i = 0;i != 8; i++){ 
		uint16	random = rand();
		#ifndef AMD
		memcpy(&userhash[i*2],&random,2);
		#else
		memcpy_amd(&userhash[i*2],&random,2);
		#endif
	}
	// mark as emule client. that will be need in later version
	userhash[5] = 14;
	userhash[14] = 111;
}

int CPreferences::GetColumnWidth(Table t, int index) const {
	switch(t) {
	case tableDownload:
		return prefs->downloadColumnWidths[index];
	case tableUpload:
		return prefs->uploadColumnWidths[index];
	case tableQueue:
		return prefs->queueColumnWidths[index];
	case tableSearch:
		return prefs->searchColumnWidths[index];
	case tableShared:
		return prefs->sharedColumnWidths[index];
	case tableServer:
		return prefs->serverColumnWidths[index];
	case tableIrc:
		return prefs->ircColumnWidths[index];
	}
	return 0;
}

void CPreferences::SetColumnWidth(Table t, int index, int width) {
	switch(t) {
	case tableDownload:
		prefs->downloadColumnWidths[index] = width;
		break;
	case tableUpload:
		prefs->uploadColumnWidths[index] = width;
		break;
	case tableQueue:
		prefs->queueColumnWidths[index] = width;
		break;
	case tableSearch:
		prefs->searchColumnWidths[index] = width;
		break;
	case tableShared:
		prefs->sharedColumnWidths[index] = width;
		break;
	case tableServer:
		prefs->serverColumnWidths[index] = width;
		break;
	case tableIrc:
		prefs->ircColumnWidths[index] = width;
		break;
	}
}

BOOL CPreferences::GetColumnHidden(Table t, int index) const {
	switch(t) {
	case tableDownload:
		return prefs->downloadColumnHidden[index];
	case tableUpload:
		return prefs->uploadColumnHidden[index];
	case tableQueue:
		return prefs->queueColumnHidden[index];
	case tableSearch:
		return prefs->searchColumnHidden[index];
	case tableShared:
		return prefs->sharedColumnHidden[index];
	case tableServer:
		return prefs->serverColumnHidden[index];
	case tableIrc:
		return prefs->ircColumnHidden[index];
	}
	return FALSE;
}

void CPreferences::SetColumnHidden(Table t, int index, BOOL bHidden) {
	switch(t) {
	case tableDownload:
		prefs->downloadColumnHidden[index] = bHidden;
		break;
	case tableUpload:
		prefs->uploadColumnHidden[index] = bHidden;
		break;
	case tableQueue:
		prefs->queueColumnHidden[index] = bHidden;
		break;
	case tableSearch:
		prefs->searchColumnHidden[index] = bHidden;
		break;
	case tableShared:
		prefs->sharedColumnHidden[index] = bHidden;
		break;
	case tableServer:
		prefs->serverColumnHidden[index] = bHidden;
		break;
	case tableIrc:
		prefs->ircColumnHidden[index] = bHidden;
		break;
	}
}

int CPreferences::GetColumnOrder(Table t, int index) const {
	switch(t) {
	case tableDownload:
		return prefs->downloadColumnOrder[index];
	case tableUpload:
		return prefs->uploadColumnOrder[index];
	case tableQueue:
		return prefs->queueColumnOrder[index];
	case tableSearch:
		return prefs->searchColumnOrder[index];
	case tableShared:
		return prefs->sharedColumnOrder[index];
	case tableServer:
		return prefs->serverColumnOrder[index];
	case tableIrc:
		return prefs->ircColumnOrder[index];
	}
	return 0;
}

#ifndef AMD
void CPreferences::SetColumnOrder(Table t, INT *piOrder) {
	switch(t) {
	case tableDownload:
		memcpy(prefs->downloadColumnOrder, piOrder, sizeof(prefs->downloadColumnOrder));
		break;
	case tableUpload:
		memcpy(prefs->uploadColumnOrder, piOrder, sizeof(prefs->uploadColumnOrder));
		break;
	case tableQueue:
		memcpy(prefs->queueColumnOrder, piOrder, sizeof(prefs->queueColumnOrder));
		break;
	case tableSearch:
		memcpy(prefs->searchColumnOrder, piOrder, sizeof(prefs->searchColumnOrder));
		break;
	case tableShared:
		memcpy(prefs->sharedColumnOrder, piOrder, sizeof(prefs->sharedColumnOrder));
		break;
	case tableServer:
		memcpy(prefs->serverColumnOrder, piOrder, sizeof(prefs->serverColumnOrder));
		break;
	case tableIrc:
		memcpy(prefs->ircColumnOrder, piOrder, sizeof(prefs->ircColumnOrder));
		break;
	}
}
#else
void CPreferences::SetColumnOrder(Table t, INT *piOrder) {
	switch(t) {
	case tableDownload:
		memcpy_amd(prefs->downloadColumnOrder, piOrder, sizeof(prefs->downloadColumnOrder));
		break;
	case tableUpload:
		memcpy_amd(prefs->uploadColumnOrder, piOrder, sizeof(prefs->uploadColumnOrder));
		break;
	case tableQueue:
		memcpy_amd(prefs->queueColumnOrder, piOrder, sizeof(prefs->queueColumnOrder));
		break;
	case tableSearch:
		memcpy_amd(prefs->searchColumnOrder, piOrder, sizeof(prefs->searchColumnOrder));
		break;
	case tableShared:
		memcpy_amd(prefs->sharedColumnOrder, piOrder, sizeof(prefs->sharedColumnOrder));
		break;
	case tableServer:
		memcpy_amd(prefs->serverColumnOrder, piOrder, sizeof(prefs->serverColumnOrder));
		break;
	case tableIrc:
		memcpy_amd(prefs->ircColumnOrder, piOrder, sizeof(prefs->ircColumnOrder));
		break;
	}
}
#endif

CPreferences::~CPreferences(){	
	delete prefs;
	delete prefsExt;
}

int CPreferences::GetRecommendedMaxConnections() {
	int iRealMax = ::GetMaxConnections();
	if(iRealMax == -1 || iRealMax > 520)
		return 500;

	if(iRealMax < 20)
		return iRealMax;

	if(iRealMax <= 256)
		return iRealMax - 10;

	return iRealMax - 20;
}

void CPreferences::SavePreferences()
{
	EMULE_TRY
	CString buffer;
	CString fullpath; // i_a
	fullpath.Format(_T("%spreferences.ini"),appdir);
	
	CIni ini( fullpath, _T("eMule") );

	//useful!
	ini.WriteString(_T("AppVersion"), CURRENT_VERSION_LONG);

	buffer = m_sNick.Get();
	ini.WriteString(_T("Nick"),buffer);

	buffer = m_sIncomingDir.Get();
	ini.WriteString(_T("IncomingDir"),buffer );

	buffer = m_sTempDir.Get();
	ini.WriteString(_T("TempDir"),buffer );

	buffer = m_sVideoPlayer.Get();	//Cax2 - official mule code.
	ini.WriteString(_T("VideoPlayer"),buffer );

	ini.WriteInt(_T("MaxUpload"), m_nMaxUpload);
	ini.WriteInt(_T("MaxDownload"), m_nMaxDownload);
	ini.WriteInt(_T("MaxConnections"), m_nMaxConnections);
	ini.WriteInt(_T("DeadServerAction"), m_nDeadServer);
	ini.WriteInt(_T("Port"), m_nPort);
	ini.WriteInt(_T("UDPPort"), m_nUDPPort);
	ini.WriteInt(_T("MaxSourcesPerFile"), m_nMaxSourcePerFile);
	ini.WriteWORD(_T("Language"), m_nLanguageID);
	ini.WriteInt(_T("SeeShare"), m_nSeeShares.Get());
	//--- xrmb:partTrafficPrefs ---
	ini.WriteBool(_T("PartTrafficEnabled"), m_bUsePartTraffic);
	ini.WriteInt(_T("UploadbarStyle"), m_nUpbarPartTraffic);
	ini.WriteInt(_T("UploadbarColor"), m_nUpbarColorPartTraffic);
	//--- :xrmb ---
	ini.WriteInt(_T("ToolTipDelay"), m_nTooltipDelayTime);
	ini.WriteInt(_T("QueueSize"), m_nQueueSize);
	//ini.WriteInt(_T("RemoteQueueLimit"), prefs->remotequeuelimit); //remote QR 
	ini.WriteBool(_T("UseRemoteQueueLimit"), m_bUseRQlimit);
	ini.WriteInt(_T("StatGraphsInterval"), m_nTrafficOMeterInterval);
	ini.WriteInt(_T("StatsInterval"), m_nStatsInterval);
	ini.WriteInt(_T("StatsColumnWidth"), m_nStatsColumnWidth);
	ini.WriteInt(_T("DetailColumnWidth"), m_nDetailColumnWidth);
	ini.WriteString(_T("StatsListExpanded"), m_sStatsListExpanded.Get());
	ini.WriteInt(_T("DownloadCapacity"), m_nMaxGraphDownloadRate);
	ini.WriteInt(_T("UploadCapacity"), m_nMaxGraphUploadRate);
	ini.WriteInt(_T("DeadServerRetries"), m_nDeadServerRetries);
	ini.WriteInt(_T("SplitterbarPosition"), m_nSplitterBarPosition);
	ini.WriteInt(_T("VariousStatisticsMaxValue"), m_nStatsMax);
	ini.WriteInt(_T("StatsAverageMinutes"), m_nStatsAverageMinutes);
	ini.WriteInt(_T("MaxConnectionsPerFiveSeconds"), m_nMaxConperFive);

	ini.WriteInt(_T("BadClientBanTime"), m_nBadClientBanTime);
	ini.WriteInt(_T("BadClientMinRequestTime"), m_nMinRequestTime);
	ini.WriteInt(_T("BadClientMinRequestNum"), m_nBadClientBan);
	ini.WriteBool(_T("BanMessageEnabled"), m_bBanMessageEnabled);
	ini.WriteBool(_T("BanEnabled"), m_bBanEnabled); // DbT:BanEnDisable

	ini.WriteInt(_T("SLS_RestoreSourcesTime"), m_nSLSRestoreSourcesTime);
	ini.WriteInt(_T("SLS_WhenSourcesOutdated"), m_nSLSOutdated);
	ini.WriteInt(_T("SLS_MaxSourcesPerFile"),m_SLSMaxSourcesPerFile);
	ini.WriteBool(_T("SLS_Enable"), m_bSLSEnable);

	ini.WriteBool(_T("Reconnect"), m_bReconnect);
	ini.WriteBool(_T("Scoresystem"), m_nScoreSystem);

	ini.WriteBool(_T("UAP"), m_bUAP);
	ini.WriteBool(_T("DAP"), m_bDAP);
	ini.WriteBool(_T("DisableXS"), m_bDisableXS);
	ini.WriteBool(_T("StartDownloadPaused"), m_bStartDownloadPaused);
	ini.WriteBool(_T("DownloadPausedOnComplete"), m_bDownloadPausedOnComplete);
	
	ini.WriteString(_T("ToolbarSettings"), m_sToolbarSettings.Get()); // Added by reCDVst
	buffer = m_sToolbarBitmap.Get();
	ini.WriteString(_T("ToolbarBitmap"),buffer);
	buffer = m_sToolbarBitmapFolder.Get();
	ini.WriteString(_T("ToolbarBitmapFolder"),buffer);
	ini.WriteInt(_T("ToolbarLabels"), m_nToolbarLabels);
	ini.WriteBool(_T("Serverlist"), m_bAutoServerList);
	ini.WriteBool(_T("UpdateNotify"), m_bUpdateNotify);
	ini.WriteBool(_T("MinimizeToTray"), m_bMinToTray);
	ini.WriteBool(_T("AddServersFromConnections"), m_bAddServersFromConn);
	ini.WriteBool(_T("Splashscreen"), m_bSplashScreen);
	ini.WriteBool(_T("BringToFront"), m_bBringToForeground);
	ini.WriteBool(_T("TransferDoubleClick"), m_nTransferDoubleclick);
	ini.WriteBool(_T("BeepOnError"), m_bBeepOnError);
	ini.WriteBool(_T("ConfirmExit"), m_bConfirmExit);
	ini.WriteBool(_T("FilterBadIPs"), m_bFilterBadIP);
	ini.WriteBool(_T("FilterServersByIP"), m_bFilterServersByIP);
	ini.WriteBool(_T("Autoconnect"), m_bAutoConnect);
	ini.WriteBool(_T("OnlineSignature"), m_bOnlineSig);
	ini.WriteBool(_T("StartupMinimized"), m_bStartMinimized);
	ini.WriteBool(_T("SafeConnect"), m_bSafeConnect);
	ini.WriteBool(_T("ToolbarSpeedMeter"), m_bShowToolbarSpeedMeter);
	ini.WriteBool(_T("CloseToTray"), m_bCloseToTray);
	ini.WriteBool(_T("DetailsOnClick"), m_bDetailsOnClick);
	ini.WriteInt(_T("SrvConTimeout"), m_nSrvConTimeout);
	ini.WriteBool(_T("SlowComplete"), m_bSlowComplete);
	ini.WriteInt(_T("SlowCompleteBlockSize"), m_nSlowCompleteBlockSize);
	ini.WriteBool(_T("ShowRatesOnTitle"),prefs->showRatesInTitle);

	// Barry - New properties...
    ini.WriteBool(_T("AutoConnectStaticOnly"), m_bAutoConnectStaticOnly);  
    ini.WriteBool(_T("GlobalSearch"), prefs->globalsearch);  
    ini.WriteInt (_T("3DDepth"), m_nDepth3D); 

	// Added by enkeyDEV
	ini.WriteBool(_T("NotifyOnDownload"), m_bUseDownloadNotifier);
	ini.WriteBool(_T("NotifyOnDownloadAdd"), m_bUseDownloadAddNotifier);
	ini.WriteBool(_T("NotifyOnChat"), m_bUseChatNotifier);		  
	ini.WriteBool(_T("NotifyOnLog"), m_bUseLogNotifier);
	ini.WriteBool(_T("NotifierUseSound"), m_bUseSoundInNotifier);
	ini.WriteBool(_T("NotifierPopEveryChatMessage"), m_bNotifierPopsEveryChatMsg);
	ini.WriteBool(_T("NotifyOnImportantError"), m_bNotifierImportantError);
	ini.WriteString(_T("NotifierSoundPath"), CString(m_sNotifierSoundFilePath.Get()));

	ini.WriteString(_T("MessageFilter"),prefs->messageFilter);
	ini.WriteString(_T("CommentFilter"), m_sCommentFilter.Get());	//(SyruS) netwolf commentfilter
	ini.WriteString(_T("TxtEditor"),prefs->TxtEditor);
	ini.WriteBool(_T("DisableRatingIcons"), m_bDisableRatingIcons);	//netwolf: disable icons
    ini.WriteString(_T("IRCServer"), m_sIRCServer.Get());
	ini.WriteString(_T("IRCNick"), m_sIRCNick.Get());
	ini.WriteBool(_T("IRCAddTimestamp"), m_bIRCAddTimestamp);
	ini.WriteString(_T("IRCFilterName"), m_sIRCChanNameFilter.Get());
	ini.WriteInt(_T("IRCFilterUser"), m_nIRCChannelUserFilter);
	ini.WriteBool(_T("IRCUseFilter"), m_bIRCUseChanFilter);
	ini.WriteString(_T("IRCPerformString"), m_sIRCPerformString.Get());
	ini.WriteBool(_T("IRCUsePerform"), m_bIRCUsePerform);
	ini.WriteBool(_T("IRCListOnConnect"), m_bIRCListOnConnect);
	//	ini.WriteBool("IRCAcceptLinks", m_bIRCAcceptLinks);
	ini.WriteBool(_T("IRCIgnoreInfoMessage"), m_bIRCIgnoreInfoMessage);
	ini.WriteBool(_T("IRCIgnoreEmuleProtoInfoMessage"), m_bIRCIgnoreEmuleProtoInfoMessage);
	ini.WriteBool(_T("SmartIdCheck"), m_bSmartIdCheck);

	ini.WriteBool(_T("UseSortingDefaults"), m_bUseSort);
	ini.WriteBool(_T("Use2ndSortSources"), m_bUseSrcSortCol2);
	ini.WriteInt(_T("SortServersCode"), m_nServerSortCol);	
	ini.WriteInt(_T("SortUploadsCode"), m_nUploadSortCol);
	ini.WriteInt(_T("SortQueueCode"), m_nQueueSortCol);
	ini.WriteInt(_T("SortSearchCode"), m_nSearchSortCol);
	ini.WriteInt(_T("SortIrcCode"), m_nIRCSortCol);
	ini.WriteInt(_T("SortFilesCode"), m_nFileSortCol);
	ini.WriteInt(_T("SortDownloadsCode"), m_nDownloadSortCol);
	ini.WriteInt(_T("1stSortSourcesCode"), m_nSrcSortCol1);
	ini.WriteInt(_T("2ndSortSourcesCode"), m_nSrcSortCol2);

	// DonGato
	ini.WriteBool(_T("CommunityEnabled"), m_bCommunityEnabled);
	ini.WriteString(_T("CommunityString"), m_sCommunityString.Get());
	ini.WriteBool(_T("CommunityNoBan"), m_bCommunityNoBan);
	ini.WriteInt(_T("NotificationDisplayTime"), m_nNotificationDisplayTime);
	ini.WriteInt(_T("NotificationFontSize"), m_nNotificationFontSize);
	ini.WriteInt(_T("Profile"), m_nProfile);
	ini.WriteInt(_T("PriorityHigh"), m_nPriorityHigh);
	ini.WriteInt(_T("PriorityLow"), m_nPriorityLow);
	ini.WriteBool(_T("RestartWaiting"), m_bRestartWaiting);
	ini.WriteBool(_T("LogToFile"), m_bLogToFile);
	ini.WriteBool(_T("BugReport"), prefs->BugReport);
	ini.WriteBool(_T("DisableXSUpTo"), m_bDisableXSUpTo);
	ini.WriteInt(_T("XSUpTo"), m_nXSUpTo);
	ini.WriteString(_T("UsedFont"), m_sUsedFont.Get());
	ini.WriteInt(_T("FontSize"), m_nFontSize);
	ini.WriteInt(_T("HashingPriority"), m_HashingPriority);
	ini.WriteBool(_T("UploadParts"), m_bUploadParts);
	ini.WriteBool(_T("A4AFStringEnabled"), m_bA4AFStringEnabled);
	ini.WriteInt(_T("SearchMethod"), m_nSearchMethod);
	ini.WriteBool(_T("AutoTakeED2KLinks"), m_bAutoTakeLinks); 
	ini.WriteInt(_T("FileBufferSizePref"), m_nFileBufferSize);
	ini.WriteInt(_T("MainProcessPriority"), m_nMainProcessPriority);
	ini.WriteBool(_T("ShowAverageDatarate"),m_bShowAverageDatarate);
	ini.WriteBool(_T("LimitlessDownload"),m_bLimitlessDownload);
	ini.WriteBool(_T("SCHEnabled"), m_bSCHEnabled);
	ini.WriteBool(_T("SCHExceptMon"), m_bSCHExceptMon);
	ini.WriteBool(_T("SCHExceptTue"), m_bSCHExceptTue);
	ini.WriteBool(_T("SCHExceptWed"), m_bSCHExceptWed);
	ini.WriteBool(_T("SCHExceptThu"), m_bSCHExceptThu);
	ini.WriteBool(_T("SCHExceptFri"), m_bSCHExceptFri);
	ini.WriteBool(_T("SCHExceptSat"), m_bSCHExceptSat);
	ini.WriteBool(_T("SCHExceptSun"), m_bSCHExceptSun);
	ini.WriteInt(_T("SCHShift1"),m_nSCHShift1);
	ini.WriteInt(_T("SCHShift2"),m_nSCHShift2);
	ini.WriteInt(_T("SCHShift1Up"),m_nSCHShift1Upload);
	ini.WriteInt(_T("SCHShift1Down"),m_nSCHShift1Download);
	ini.WriteInt(_T("SCHShift1Conn"),m_nSCHShift1conn);
	ini.WriteInt(_T("SCHShift15sec"),m_nSCHShift15sec);
	ini.WriteInt(_T("SCHShift2Up"),m_nSCHShift2Upload);
	ini.WriteInt(_T("SCHShift2Down"),m_nSCHShift2Download);
	ini.WriteInt(_T("SCHShift2Conn"),m_nSCHShift2conn);
	ini.WriteInt(_T("SCHShift25sec"),m_nSCHShift25sec);

	// EpicStruggle
	ini.WriteBool(_T("MultipleInstances"), m_bMultiple);

	ini.WriteBool(_T("Verbose"), m_bVerbose);
	ini.WriteBool(_T("UpdateQueueList"), m_bUpdateQueueList);
	ini.WriteBool(_T("ManualHighPrio"), m_bManualHighPrio);
	//ini.WriteBool("StartNextFile", prefs->m_bstartnextfile); //duplicates downloadpausedoncomplete
	ini.WriteBool(_T("ShowOverhead"), m_bShowOverhead);	//Cax2 re-enabled this preference
	ini.WriteBool(_T("VideoPreviewBackupped"), m_bBackupPreview);	//Cax2 string kept for compatibility with off client

	ini.SerGet(false, prefs->downloadColumnWidths,
		ELEMENT_COUNT(prefs->downloadColumnWidths), _T("DownloadColumnWidths"));
	ini.SerGet(false, prefs->downloadColumnHidden,
		ELEMENT_COUNT(prefs->downloadColumnHidden), _T("DownloadColumnHidden"));
	ini.SerGet(false, prefs->downloadColumnOrder,
		ELEMENT_COUNT(prefs->downloadColumnOrder), _T("DownloadColumnOrder"));
	ini.SerGet(false, prefs->uploadColumnWidths,
		ELEMENT_COUNT(prefs->uploadColumnWidths), _T("UploadColumnWidths"));
	ini.SerGet(false, prefs->uploadColumnHidden,
		ELEMENT_COUNT(prefs->uploadColumnHidden), _T("UploadColumnHidden"));
	ini.SerGet(false, prefs->uploadColumnOrder,
		ELEMENT_COUNT(prefs->uploadColumnOrder), _T("UploadColumnOrder"));
	ini.SerGet(false, prefs->queueColumnWidths,
		ELEMENT_COUNT(prefs->queueColumnWidths), _T("QueueColumnWidths"));
	ini.SerGet(false, prefs->queueColumnHidden,
		ELEMENT_COUNT(prefs->queueColumnHidden), _T("QueueColumnHidden"));
	ini.SerGet(false, prefs->queueColumnOrder,
		ELEMENT_COUNT(prefs->queueColumnOrder), _T("QueueColumnOrder"));
	ini.SerGet(false, prefs->searchColumnWidths,
		ELEMENT_COUNT(prefs->searchColumnWidths), _T("SearchColumnWidths"));
	ini.SerGet(false, prefs->searchColumnHidden,
		ELEMENT_COUNT(prefs->searchColumnHidden), _T("SearchColumnHidden"));
	ini.SerGet(false, prefs->searchColumnOrder,
		ELEMENT_COUNT(prefs->searchColumnOrder), _T("SearchColumnOrder"));
	ini.SerGet(false, prefs->sharedColumnWidths,
		ELEMENT_COUNT(prefs->sharedColumnWidths), _T("SharedColumnWidths"));
	ini.SerGet(false, prefs->sharedColumnHidden,
		ELEMENT_COUNT(prefs->sharedColumnHidden), _T("SharedColumnHidden"));
	ini.SerGet(false, prefs->sharedColumnOrder,
		ELEMENT_COUNT(prefs->sharedColumnOrder), _T("SharedColumnOrder"));
	ini.SerGet(false, prefs->serverColumnWidths,
		ELEMENT_COUNT(prefs->serverColumnWidths), _T("ServerColumnWidths"));
	ini.SerGet(false, prefs->serverColumnHidden,
		ELEMENT_COUNT(prefs->serverColumnHidden), _T("ServerColumnHidden"));
	ini.SerGet(false, prefs->serverColumnOrder,
		ELEMENT_COUNT(prefs->serverColumnOrder), _T("ServerColumnOrder"));
	ini.SerGet(false, prefs->ircColumnWidths,
		ELEMENT_COUNT(prefs->ircColumnWidths), _T("IrcColumnWidths"));
	ini.SerGet(false, prefs->ircColumnHidden,
		ELEMENT_COUNT(prefs->ircColumnHidden), _T("IrcColumnHidden"));
	ini.SerGet(false, prefs->ircColumnOrder,
		ELEMENT_COUNT(prefs->ircColumnOrder), _T("IrcColumnOrder"));

	// Barry - Provide a mechanism for all tables to store/retrieve sort order
	ini.WriteInt(_T("TableSortItemDownload"), prefs->tableSortItemDownload);
	ini.WriteInt(_T("TableSortItemUpload"), prefs->tableSortItemUpload);
	ini.WriteInt(_T("TableSortItemQueue"), prefs->tableSortItemQueue);
	ini.WriteInt(_T("TableSortItemSearch"), prefs->tableSortItemSearch);
	ini.WriteInt(_T("TableSortItemShared"), prefs->tableSortItemShared);
	ini.WriteInt(_T("TableSortItemServer"), prefs->tableSortItemServer);
	ini.WriteBool(_T("TableSortAscendingDownload"), prefs->tableSortAscendingDownload);
	ini.WriteBool(_T("TableSortAscendingUpload"), prefs->tableSortAscendingUpload);
	ini.WriteBool(_T("TableSortAscendingQueue"), prefs->tableSortAscendingQueue);
	ini.WriteBool(_T("TableSortAscendingSearch"), prefs->tableSortAscendingSearch);
	ini.WriteBool(_T("TableSortAscendingShared"), prefs->tableSortAscendingShared);
	ini.WriteBool(_T("TableSortAscendingServer"), prefs->tableSortAscendingServer);

	CString buffer2;
	for (int i=0;i<15;i++) {
		buffer.Format(_T("%I64Lu"),GetStatsColor(i));
		buffer2.Format(_T("StatColor%i"),i);
		ini.WriteString(buffer2,buffer);
	}
	ini.WriteInt(_T("ConnectionsGraphRatio"), m_nGraphRatio); // propstats Cax2

	// Added By Maverick
	ini.WriteBool(_T("MlDonkeyCM"), m_bMlDonkeyCM);
	ini.WriteBool(_T("CMNotLog"), m_bCMNotLog);	// netwolf

	// Added by Tarod -saving HASH of A4AF auto file
	CString hex_hash = FileHashToString(prefs->A4AF_file_hash);
	
	ini.WriteString(_T("A4AFAutoFileHash"), hex_hash);	

	// Maverick
	ini.WriteBool(_T("EnablePassword"),prefs->proxy.EnablePassword,_T("Proxy"));
	ini.WriteBool(_T("EnableProxy"),prefs->proxy.UseProxy,_T("Proxy"));
	ini.WriteString(_T("Name"),prefs->proxy.name,_T("Proxy"));
	ini.WriteString(_T("Password"),prefs->proxy.password,_T("Proxy"));
	ini.WriteString(_T("User"),prefs->proxy.user,_T("Proxy"));
	ini.WriteInt(_T("Port"),prefs->proxy.port,_T("Proxy"));
	ini.WriteInt(_T("Type"),prefs->proxy.type,_T("Proxy"));
	ini.WriteBool(_T("ConnectWithoutProxy"),m_bIsASCWOP,_T("Proxy"));

	buffer.Format(_T("%I64Lu"), m_nTotalDownloadedBytes.Get());
	ini.WriteString(_T("TotalDownloadedBytes"),buffer ,_T("Statistics"));

	buffer.Format(_T("%I64Lu"), m_nTotalUploadedBytes.Get());
	ini.WriteString(_T("TotalUploadedBytes"),buffer ,_T("Statistics"));

	// Web Server
	ini.WriteString(_T("Password"), CString(m_sWebPassword.Get()), _T("WebServer"));	
	ini.WriteString(_T("PasswordLow"), CString(m_sLowWebPassword.Get()), _T("WebServer"));	
	ini.WriteInt(_T("Port"), m_nWebPort, _T("WebServer"));
	ini.WriteBool(_T("Enabled"), m_bWebEnabled, _T("WebServer"));
	ini.WriteBool(_T("EnabledLow"), m_bWebLowEnabled, _T("WebServer"));
	ini.WriteBool(_T("UseGzip"), m_bWebUseGzip, _T("WebServer"));
	ini.WriteInt(_T("PageRefreshTime"), m_nWebPageRefresh, _T("WebServer"));
	// Added by Johnny-B
	ini.WriteBool(_T("IntruderDetection"), m_bWebIntruderDetection, _T("WebServer"));
	ini.WriteInt(_T("TempDisableLogin"), m_nWebTempDisableLogin, _T("WebServer"));
	ini.WriteInt(_T("LoginAttemptsAllowed"), m_nWebLoginAttemptsAllowed, _T("WebServer"));
	//End Johnny-B
	ini.WriteString(_T("WebTemplateFile"),prefs->m_sTemplateFile,_T("WebServer"));	// netwolf

	//////////////////////////////////////////////////////////////////
	// LANCAST (moosetea) - Save the settings
	
	CString t_strLanCastIP;
	t_strLanCastIP.Format(_T("%i.%i.%i.%i"),(uint8)m_LancastIP,(uint8)(m_LancastIP>>8),(uint8)(m_LancastIP>>16),(uint8)(m_LancastIP>>24));

	CString t_strLanCastSubnet;
	t_strLanCastSubnet.Format(_T("%i.%i.%i.%i"),(uint8)m_LancastSubnet,(uint8)(m_LancastSubnet>>8),(uint8)(m_LancastSubnet>>16),(uint8)(m_LancastSubnet>>24));
				
	ini.WriteBool(_T("Enabled"),m_LancastEnabled,_T("Lancast"));
	ini.WriteBool(_T("UseDefaultIP"),m_UseDefaultIP,_T("Lancast"));
	ini.WriteString(_T("IPAddress"),t_strLanCastIP,_T("Lancast"));
	ini.WriteString(_T("Subnet"),t_strLanCastSubnet,_T("Lancast"));
	ini.WriteInt(_T("Port"),m_LancastPort,_T("Lancast"));

	//////////////////////////////////////////////////////////////////

	//START enkeyDEV(kei-kun) -Messaging preferences-
	ini.WriteInt(_T("AcceptMessageFrom"),m_nAcceptMessagesFrom);	
	ini.WriteString(_T("AwayStateMessage"), CString(m_sAwayStateMessage.Get()));
	//END enkeyDEV(kei-kun) -Messaging preferences-

	//SyruS: cleanup
	ini.WriteBool(_T("AutoFilenameCleanup"), m_bAutoFilenameCleanup, _T("Cleanup"));
	ini.WriteBool(_T("CleanupTags"), m_bFilenameCleanupTags, _T("Cleanup"));
	ini.WriteString(_T("FilenameCleanups"), CString(m_sFilenameCleanups.Get()), _T("Cleanup"));

	EMULE_CATCH
}

DWORD CPreferences::DefaultStatsColor(int index){
	switch(index)
	{
		case 0 : return RGB(0,0,64);
		case 1 : return RGB(192,192,255);
		case 2 : return RGB(128, 255, 128);
		case 3 : return RGB(0, 210, 0);
		case 4 : return RGB(0, 128, 0);
		case 5 : return RGB(255, 128, 128);
		case 6 : return RGB(200, 0, 0);
		case 7 : return RGB(140, 0, 0);
		case 8 : return RGB(150, 150, 255);
		case 9 : return RGB(192,   0, 192);
		case 10 : return RGB(255, 192, 255);
		case 11 : return RGB(255, 255, 128);
		default : return RGB(255, 255, 255);
	}
}

void CPreferences::LoadPreferences(){
//	CSingleLock PrefLock(&m_Pref_Mutex,TRUE);
	
	//--- Quick hack to add version tag to preferences.ini-file and solve the issue with the FlatStatusBar tag...
	CString strFileName;
	strFileName.Format(_T("%spreferences.ini"), appdir);
	CIni* pIni = new CIni(strFileName, _T("eMule"));
	CString strCurrVersion, strPrefsVersion;
	strCurrVersion = CURRENT_VERSION_LONG;
	strPrefsVersion = pIni->GetString(_T("AppVersion"));
	delete pIni;
	CFileFind findFileName;

	bool bChangedVersion = false;

	if ((strCurrVersion != strPrefsVersion) && findFileName.FindFile(strFileName))
	{
		CFile file;
		CFileFind findNewName;
		CString strNewName;
		//Cax2 we can add the code to find our version's setting, if the ini file is from a different version...
		strNewName.Format(_T("%spreferences[%s].ini"), appdir,strPrefsVersion);	
		if (findNewName.FindFile(strNewName))
			file.Remove(strNewName);
		file.Rename(strFileName, strNewName);
		strFileName = strNewName;

		bChangedVersion = true;
	}

	CIni ini(strFileName, _T("eMule"));
	//--- end Ozon :)

	CString strDefNick;	// random nick [FoRcHa]
	srand((unsigned)time(NULL));

	CString buffer;
	for(int i = 0; i < 6; i++)
		buffer += char(rand() % ('z' - 'a') + 'a');
	
	strDefNick.Format(_T("%s [eMule Plus v%s]"), buffer, CURRENT_VERSION_LONG);
	m_sNick.Put(ini.GetString(_T("Nick"),strDefNick).GetBuffer(0));
	
	buffer.Format(_T("%sIncoming"),appdir);
	m_sIncomingDir.Put(ini.GetString(_T("IncomingDir"),buffer).GetBuffer(0));

	buffer.Format(_T("%sTemp"),appdir);
	m_sTempDir.Put(ini.GetString(_T("TempDir"),buffer).GetBuffer(0));

	buffer.Format(_T("%sVLC"),appdir);
	m_sVlcDir.Put(ini.GetString(_T("VLCDir"),buffer).GetBuffer(0));
	buffer.Format(_T("%s\\vlc.exe"),GetVlcDir());		//Cax2 - for backward compatibility
	m_sVideoPlayer.Put(ini.GetString(_T("VideoPlayer"),buffer).GetBuffer(0));		//Cax2 - replace buffer with "" in later (non b-compatible) versions
	m_nMaxGraphUploadRate = ini.GetInt(_T("UploadCapacity"),16);
	m_nMaxGraphDownloadRate = ini.GetInt(_T("DownloadCapacity"),96);
	m_nMaxUpload = ini.GetInt(_T("MaxUpload"),UNLIMITED);		//Cax2 new limits enforced on old settings
	if (m_nMaxUpload>m_nMaxGraphUploadRate)
		m_nMaxUpload = m_nMaxGraphUploadRate;
	m_nMaxDownload = ini.GetInt(_T("MaxDownload"),UNLIMITED);
	if (m_nMaxDownload>m_nMaxGraphDownloadRate)
		m_nMaxDownload = m_nMaxGraphDownloadRate;
	m_nMaxConnections = ini.GetInt(_T("MaxConnections"),GetRecommendedMaxConnections());
	m_nDeadServer = ini.GetInt(_T("DeadServerAction"),2);
	m_nPort = ini.GetInt(_T("Port"),4662);
	m_nUDPPort = ini.GetInt(_T("UDPPort"),m_nPort + 10);
	m_nMaxSourcePerFile = ini.GetInt(_T("MaxSourcesPerFile"),400);
	m_nQueueSize = ini.GetInt(_T("QueueSize"), MIN_QUEUE*3);
	if(m_nQueueSize > MAX_QUEUE)
		m_nQueueSize = MAX_QUEUE;
	else if(m_nQueueSize < MIN_QUEUE)
		m_nQueueSize = MIN_QUEUE;
	m_nRQlimit = (m_nQueueSize >= 2000)? m_nQueueSize : 2000; //Cax2 - remote queue limit linked to queue size, but no lower than 2000
	m_bUseRQlimit = ini.GetBool(_T("UseRemoteQueueLimit"));
	m_nLanguageID = ini.GetWORD(_T("Language"),0);
	m_nSeeShares.Put((enSeeShare)ini.GetInt(_T("SeeShare"),SEE_SHARE_EVERYBODY));
	//--- xrmb:partTrafficPrefs ---
	m_bUsePartTraffic = ini.GetBool(_T("PartTrafficEnabled"),false);
	m_nUpbarPartTraffic = ini.GetInt(_T("UploadbarStyle"), 2);
	m_nUpbarColorPartTraffic = ini.GetInt(_T("UploadbarColor"), 0);
	//--- :xrmb ---
	m_nTooltipDelayTime = ini.GetInt(_T("ToolTipDelay"),1);
	m_nTrafficOMeterInterval = ini.GetInt(_T("StatGraphsInterval"),3);
	m_nStatsInterval = ini.GetInt(_T("statsInterval"),5);
	m_nStatsColumnWidth = ini.GetInt(_T("StatsColumnWidth"), 150);
	m_nDetailColumnWidth = ini.GetInt(_T("DetailColumnWidth"), 150);
	
	m_sStatsListExpanded.Put(ini.GetString(_T("StatsListExpanded"), _T("")).GetBuffer(0)); // missing entrys get expanded ("" = all expanded)

	m_nDeadServerRetries = ini.GetInt(_T("DeadServerRetries"),1);
	m_nSplitterBarPosition = ini.GetInt(_T("SplitterbarPosition"),75);
	m_nStatsMax = ini.GetInt(_T("VariousStatisticsMaxValue"),100);
	m_nStatsAverageMinutes = ini.GetInt(_T("StatsAverageMinutes"),5);
	m_nMaxConperFive = ini.GetInt(_T("MaxConnectionsPerFiveSeconds"),GetDefaultMaxConperFive());
	prefs->showRatesInTitle=ini.GetBool(_T("ShowRatesOnTitle"),false);

	// DonGato
	int nMax = (LOBYTE(GetWindowsVersion()) == 0x04)?100:200;
	if(m_nMaxConperFive > nMax) {
		m_nMaxConperFive = nMax;
	} else {
		if(m_nMaxConperFive < 5)
			m_nMaxConperFive = 5;
	}

	m_nBadClientBanTime = ini.GetInt(_T("BadClientBanTime"),3600000);
	if(m_nBadClientBanTime > 18000000)
		m_nBadClientBanTime = 18000000;
	
	m_nMinRequestTime = ini.GetInt(_T("BadClientMinRequestTime"),590000);
	if(m_nMinRequestTime > 590000)
		m_nMinRequestTime = 590000;
	
	m_nBadClientBan = ini.GetInt(_T("BadClientMinRequestNum"),3);
	if(m_nBadClientBan < 3)
		m_nBadClientBan = 3;
	m_bBanMessageEnabled = ini.GetBool(_T("BanMessageEnabled"), false);
	m_bBanEnabled = ini.GetBool(_T("BanEnabled"),true); // DbT:BanEnDisable

	// Original by reCDVst updated by DonGato
	m_nSLSRestoreSourcesTime = ini.GetInt(_T("SLS_RestoreSourcesTime"),7200000);
	if(m_nSLSRestoreSourcesTime < 3600000)
		m_nSLSRestoreSourcesTime = 3600000;

	m_nSLSOutdated = ini.GetInt(_T("SLS_WhenSourcesOutdated"),2);
	if(m_nSLSOutdated > 10)
		m_nSLSOutdated = 10;
	
	m_SLSMaxSourcesPerFile = ini.GetInt(_T("SLS_MaxSourcesPerFile"),10);
	if(m_SLSMaxSourcesPerFile > 40)
		m_SLSMaxSourcesPerFile = 40;
	else if(m_SLSMaxSourcesPerFile < 10)
		m_SLSMaxSourcesPerFile = 10;

	m_bSLSEnable = ini.GetBool(_T("SLS_Enable"),false);

	m_bReconnect = ini.GetBool(_T("Reconnect"),true);
	m_nScoreSystem = ini.GetBool(_T("Scoresystem"),true);
	
	m_bUAP = ini.GetBool(_T("UAP"), true);
	m_bDAP = ini.GetBool(_T("DAP"), true);
	m_bDisableXS = ini.GetBool(_T("DisableXS"), false);
	m_bStartDownloadPaused = ini.GetBool(_T("StartDownloadPaused"), false);
	m_bDownloadPausedOnComplete = ini.GetBool(_T("DownloadPausedOnComplete"), false);
	
	m_sToolbarSettings.Put(ini.GetString(_T("ToolbarSettings"), strDefaultToolbar).GetBuffer(0));
	m_sToolbarBitmap.Put(ini.GetString(_T("ToolbarBitmap"), _T("")).GetBuffer(0)); // Added by reCDVst
	m_sToolbarBitmapFolder.Put(ini.GetString(_T("ToolbarBitmapFolder"), _T("")).GetBuffer(0)); // Added by reCDVst
	m_nToolbarLabels = ini.GetInt(_T("ToolbarLabels"),1);
	m_bAutoServerList = ini.GetBool(_T("Serverlist"),false);
	m_bUpdateNotify = ini.GetBool(_T("UpdateNotify"),true);
	m_bMinToTray = ini.GetBool(_T("MinimizeToTray"),true);
	m_bAddServersFromConn = ini.GetBool(_T("AddServersFromConnections"),true);
	m_bSplashScreen = ini.GetBool(_T("Splashscreen"),true);
	m_bBringToForeground = ini.GetBool(_T("BringToFront"),true);
	m_nTransferDoubleclick = ini.GetBool(_T("TransferDoubleClick"),true);
	m_bBeepOnError = ini.GetBool(_T("BeepOnError"),true);
	m_bConfirmExit = ini.GetBool(_T("ConfirmExit"),true);
	m_bFilterBadIP = ini.GetBool(_T("FilterBadIPs"),true);
	m_bFilterServersByIP = ini.GetBool(_T("FilterServersByIP"),false);
	m_bAutoConnect = ini.GetBool(_T("Autoconnect"),false);
	m_bOnlineSig = ini.GetBool(_T("OnlineSignature"),false);
	m_bStartMinimized = ini.GetBool(_T("StartupMinimized"),false);
	m_bSafeConnect = ini.GetBool(_T("SafeConnect"),true);
	m_bShowToolbarSpeedMeter = ini.GetBool(_T("ToolbarSpeedMeter"), false);
	m_bCloseToTray = ini.GetBool(_T("CloseToTray"),false);	
	m_bDetailsOnClick = ini.GetBool(_T("DetailsOnClick"),false);	

	m_nSrvConTimeout = ini.GetInt(_T("SrvConTimeout"),20000);		//Cax2 - bugfix for time out - ticks are in milliseconds!!!!!
	if(m_nSrvConTimeout < 10000)
		m_nSrvConTimeout = 10000;
	else if(m_nSrvConTimeout > 60000)
		m_nSrvConTimeout = 60000;
	
	m_bSlowComplete = ini.GetBool(_T("SlowComplete"),false);	
	m_nSlowCompleteBlockSize = ini.GetInt(_T("SlowCompleteBlockSize"),32);
	if(m_nSlowCompleteBlockSize > 1024)
		m_nSlowCompleteBlockSize = 1024;

	// Barry - New properties...
	m_bAutoConnectStaticOnly = ini.GetBool(_T("AutoConnectStaticOnly"),false); 
	prefs->globalsearch = ini.GetBool(_T("GlobalSearch"),true); 
	m_nDepth3D = ini.GetInt(_T("3DDepth"), 0);

	// as a temporary converter for previous versions
	if (strPrefsVersion < _T("0.25a")) // before 0.25a
		if (ini.GetBool(_T("FlatStatusBar"),false))
			m_nDepth3D = 0;
		else 
			m_nDepth3D = 5;

	// Added by enkeyDEV
	m_bUseDownloadAddNotifier = ini.GetBool(_T("NotifyOnDownloadAdd"),false);
	m_bUseDownloadNotifier = ini.GetBool(_T("NotifyOnDownload"),false);
	m_bUseChatNotifier = ini.GetBool(_T("NotifyOnChat"),false);
	m_bUseLogNotifier = ini.GetBool(_T("NotifyOnLog"),false);
	m_bUseSoundInNotifier = ini.GetBool(_T("NotifierUseSound"),false);
	m_bNotifierPopsEveryChatMsg = ini.GetBool(_T("NotifierPopEveryChatMessage"),false);
	m_bNotifierImportantError = ini.GetBool(_T("NotifyOnImportantError"),false);
	m_sNotifierSoundFilePath.Put(ini.GetString(_T("NotifierSoundPath"),_T("")).GetBuffer(0));

	m_sIRCServer.Put(ini.GetString(_T("IRCServer"),_T("irc.liquidirc.com")).GetBuffer(0));
	m_sIRCNick.Put(ini.GetString(_T("IRCNick"),_T("eMule Plus")).GetBuffer(0));
	m_bIRCAddTimestamp = ini.GetBool(_T("IRCAddTimestamp"),true);
	m_sIRCChanNameFilter.Put(ini.GetString(_T("IRCFilterName"), _T("")).GetBuffer(0));
	m_bIRCUseChanFilter = ini.GetBool(_T("IRCUseFilter"), false);
	m_nIRCChannelUserFilter = ini.GetInt(_T("IRCFilterUser"), 0);
	m_sIRCPerformString.Put(ini.GetString(_T("IRCPerformString"), _T("/join #emule")).GetBuffer(0));
	m_bIRCUsePerform = ini.GetBool(_T("IRCUsePerform"), false);
	m_bIRCListOnConnect = ini.GetBool(_T("IRCListOnConnect"), true);
	//	m_bIRCAcceptLinks = ini.GetBool("IRCAcceptLinks", false);
	m_bIRCIgnoreInfoMessage = ini.GetBool(_T("IRCIgnoreInfoMessage"), false);
	m_bIRCIgnoreEmuleProtoInfoMessage = ini.GetBool(_T("IRCIgnoreEmuleProtoInfoMessage"), true);
	m_bSmartIdCheck = ini.GetBool(_T("SmartIdCheck"),true);
	m_bVerbose = ini.GetBool(_T("Verbose"),false);
	m_bUpdateQueueList = ini.GetBool(_T("UpdateQueueList"),false);
	m_bManualHighPrio=ini.GetBool(_T("ManualHighPrio"),false);
	m_bShowOverhead = ini.GetBool(_T("ShowOverhead"),true);
	m_bBackupPreview = ini.GetBool(_T("VideoPreviewBackupped"),false);	//Cax2 string kept for compatibility with off client

	prefs->TxtEditor = ini.GetString(_T("TxtEditor"),_T("notepad.exe"));
	prefs->messageFilter=ini.GetString(_T("MessageFilter"),_T("Your client has an infinite queue|http://www.chez.com/theworld/"));
	m_sCommentFilter.Put(ini.GetString(_T("CommentFilter"),_T("http://www.|emulespeedup")).GetBuffer(0));	//(SyruS) netwolf commentfilter
	m_bDisableRatingIcons=ini.GetBool(_T("DisableRatingIcons"),false);	//netwolf: disable icons

	m_bUseSort = ini.GetBool(_T("UseSortingDefaults"),true);
	m_bUseSrcSortCol2 = ini.GetBool(_T("Use2ndSortSources"),false);
	m_nServerSortCol = ini.GetInt(_T("SortServersCode"),0);
	m_nUploadSortCol = ini.GetInt(_T("SortUploadsCode"),12);
	m_nQueueSortCol = ini.GetInt(_T("SortQueueCode"),105);
	m_nSearchSortCol = ini.GetInt(_T("SortSearchCode"),12);
	m_nIRCSortCol = ini.GetInt(_T("SortIrcCode"),11);
	m_nFileSortCol = ini.GetInt(_T("SortFilesCode"),0);
	m_nDownloadSortCol = ini.GetInt(_T("SortDownloadsCode"),8);
	m_nSrcSortCol1 = ini.GetInt(_T("1stSortSourcesCode"),0);
	m_nSrcSortCol2 = ini.GetInt(_T("2ndSortSourcesCode"),255);

	// DonGato
	m_bCommunityEnabled = ini.GetBool(_T("CommunityEnabled"), false);
	m_sCommunityString.Put(ini.GetString(_T("CommunityString"),_T("")).GetBuffer(0));
	m_bCommunityNoBan = ini.GetBool(_T("CommunityNoBan"), false);
	m_nNotificationDisplayTime = ini.GetInt(_T("NotificationDisplayTime"),4000);
	m_nNotificationFontSize = ini.GetInt(_T("NotificationFontSize"),70);
	m_nProfile = ini.GetInt(_T("Profile"),0);
	m_nPriorityHigh = ini.GetInt(_T("PriorityHigh"),100);
	m_nPriorityLow = ini.GetInt(_T("PriorityLow"),300);
	if(m_nPriorityLow != 0){
		if(m_nPriorityLow < m_nPriorityHigh)
			m_nPriorityLow = m_nPriorityHigh * 2;
	} else {
		m_nPriorityLow = 300;
		m_nPriorityHigh = 100;
	}
	m_bRestartWaiting = ini.GetBool(_T("RestartWaiting"),true);
	m_bLogToFile = ini.GetBool(_T("LogToFile"),false);
	prefs->BugReport = ini.GetBool(_T("BugReport"),false);
	m_bDisableXSUpTo = ini.GetBool(_T("DisableXSUpTo"),false);
	m_nXSUpTo = ini.GetInt(_T("XSUpTo"),GetMaxSourcePerFileSoft());
	if(m_nXSUpTo > GetMaxSourcePerFileSoft())
		m_nXSUpTo = GetMaxSourcePerFileSoft();
	m_sUsedFont.Put(ini.GetString(_T("UsedFont"), _T("MS Sans Serif")).GetBuffer(0));
	m_nFontSize = ini.GetInt(_T("FontSize"),80);
	m_HashingPriority.Put((enHashPriority)ini.GetInt(_T("HashingPriority"),2));
	m_bUploadParts = ini.GetBool(_T("UploadParts"),false);
	m_bA4AFStringEnabled = ini.GetBool(_T("A4AFStringEnabled"), false);
	m_bBanMessageEnabled = ini.GetBool(_T("BanMessageEnabled"), false);
	m_nSearchMethod = ini.GetInt(_T("SearchMethod"),0);
	m_bAutoTakeLinks = ini.GetBool(_T("AutoTakeED2KLinks"),false);
	m_nFileBufferSize = ini.GetInt(_T("FileBufferSizePref"),16);
	m_nMainProcessPriority = ini.GetInt(_T("MainProcessPriority"),0);	// netwolf 05.05.03 (default: normal MainProcessPriority)
	m_bShowAverageDatarate = ini.GetBool(_T("ShowAverageDatarate"),true);
	m_bLimitlessDownload = ini.GetBool(_T("LimitlessDownload"),false);

	m_bSCHEnabled = ini.GetBool(_T("SCHEnabled"), false);
	m_bSCHExceptMon = ini.GetBool(_T("SCHExceptMon"), false);
	m_bSCHExceptTue = ini.GetBool(_T("SCHExceptTue"), false);
	m_bSCHExceptWed = ini.GetBool(_T("SCHExceptWed"), false);
	m_bSCHExceptThu = ini.GetBool(_T("SCHExceptThu"), false);
	m_bSCHExceptFri = ini.GetBool(_T("SCHExceptFri"), false);
	m_bSCHExceptSat = ini.GetBool(_T("SCHExceptSat"), true);
	m_bSCHExceptSun = ini.GetBool(_T("SCHExceptSun"), true);
	m_nSCHShift1 = ini.GetInt(_T("SCHShift1"),64800);
	m_nSCHShift2 = ini.GetInt(_T("SCHShift2"),21600);
	m_nSCHShift1Upload = ini.GetInt(_T("SCHShift1Up"),UNLIMITED);
	m_nSCHShift1Download = ini.GetInt(_T("SCHShift1Down"),UNLIMITED);
	m_nSCHShift1conn = ini.GetInt(_T("SCHShift1conn"),512);
	m_nSCHShift15sec = ini.GetInt(_T("SCHShift15sec"),40);
	m_nSCHShift2Upload = ini.GetInt(_T("SCHShift2Up"),UNLIMITED);
	m_nSCHShift2Download = ini.GetInt(_T("SCHShift2Down"),UNLIMITED);
	m_nSCHShift2conn = ini.GetInt(_T("SCHShift2conn"),256);
	m_nSCHShift25sec = ini.GetInt(_T("SCHShift25sec"),20);

	m_bMultiple = ini.GetBool(_T("MultipleInstances"),false);

	ini.SerGet(true, prefs->downloadColumnWidths,
		ELEMENT_COUNT(prefs->downloadColumnWidths), _T("DownloadColumnWidths"));
	ini.SerGet(true, prefs->downloadColumnHidden,
		ELEMENT_COUNT(prefs->downloadColumnHidden), _T("DownloadColumnHidden"));
	ini.SerGet(true, prefs->downloadColumnOrder,
		ELEMENT_COUNT(prefs->downloadColumnOrder), _T("DownloadColumnOrder"));
	ini.SerGet(true, prefs->uploadColumnWidths,
		ELEMENT_COUNT(prefs->uploadColumnWidths), _T("UploadColumnWidths"));
	ini.SerGet(true, prefs->uploadColumnHidden,
		ELEMENT_COUNT(prefs->uploadColumnHidden), _T("UploadColumnHidden"));
	ini.SerGet(true, prefs->uploadColumnOrder,
		ELEMENT_COUNT(prefs->uploadColumnOrder), _T("UploadColumnOrder"));
	ini.SerGet(true, prefs->queueColumnWidths,
		ELEMENT_COUNT(prefs->queueColumnWidths), _T("QueueColumnWidths"));
	ini.SerGet(true, prefs->queueColumnHidden,
		ELEMENT_COUNT(prefs->queueColumnHidden), _T("QueueColumnHidden"));
	ini.SerGet(true, prefs->queueColumnOrder,
		ELEMENT_COUNT(prefs->queueColumnOrder), _T("QueueColumnOrder"));
	ini.SerGet(true, prefs->searchColumnWidths,
		ELEMENT_COUNT(prefs->searchColumnWidths), _T("SearchColumnWidths"));
	ini.SerGet(true, prefs->searchColumnHidden,
		ELEMENT_COUNT(prefs->searchColumnHidden), _T("SearchColumnHidden"));
	ini.SerGet(true, prefs->searchColumnOrder,
		ELEMENT_COUNT(prefs->searchColumnOrder), _T("SearchColumnOrder"));
	ini.SerGet(true, prefs->sharedColumnWidths,
		ELEMENT_COUNT(prefs->sharedColumnWidths), _T("SharedColumnWidths"));
	ini.SerGet(true, prefs->sharedColumnHidden,
		ELEMENT_COUNT(prefs->sharedColumnHidden), _T("SharedColumnHidden"));
	ini.SerGet(true, prefs->sharedColumnOrder,
		ELEMENT_COUNT(prefs->sharedColumnOrder), _T("SharedColumnOrder"));
	ini.SerGet(true, prefs->serverColumnWidths,
		ELEMENT_COUNT(prefs->serverColumnWidths), _T("ServerColumnWidths"));
	ini.SerGet(true, prefs->serverColumnHidden,
		ELEMENT_COUNT(prefs->serverColumnHidden), _T("ServerColumnHidden"));
	ini.SerGet(true, prefs->serverColumnOrder,
		ELEMENT_COUNT(prefs->serverColumnOrder), _T("ServerColumnOrder"));
	ini.SerGet(true, prefs->ircColumnWidths,
		ELEMENT_COUNT(prefs->ircColumnWidths), _T("IrcColumnWidths"));
	ini.SerGet(true, prefs->ircColumnHidden,
		ELEMENT_COUNT(prefs->ircColumnHidden), _T("IrcColumnHidden"));
	ini.SerGet(true, prefs->ircColumnOrder,
		ELEMENT_COUNT(prefs->ircColumnOrder), _T("IrcColumnOrder"));

	// Barry - Provide a mechanism for all tables to store/retrieve sort order
	prefs->tableSortItemDownload = ini.GetInt(_T("TableSortItemDownload"), 0);
	prefs->tableSortItemUpload = ini.GetInt(_T("TableSortItemUpload"), 0);
	prefs->tableSortItemQueue = ini.GetInt(_T("TableSortItemQueue"), 0);
	prefs->tableSortItemSearch = ini.GetInt(_T("TableSortItemSearch"), 0);
	prefs->tableSortItemShared = ini.GetInt(_T("TableSortItemShared"), 0);
	prefs->tableSortItemServer = ini.GetInt(_T("TableSortItemServer"), 0);
	prefs->tableSortAscendingDownload = ini.GetBool(_T("TableSortAscendingDownload"), true);
	prefs->tableSortAscendingUpload = ini.GetBool(_T("TableSortAscendingUpload"), true);
	prefs->tableSortAscendingQueue = ini.GetBool(_T("TableSortAscendingQueue"), true);
	prefs->tableSortAscendingSearch = ini.GetBool(_T("TableSortAscendingSearch"), true);
	prefs->tableSortAscendingShared = ini.GetBool(_T("TableSortAscendingShared"), true);
	prefs->tableSortAscendingServer = ini.GetBool(_T("TableSortAscendingServer"), true);

	if (m_nStatsAverageMinutes < 1)
		m_nStatsAverageMinutes = 5;

	CString buffer2;
	for (int i=0;i<15;i++) {
		buffer2.Format(_T("StatColor%i"),i);
		buffer.Format(ini.GetString(buffer2,_T("-1")));	//we can now have black as background
		m_nStatColors[i] = _tstoi64(buffer);
		if (m_nStatColors[i] == -1) 
			m_nStatColors[i] = DefaultStatsColor(i);
	}
	m_nGraphRatio = ini.GetInt(_T("ConnectionsGraphRatio"),1);

	m_bMlDonkeyCM = ini.GetBool(_T("MlDonkeyCM"),true);
	m_bCMNotLog = ini.GetBool(_T("CMNotLog"),false);	//netwolf

	// Added by Tarod -to load hash of A4AF auto file
	char hex_byte[3];
	int byte;
	CString A4AF_hash = ini.GetString(_T("A4AFAutoFileHash"), _T("00000000000000000000000000000000"));
	if (A4AF_hash.GetLength() != 32) A4AF_hash = _T("00000000000000000000000000000000");
	hex_byte[2] = '\0';
	for (int i = 0, j = 0; i < 16; i++, j += 2) {
		hex_byte[0] = A4AF_hash.GetAt(j);
		hex_byte[1] = A4AF_hash.GetAt((j + 1));
		sscanf(hex_byte, "%02X", &byte);
		prefs->A4AF_file_hash[i] = (uchar)byte;
	}

	buffer.Format(ini.GetString(_T("TotalDownloadedBytes"),0,_T("Statistics")));
	m_nTotalDownloadedBytes.Put(_tstoi64(buffer));

	buffer.Format(ini.GetString(_T("TotalUploadedBytes"),0,_T("Statistics")));
	m_nTotalUploadedBytes.Put(_tstoi64(buffer));
	// Maverick
	prefs->proxy.EnablePassword = ini.GetBool(_T("EnablePassword"),false,_T("Proxy"));
	prefs->proxy.UseProxy = ini.GetBool(_T("EnableProxy"),false,_T("Proxy"));
	buffer = "";
	prefs->proxy.name = ini.GetString(_T("Name"),buffer,_T("Proxy"));
	prefs->proxy.password = ini.GetString(_T("Password"),buffer,_T("Proxy"));
	prefs->proxy.user = ini.GetString(_T("User"),buffer,_T("Proxy"));
	prefs->proxy.port = ini.GetInt(_T("Port"),1080,_T("Proxy"));
	prefs->proxy.type = ini.GetInt(_T("Type"),PROXYTYPE_NOPROXY,_T("Proxy"));
	m_bIsASCWOP = ini.GetBool(_T("ConnectWithoutProxy"),false,_T("Proxy"));
	SetLanguage();

	// Web Server
	m_sWebPassword.Put(ini.GetString(_T("Password"), _T("mypass"), _T("WebServer")).GetBuffer(0));
	m_sLowWebPassword.Put(ini.GetString(_T("PasswordLow"), _T("mypass"), _T("WebServer")).GetBuffer(0));
	m_nWebPort = ini.GetInt(_T("Port"), 4711, _T("WebServer"));
	m_bWebEnabled = ini.GetBool(_T("Enabled"), false, _T("WebServer"));
	m_bWebLowEnabled = ini.GetBool(_T("EnabledLow"), false, _T("WebServer"));
	m_bWebUseGzip = ini.GetBool(_T("UseGzip"), true, _T("WebServer"));
	m_nWebPageRefresh = ini.GetInt(_T("PageRefreshTime"), 120, _T("WebServer"));
	// Added by Johnny-B 
	m_bWebIntruderDetection = ini.GetBool(_T("IntruderDetection"), true, _T("WebServer"));
	m_nWebTempDisableLogin = ini.GetInt(_T("TempDisableLogin"), 30, _T("WebServer"));
	m_nWebLoginAttemptsAllowed = ini.GetInt(_T("LoginAttemptsAllowed"), 3, _T("WebServer"));
	// End Johnny-B
	sprintf(prefs->m_sTemplateFile,"%s",ini.GetString(_T("WebTemplateFile"),_T("eMule.tmpl")),_T("WebServer"));	// netwolf

	//////////////////////////////////////////////////////////////////
	// LANCAST (moosetea) - Read the settings

	// Get the Defualt Local IP address, using this clients hostname,
	// we will use this ip as the "default" Lancast ip, if no ip is found in the ini file.
	char localHost[255];
	gethostname(localHost, 255);						// Get this clients hostname
	struct hostent *host = gethostbyname(localHost);	// Get the host info

	// Get this clients default ip as a string
	CString defaultIP(inet_ntoa (*(struct in_addr*)*host->h_addr_list));

	// Get the ini file settings
	m_LancastEnabled = ini.GetBool(_T("Enabled"),false,_T("Lancast"));
	m_UseDefaultIP = ini.GetBool(_T("UseDefaultIP"),true,_T("Lancast"));
	m_DefaultLancastIP = inet_addr(ini.GetString(_T("IPAddress"),defaultIP,_T("Lancast")));
	m_LancastIP = inet_addr(ini.GetString(_T("IPAddress"),defaultIP,_T("Lancast")));
	m_LancastSubnet = inet_addr(ini.GetString(_T("Subnet"),_T("255.255.0.0"),_T("Lancast")));
	m_LancastPort = ini.GetInt(_T("Port"),5000,_T("Lancast"));

	/////////////////////////////////////////////////////////////////////////////

	//START enkeyDEV(kei-kun) -Messaging preferences-
	m_nAcceptMessagesFrom = ini.GetInt(_T("AcceptMessageFrom"),1);
	m_nAwayState = false;
	m_sAwayStateMessage.Put(ini.GetString(_T("AwayStateMessage"),_T("")).GetBuffer(0));
	//END enkeyDEV(kei-kun) -Messaging preferences-
	prefs->maxmsgsessions=ini.GetInt(_T("MaxMessageSessions"),50);

	//SyruS: cleanup
	m_bAutoFilenameCleanup = ini.GetBool(_T("AutoFilenameCleanup"), false, _T("Cleanup"));
	m_bFilenameCleanupTags = ini.GetBool(_T("CleanupTags"), false, _T("Cleanup"));
	buffer2.SetString(_T("http|www.|.com|sharereactor|sharelive|filedonkey|saugstube|eselfilme|eseldownloads|donkey-mania|emulemovies|spanishare|eselpsychos.de|saughilfe.de|goldesel.6x.to|goldesel.tk|freedivx.org|elitedivx|deviance|filenexus|ac3-guru"));
	m_sFilenameCleanups.Put(ini.GetString(_T("FilenameCleanups"), buffer2, _T("Cleanup")).GetBuffer(0));

	if(bChangedVersion)
		SavePreferences();
}

// LANCAST (moosetea) - get the lancast ip address
uint32 CPreferences::GetLancastIP(){
	if (GetLancastUseDefaultIP())
		return m_DefaultLancastIP;
	else
		return m_LancastIP;
}

WORD CPreferences::GetWindowsVersion(){
	if(!bWinVerAlreadyDetected)
	{	
		bWinVerAlreadyDetected = true;
		BOOL bIsNTNBased	=	FALSE;
		m_wWinVer = DetectWinVersion(bIsNTNBased);	
		m_bIsNTBased = bIsNTNBased;
	}	
	return m_wWinVer;
}

uint16 CPreferences::GetDefaultMaxConperFive(){
	switch (GetWindowsVersion()){
		case _WINVER_98_: /*= _WINVER_NT4_*/
			if ( m_bIsNTBased )
			{
				return 10;
			}
			return 5;
		case _WINVER_95_:	
			return 5;
		case _WINVER_ME_:
			return 10;
		case _WINVER_2K_:
			return 20;
		case _WINVER_XP_:
			return 20;
		case _WINVER_SE_:	//Win Server 2003
			return 20;
		default:
			return 10;
	}
}

// Barry - Provide a mechanism for all tables to store/retrieve sort order
int CPreferences::GetColumnSortItem(Table t) const
{
	switch(t) 
	{
		case tableDownload:
			return prefs->tableSortItemDownload;
		case tableUpload:
			return prefs->tableSortItemUpload;
		case tableQueue:
			return prefs->tableSortItemQueue;
		case tableSearch:
			return prefs->tableSortItemSearch;
		case tableShared:
			return prefs->tableSortItemShared;
		case tableServer:
			return prefs->tableSortItemServer;
	}
	return 0;
}

// Barry - Provide a mechanism for all tables to store/retrieve sort order
bool CPreferences::GetColumnSortAscending(Table t) const
{
	switch(t) 
	{
		case tableDownload:
			return prefs->tableSortAscendingDownload;
		case tableUpload:
			return prefs->tableSortAscendingUpload;
		case tableQueue:
			return prefs->tableSortAscendingQueue;
		case tableSearch:
			return prefs->tableSortAscendingSearch;
		case tableShared:
			return prefs->tableSortAscendingShared;
		case tableServer:
			return prefs->tableSortAscendingServer;
	}
	return true;
}

// Barry - Provide a mechanism for all tables to store/retrieve sort order
void CPreferences::SetColumnSortItem(Table t, int sortItem)
{
	switch(t) 
	{
		case tableDownload:
			prefs->tableSortItemDownload = sortItem;
			break;
		case tableUpload:
			prefs->tableSortItemUpload = sortItem;
			break;
		case tableQueue:
			prefs->tableSortItemQueue = sortItem;
			break;
		case tableSearch:
			prefs->tableSortItemSearch = sortItem;
			break;
		case tableShared:
			prefs->tableSortItemShared = sortItem;
			break;
		case tableServer:
			prefs->tableSortItemServer = sortItem;
			break;
	}
}

// Barry - Provide a mechanism for all tables to store/retrieve sort order
void CPreferences::SetColumnSortAscending(Table t, bool sortAscending)
{
	switch(t) 
	{
		case tableDownload:
			prefs->tableSortAscendingDownload = sortAscending;
			break;
		case tableUpload:
			prefs->tableSortAscendingUpload = sortAscending;
			break;
		case tableQueue:
			prefs->tableSortAscendingQueue = sortAscending;
			break;
		case tableSearch:
			prefs->tableSortAscendingSearch = sortAscending;
			break;
		case tableShared:
			prefs->tableSortAscendingShared = sortAscending;
			break;
		case tableServer:
			prefs->tableSortAscendingServer = sortAscending;
			break;
	}
}

void CPreferences::SetLanguage() {
	switch(m_nLanguageID) {
		// read from ini and supported by the program?
		case MAKELANGID(LANG_ENGLISH,SUBLANG_DEFAULT):setlocale( LC_ALL, "english");break;
		case MAKELANGID(LANG_GERMAN,SUBLANG_DEFAULT):setlocale( LC_ALL, "german");break;
		case MAKELANGID(LANG_FRENCH,SUBLANG_DEFAULT):setlocale( LC_ALL, "french");break;
		case MAKELANGID(LANG_DANISH,SUBLANG_DEFAULT):setlocale( LC_ALL, "danish");break;
		case MAKELANGID(LANG_SPANISH,SUBLANG_SPANISH):setlocale( LC_ALL, "spanish");break;
		case MAKELANGID(LANG_DUTCH,SUBLANG_DEFAULT):setlocale( LC_ALL, "dutch");break;
		case MAKELANGID(LANG_RUSSIAN,SUBLANG_DEFAULT):setlocale( LC_ALL, "russian");break;
		case MAKELANGID(LANG_CHINESE,SUBLANG_CHINESE_SIMPLIFIED):setlocale( LC_ALL, "chs");break;
		case MAKELANGID(LANG_CHINESE,SUBLANG_CHINESE_TRADITIONAL):setlocale( LC_ALL, "cht");break;
		case MAKELANGID(LANG_KOREAN,SUBLANG_DEFAULT):setlocale( LC_ALL, "korean");break;
		case MAKELANGID(LANG_POLISH,SUBLANG_DEFAULT):setlocale( LC_ALL, "polish");break;
		case MAKELANGID(LANG_PORTUGUESE,SUBLANG_PORTUGUESE_BRAZILIAN):setlocale( LC_ALL, "ptb");break;
		case MAKELANGID(LANG_FINNISH,SUBLANG_DEFAULT):setlocale( LC_ALL, "finnish");break;
		case MAKELANGID(LANG_ITALIAN,SUBLANG_DEFAULT):setlocale( LC_ALL, "italian");break;
		case MAKELANGID(LANG_CATALAN,SUBLANG_DEFAULT):setlocale( LC_ALL, "esp");break;
		case MAKELANGID(LANG_GREEK,SUBLANG_DEFAULT):setlocale( LC_ALL, "greek");break;
		case MAKELANGID(LANG_HEBREW,SUBLANG_DEFAULT):setlocale( LC_ALL, "hebrew");break;
		case MAKELANGID(LANG_LITHUANIAN,SUBLANG_DEFAULT):setlocale( LC_ALL, "lithuanian");break;
		case MAKELANGID(LANG_ROMANIAN,SUBLANG_DEFAULT):setlocale( LC_ALL, "romanian");break;
		case MAKELANGID(LANG_BASQUE,SUBLANG_DEFAULT):setlocale( LC_ALL, "basque");break;
        case MAKELANGID(LANG_TURKISH,SUBLANG_DEFAULT):setlocale( LC_ALL, "turkish");break;
		default:{
			uint8 nLocalLang = ::GetThreadLocale();
			
            CString strLng1 = GetResString(IDS_MB_LANGUAGEINFO,MAKELANGID(LANG_ENGLISH,SUBLANG_DEFAULT));
			CString strLng2 = _T("");
			switch(nLocalLang) {
				case	LANG_ENGLISH:
				case	LANG_GREEK:
				case 	LANG_GERMAN:
				case 	LANG_FRENCH:
				case 	LANG_DANISH:
				case 	LANG_SPANISH:
				case 	LANG_DUTCH:
				case 	LANG_RUSSIAN:
				case 	LANG_CHINESE:
				case 	LANG_KOREAN:
				case 	LANG_POLISH:
				case 	LANG_PORTUGUESE:
				case	LANG_FINNISH:
				case	LANG_ITALIAN:
				case	LANG_CATALAN:
				case	LANG_HEBREW:
				case	LANG_LITHUANIAN:
				case    LANG_ROMANIAN:
				case    LANG_BASQUE:
				case    LANG_TURKISH:
				{
					m_nLanguageID = MAKELANGID(nLocalLang,SUBLANG_DEFAULT);
					strLng1 = GetResString(IDS_MB_LANGUAGEINFO, m_nLanguageID);//Cax2 fix
					break;
				}
				default:
					m_nLanguageID = MAKELANGID(LANG_ENGLISH,SUBLANG_DEFAULT);
			}
			if ( m_bIsNTBased )
			{
				SetThreadLocale(m_nLanguageID); // THIS FUNCTION DOEN NOT WORK ON WIN9X !!!!!!!!1
			}
			else
			{
				ASSERT(false); // locale NOT set because we just can't do it on Win9x!!!!
			}

			CString strMessage = strLng1 + CString("\n\n") + strLng2;;
			MessageBox(0, strMessage.GetBuffer(),_T("Language"),64);
			
			// DonGato
			CString strQuestion;
			if (nLocalLang == LANG_SPANISH)
				strQuestion = CString("\xbf") + GetResString(IDS_MULTIPLE, m_nLanguageID) + CString("?");
			else
				strQuestion = GetResString(IDS_MULTIPLE, m_nLanguageID) + CString("?");
			
			if (IDNO==MessageBox(0, strQuestion.GetBuffer(),_T("eMule Plus"),MB_YESNO | MB_DEFBUTTON2))	//SyruS make "no" the default for users
				return;
			else
				m_bMultiple = true;
		}
	}
	if ( m_bIsNTBased )
	{
		SetThreadLocale(m_nLanguageID);
	}

	// if loading a string fails, set language to English
	if (GetResString(IDS_MB_LANGUAGEINFO, m_nLanguageID).GetLength()==0) {
		m_nLanguageID = MAKELANGID(LANG_ENGLISH,SUBLANG_DEFAULT);
		if ( theApp.glob_prefs->IsNTBased() )
		{
			SetThreadLocale(m_nLanguageID);
		}
	}
}

void CPreferences::SetMainProcessPriority(int Value) 			
{
	m_nMainProcessPriority = Value;
}

BOOL CPreferences::IsNTBased(void)
{
	GetWindowsVersion();
	return (m_bIsNTBased);
}