#include "StdAfx.h"
#include "emule.h"
#include "webserver.h"
#include "ED2KLink.h"
#include "MD5Sum.h"
#include <stdlib.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define HTTPInit _T("Server: eMule\r\nConnection: close\r\nContent-Type: text/html\r\n")
#define HTTPInitGZ _T("Server: eMule\r\nConnection: close\r\nContent-Type: text/html\r\nContent-Encoding: gzip\r\n")

#define WEB_SERVER_TEMPLATES_VERSION	1000

static dwThread = 0;

CWebServer::CWebServer(void)
{
	m_Params.bShowUploadQueue = false;
	m_Params.bShowUploadQueueBanned = false;
	m_Params.bShowUploadQueueFriend = false;
	m_Params.bShowUploadQueueCredit = false;

	m_Params.DownloadSort = DOWN_SORT_NAME;
	m_Params.bDownloadSortReverse = false;
	m_Params.ServerSort = SERVER_SORT_NAME;
	m_Params.bServerSortReverse = false;
	m_Params.SharedSort = SHARED_SORT_NAME;
	m_Params.bSharedSortReverse = false;

	m_Params.sLastModified = _T("");
	m_Params.sETag = _T("");

	dwThread = GetCurrentThreadId();
	m_bServerWorking = false;

	m_nIntruderDetect = 0;
	m_nStartTempDisabledTime = 0;
	m_bIsTempDisabled = false;
}

void CWebServer::ReloadTemplates()
{
	EMULE_TRY

	CString sPrevLocale(setlocale(LC_TIME, NULL));
	_tsetlocale(LC_TIME, _T("English"));
	CTime t = CTime::GetCurrentTime();
	m_Params.sLastModified = t.FormatGmt("%a, %d %b %Y %H:%M:%S GMT");
	m_Params.sETag = MD5Sum(m_Params.sLastModified).GetHash();
	_tsetlocale(LC_TIME, sPrevLocale);

	CString sFile;
	if (theApp.glob_prefs->GetTemplate()=="" || theApp.glob_prefs->GetTemplate().MakeLower()=="emule.tmpl")
		sFile= theApp.glob_prefs->GetAppDir() + CString("eMule.tmpl");
	else sFile=theApp.glob_prefs->GetTemplate();

	CStdioFile file;
	if(file.Open(sFile, CFile::modeRead|CFile::typeText))
	{
		CString sAll;
		for(;;)
		{
			CString sLine;
			if(!file.ReadString(sLine))
				break;

			sAll += sLine + _T("\n");
		}
		file.Close();

		CString sVersion = _LoadTemplate(sAll,_T("TMPL_VERSION"));
		long lVersion = _tstol(sVersion);
		if(lVersion < WEB_SERVER_TEMPLATES_VERSION)
		{
			if(theApp.glob_prefs->GetWSIsEnabled() || m_bServerWorking)
				AddLogLine(true,IDS_WS_ERR_LOADTEMPLATE,sFile);
			if (m_bServerWorking)
				StopSockets(); 
            m_bServerWorking = false; 
			theApp.glob_prefs->SetWSIsEnabled(false);
		}
		else
		{
			m_Templates.sHeader = _LoadTemplate(sAll,_T("TMPL_HEADER"));
			m_Templates.sHeaderMetaRefresh = _LoadTemplate(sAll,_T("TMPL_HEADER_META_REFRESH"));
			m_Templates.sHeaderStylesheet = _LoadTemplate(sAll,_T("TMPL_HEADER_STYLESHEET"));
			m_Templates.sFooter = _LoadTemplate(sAll,_T("TMPL_FOOTER"));
			m_Templates.sServerList = _LoadTemplate(sAll,_T("TMPL_SERVER_LIST"));
			m_Templates.sServerLine = _LoadTemplate(sAll,_T("TMPL_SERVER_LINE"));
			m_Templates.sTransferImages = _LoadTemplate(sAll,_T("TMPL_TRANSFER_IMAGES"));
			m_Templates.sTransferList = _LoadTemplate(sAll,_T("TMPL_TRANSFER_LIST"));
			m_Templates.sTransferDownHeader = _LoadTemplate(sAll,_T("TMPL_TRANSFER_DOWN_HEADER"));
			m_Templates.sTransferDownFooter = _LoadTemplate(sAll,_T("TMPL_TRANSFER_DOWN_FOOTER"));
			m_Templates.sTransferDownLine = _LoadTemplate(sAll,_T("TMPL_TRANSFER_DOWN_LINE"));
			m_Templates.sTransferUpHeader = _LoadTemplate(sAll,_T("TMPL_TRANSFER_UP_HEADER"));
			m_Templates.sTransferUpFooter = _LoadTemplate(sAll,_T("TMPL_TRANSFER_UP_FOOTER"));
			m_Templates.sTransferUpLine = _LoadTemplate(sAll,_T("TMPL_TRANSFER_UP_LINE"));
			m_Templates.sTransferUpQueueShow = _LoadTemplate(sAll,_T("TMPL_TRANSFER_UP_QUEUE_SHOW"));
			m_Templates.sTransferUpQueueHide = _LoadTemplate(sAll,_T("TMPL_TRANSFER_UP_QUEUE_HIDE"));
			m_Templates.sTransferUpQueueLine = _LoadTemplate(sAll,_T("TMPL_TRANSFER_UP_QUEUE_LINE"));
			m_Templates.sTransferUpQueueBannedShow = _LoadTemplate(sAll,_T("TMPL_TRANSFER_UP_QUEUE_BANNED_SHOW"));
			m_Templates.sTransferUpQueueBannedHide = _LoadTemplate(sAll,_T("TMPL_TRANSFER_UP_QUEUE_BANNED_HIDE"));
			m_Templates.sTransferUpQueueBannedLine = _LoadTemplate(sAll,_T("TMPL_TRANSFER_UP_QUEUE_BANNED_LINE"));
			m_Templates.sTransferUpQueueFriendShow = _LoadTemplate(sAll,_T("TMPL_TRANSFER_UP_QUEUE_FRIEND_SHOW"));
			m_Templates.sTransferUpQueueFriendHide = _LoadTemplate(sAll,_T("TMPL_TRANSFER_UP_QUEUE_FRIEND_HIDE"));
			m_Templates.sTransferUpQueueFriendLine = _LoadTemplate(sAll,_T("TMPL_TRANSFER_UP_QUEUE_FRIEND_LINE"));
			m_Templates.sTransferUpQueueCreditShow = _LoadTemplate(sAll,_T("TMPL_TRANSFER_UP_QUEUE_CREDIT_SHOW"));
			m_Templates.sTransferUpQueueCreditHide = _LoadTemplate(sAll,_T("TMPL_TRANSFER_UP_QUEUE_CREDIT_HIDE"));
			m_Templates.sTransferUpQueueCreditLine = _LoadTemplate(sAll,_T("TMPL_TRANSFER_UP_QUEUE_CREDIT_LINE"));
			m_Templates.sTransferBadLink = _LoadTemplate(sAll,_T("TMPL_TRANSFER_BAD_LINK"));
			m_Templates.sSharedList = _LoadTemplate(sAll,_T("TMPL_SHARED_LIST"));
			m_Templates.sSharedLine = _LoadTemplate(sAll,_T("TMPL_SHARED_LINE"));
			m_Templates.sGraphs = _LoadTemplate(sAll,_T("TMPL_GRAPHS"));
			m_Templates.sLog = _LoadTemplate(sAll,_T("TMPL_LOG"));
			m_Templates.sServerInfo = _LoadTemplate(sAll,_T("TMPL_SERVERINFO"));
			m_Templates.sDebugLog = _LoadTemplate(sAll,_T("TMPL_DEBUGLOG"));
			m_Templates.sStats = _LoadTemplate(sAll,_T("TMPL_STATS"));
			m_Templates.sPreferences = _LoadTemplate(sAll,_T("TMPL_PREFERENCES"));
			m_Templates.sLogin = _LoadTemplate(sAll,_T("TMPL_LOGIN"));
			m_Templates.sConnectedServer = _LoadTemplate(sAll,_T("TMPL_CONNECTED_SERVER"));
			m_Templates.sAddServerBox = _LoadTemplate(sAll,_T("TMPL_ADDSERVERBOX"));
			m_Templates.sWebSearch = _LoadTemplate(sAll,_T("TMPL_WEBSEARCH"));
			m_Templates.sSearch = _LoadTemplate(sAll,_T("TMPL_SEARCH"));
			m_Templates.iProgressbarWidth = _tstoi(_LoadTemplate(sAll,_T("PROGRESSBARWIDTH")));
			m_Templates.sSearchHeader = _LoadTemplate(sAll,_T("TMPL_SEARCH_RESULT_HEADER"));			
			m_Templates.sSearchResultLine = _LoadTemplate(sAll,_T("TMPL_SEARCH_RESULT_LINE"));			
			m_Templates.sProgressbarImgs = _LoadTemplate(sAll,_T("PROGRESSBARIMGS"));
			m_Templates.sProgressbarImgsPercent = _LoadTemplate(sAll,_T("PROGRESSBARPERCENTIMG"));

			m_Templates.sProgressbarImgsPercent.Replace(_T("[PROGRESSGIFNAME]"),_T("%s"));
			m_Templates.sProgressbarImgsPercent.Replace(_T("[PROGRESSGIFINTERNAL]"),_T("%i"));
			m_Templates.sProgressbarImgs.Replace(_T("[PROGRESSGIFNAME]"),_T("%s"));
			m_Templates.sProgressbarImgs.Replace(_T("[PROGRESSGIFINTERNAL]"),_T("%i"));
		}
	}
	else
        if(m_bServerWorking)
		{
			AddLogLine(true,IDS_WEB_ERR_CANTLOAD, sFile);
			StopSockets(); 
            m_bServerWorking = false; 
			theApp.glob_prefs->SetWSIsEnabled(false);
		}

	EMULE_CATCH
}

CWebServer::~CWebServer(void)
{
	if (m_bServerWorking) StopSockets();
}

CString CWebServer::_LoadTemplate(CString sAll, CString sTemplateName)
{
	EMULE_TRY

	CString sRet = _T("");
	int nStart = sAll.Find(_T("<--") + sTemplateName + _T("-->"));
	int nEnd = sAll.Find(_T("<--") + sTemplateName + _T("_END-->"));
	if(nStart != -1 && nEnd != -1 && nStart<nEnd) //(0.29b)
	{
		nStart += sTemplateName.GetLength() + 7;
		sRet = sAll.Mid(nStart, nEnd - nStart - 1);
	} else	{
		if (sTemplateName=="TMPL_VERSION")
			AddLogLine(true,IDS_WS_ERR_LOADTEMPLATE,sTemplateName);
		if (nStart==-1) 
			AddDebugLogLine(false,IDS_WEB_ERR_CANTLOAD,sTemplateName);
	}
	return sRet;

	EMULE_CATCH

	return _T("");
}

void CWebServer::RestartServer() {	//Cax2 - restarts the server with the new port settings
	StopSockets();
	if (m_bServerWorking)	
		StartSockets(this);
}

void CWebServer::StartServer(void)
{
	EMULE_TRY
	if(m_bServerWorking != theApp.glob_prefs->GetWSIsEnabled())
		m_bServerWorking = theApp.glob_prefs->GetWSIsEnabled();
	else
		return;

	if (m_bServerWorking) 
	{
		ReloadTemplates();
		StartSockets(this); 
		m_nIntruderDetect = 0;
	    m_nStartTempDisabledTime = 0;
	    m_bIsTempDisabled = false;
	} else StopSockets();

	if(theApp.glob_prefs->GetWSIsEnabled() && m_bServerWorking)
		AddLogLine(false,_T("%s: %s"),_GetPlainResString(IDS_PW_HTTPD), _GetPlainResString(IDS_ENABLED).MakeLower());
	else
		AddLogLine(false,_T("%s: %s"),_GetPlainResString(IDS_PW_HTTPD), _GetPlainResString(IDS_DISABLED).MakeLower());

	EMULE_CATCH
}

void CWebServer::_RemoveServer(CString sIP, int nPort)
{
	EMULE_TRY

	CServer* server=theApp.serverlist->GetServerByAddress(sIP.GetBuffer(), nPort);
	if (server!=NULL) theApp.emuledlg->SendMessage(WEB_REMOVE_SERVER, (WPARAM)server, NULL);

	EMULE_CATCH
}

void CWebServer::_AddToStatic(CString sIP, int nPort)
{
	EMULE_TRY

	CServer* server=theApp.serverlist->GetServerByAddress(sIP.GetBuffer(), nPort);
	if (server!=NULL) theApp.emuledlg->SendMessage(WEB_ADD_TO_STATIC, (WPARAM)server, NULL);

	EMULE_CATCH
}

void CWebServer::_RemoveFromStatic(CString sIP, int nPort)
{
	EMULE_TRY

	CServer* server=theApp.serverlist->GetServerByAddress(sIP.GetBuffer(), nPort);
	if (server!=NULL) theApp.emuledlg->SendMessage(WEB_REMOVE_FROM_STATIC, (WPARAM)server, NULL);

	EMULE_CATCH
}

void CWebServer::_ServerPrioUp(CString sIP, int nPort) ////
{
	EMULE_TRY

	CServer* server=theApp.serverlist->GetServerByAddress(sIP.GetBuffer(), nPort);
	if(server->GetPreferences()== CServer::ePR_LOW)
		server->SetPreference((CServer::eServerPriority)PR_NORMAL);
	else
		server->SetPreference((CServer::eServerPriority)PR_HIGH);
	if (theApp.emuledlg->serverwnd) 
		theApp.emuledlg->serverwnd.serverlistctrl.RefreshServer(*server);

	EMULE_CATCH
}

void CWebServer::_ServerPrioDown(CString sIP, int nPort) ////
{
	EMULE_TRY

	CServer* server=theApp.serverlist->GetServerByAddress(sIP.GetBuffer(), nPort);
	if(server->GetPreferences()== CServer::ePR_HIGH)
		server->SetPreference((CServer::eServerPriority)PR_NORMAL);
	else
		server->SetPreference((CServer::eServerPriority)PR_LOW);
	if (theApp.emuledlg->serverwnd) 
		theApp.emuledlg->serverwnd.serverlistctrl.RefreshServer(*server);
	EMULE_CATCH
}

void CWebServer::_SetSharedFilePriority(CString hash, uint8 priority)
{
	EMULE_TRY

	CKnownFile* cur_file;
	uchar fileid[16];
	DecodeBase16(hash.GetBuffer(),hash.GetLength(),fileid);

	cur_file=theApp.sharedfiles->GetFileByID(fileid);
	
	if (cur_file==0) return;

	if(priority >= 0 && priority < 5)
	{
		cur_file->SetAutoPriority(false);
		cur_file->SetPriority(priority);
	}
	else if(priority == 5 && cur_file->IsPartFile())
	{
		cur_file->SetAutoPriority(true);
		cur_file->UpdateUploadAutoPriority();
	}

	theApp.sharedfiles->UpdateItem(cur_file); // DonGato: refresh GUI on priority change 

	EMULE_CATCH
}

//+++
void CWebServer::_SetSharedFileJumpstart(CString hash)
{
	EMULE_TRY

	CKnownFile* cur_file;
	uchar fileid[16];
	DecodeBase16(hash.GetBuffer(),hash.GetLength(),fileid);

	cur_file=theApp.sharedfiles->GetFileByID(fileid);
	
	if (cur_file==0) return;

	cur_file->SetJumpstartEnabled(true);

	theApp.sharedfiles->UpdateItem(cur_file);

	EMULE_CATCH
}

//+++
void CWebServer::_SetSharedFileNoJumpstart(CString hash)
{
	EMULE_TRY

	CKnownFile* cur_file;
	uchar fileid[16];
	DecodeBase16(hash.GetBuffer(),hash.GetLength(),fileid);

	cur_file=theApp.sharedfiles->GetFileByID(fileid);
	
	if (cur_file==0) return;

	cur_file->SetJumpstartEnabled(false);

	theApp.sharedfiles->UpdateItem(cur_file);

	EMULE_CATCH
}

void CWebServer::AddStatsLine(UpDown line)
{
	EMULE_TRY

	m_Params.PointsForWeb.Add(line);
	if(m_Params.PointsForWeb.GetCount() > WEB_GRAPH_WIDTH)
		m_Params.PointsForWeb.RemoveAt(0);

	EMULE_CATCH
}

CString CWebServer::_SpecialChars(CString str, bool noquote /*=false*/) 
{
	EMULE_TRY

	str.Replace(_T("&"),_T("&amp;"));
	str.Replace(_T("<"),_T("&lt;"));
	str.Replace(_T(">"),_T("&gt;"));
	str.Replace(_T("\""),_T("&quot;"));
	if(noquote)
	{
		str.Replace(_T("'"), _T("&#8217;"));
		str.Replace(_T("\n"), _T("\\n"));
	}
	return str;

	EMULE_CATCH

	return _T("");
}

void CWebServer::_ConnectToServer(CString sIP, int nPort)
{
	EMULE_TRY

	CServer* server=theApp.serverlist->GetServerByAddress(sIP.GetBuffer(), nPort);
	if (server!=NULL) theApp.emuledlg->SendMessage(WEB_CONNECT_TO_SERVER, (WPARAM)server, NULL);

	EMULE_CATCH
}

void CWebServer::ProcessImgFileReq(ThreadData Data) {
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return;

	CString filename=Data.sURL;
	CString contenttype;

	if (filename.Right(4).MakeLower()==".gif") contenttype="Content-Type: image/gif\r\n";
	else if (filename.Right(4).MakeLower()==".jpg"  || filename.Right(5).MakeLower()==".jpeg") contenttype="Content-Type: image/jpg\r\n";
	else if (filename.Right(4).MakeLower()==".bmp") contenttype="Content-Type: image/bmp\r\n";
	else if (filename.Right(4).MakeLower()==".png") contenttype="Content-Type: image/png\r\n";
	//(0.29b) //DonQ - additional filetypes
	else if (filename.Right(4).MakeLower()==".ico") contenttype="Content-Type: image/x-icon\r\n";
	else if (filename.Right(4).MakeLower()==".css") contenttype="Content-Type: text/css\r\n";
	else if (filename.Right(3).MakeLower()==".js") contenttype="Content-Type: text/javascript\r\n";

	contenttype += _T("Last-Modified: ") + pThis->m_Params.sLastModified + _T("\r\n") + _T("ETag: ") + pThis->m_Params.sETag + _T("\r\n");

	// DonGato: expire images after 1 year (overridable by browser)
	CString sPrevLocale(setlocale(LC_TIME, NULL));
	_tsetlocale(LC_TIME, _T("English"));
	CTime t = CTime::GetCurrentTime();
	t += 60*60*24*365;
	CString expires = t.FormatGmt("%a, %d %b %Y %H:%M:%S GMT");
	_tsetlocale(LC_TIME, sPrevLocale);
	contenttype += _T("Expires: ") + expires + _T("\r\n");

	filename.Delete(0);
	filename.Replace('/','\\');
	filename=CString(theApp.glob_prefs->GetAppDir())+_T("webserver\\")+filename;
	CFile file;
	if(file.Open(filename, CFile::modeRead|CFile::shareDenyWrite|CFile::typeBinary)) //(0.29b)
	{
		TCHAR* buffer=new TCHAR[file.GetLength()];
		int size=file.Read(buffer,file.GetLength());
		file.Close();
		Data.pSocket->SendContent(contenttype, buffer, size);
		delete[] buffer;
	}

	EMULE_CATCH
}

void CWebServer::ProcessURL(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return;

	SetThreadLocale(theApp.glob_prefs->GetLanguageID());

	//(0.29b)//////////////////////////////////////////////////////////////////
	// Here we are in real trouble! We are accessing the entire emule main thread
	// data without any syncronization!! Either we use the message pump for emuledlg
	// or use some hundreds of critical sections... For now, an exception handler
	// shoul avoid the worse things.
	//////////////////////////////////////////////////////////////////////////
	CoInitialize(NULL);

	EMULE_TRY	//(0.29b)
	bool isUseGzip = theApp.glob_prefs->GetWebUseGzip();
	CString Out = _T("");
	CString OutE = _T("");	// List Entry Templates
	CString OutS = _T("");	// ServerStatus Templates
	TCHAR *gzipOut = NULL;
	long gzipLen=0;
	
	CString HTTPProcessData = _T("");
	CString HTTPTemp = _T("");
	srand ( time(NULL) );
	DWORD dwCurTick = ::GetTickCount();

	long lSession = 0;
	if(_ParseURL(Data.sURL, _T("ses")) != "")
		lSession = _tstol(_ParseURL(Data.sURL, _T("ses")));

	//Added by Johnny-B -IntruderDetection
	if(pThis->GetIsTempDisabled() &&
	((dwCurTick - pThis->m_nStartTempDisabledTime) > (theApp.glob_prefs->GetWSTempDisableLogin() * 60000)))
	{
        pThis->m_bIsTempDisabled = false;
        pThis->m_nStartTempDisabledTime = 0;
		AddLogLine(true,IDS_WEB_INTRBLOCKEND);
	}//End Johnny-B

	if ((_ParseURL(Data.sURL, _T("w")) == "password")
	&& (!pThis->GetIsTempDisabled()))
	{
		CString test=MD5Sum(_ParseURL(Data.sURL, _T("p"))).GetHash();
		CString t_ulCurIP;//log intruder
		t_ulCurIP.Format(_T("%i.%i.%i.%i"),(uint8)pThis->m_ulCurIP,(uint8)(pThis->m_ulCurIP>>8),(uint8)(pThis->m_ulCurIP>>16),(uint8)(pThis->m_ulCurIP>>24));//log intruder

		if(MD5Sum(_ParseURL(Data.sURL, _T("p"))).GetHash() == theApp.glob_prefs->GetWSPass())
		{
			Session ses;
			ses.admin=true;
			ses.startTime = CTime::GetCurrentTime();
			ses.lSession = lSession = rand() * 10000L + rand();
			pThis->m_Params.Sessions.Add(ses);
			AddLogLine(true,IDS_WEB_ADMINLOGIN,t_ulCurIP);
			pThis->m_nIntruderDetect = 0;
		}
		else if(theApp.glob_prefs->GetWSIsLowUserEnabled() && theApp.glob_prefs->GetWSLowPass()!="" && MD5Sum(_ParseURL(Data.sURL, _T("p"))).GetHash() == theApp.glob_prefs->GetWSLowPass())
		{
			Session ses;
			ses.admin=false;
			ses.startTime = CTime::GetCurrentTime();
			ses.lSession = lSession = rand() * 10000L + rand();
			pThis->m_Params.Sessions.Add(ses);
			AddLogLine(true,IDS_WEB_GUESTLOGIN,t_ulCurIP);
			pThis->m_nIntruderDetect = 0;
		} else if(theApp.glob_prefs->IsWSIntruderDetectionEnabled())
			{
				pThis->m_nIntruderDetect++;
				AddLogLine(true,IDS_WEB_INTRTRIESLEFT,t_ulCurIP,theApp.glob_prefs->GetWSLoginAttemptsAllowed()-(pThis->m_nIntruderDetect));
				if (pThis->m_nIntruderDetect == theApp.glob_prefs->GetWSLoginAttemptsAllowed()){
					AddLogLine(true,IDS_WEB_INTRDETECTION, pThis->m_nIntruderDetect);
				if (theApp.glob_prefs->GetWSTempDisableLogin() > 0)
				{
					pThis->m_nStartTempDisabledTime = ::GetTickCount();
					pThis->m_bIsTempDisabled = true;
					pThis->AddLogLine(true,IDS_WEB_INTRBLOCKTIME, theApp.glob_prefs->GetWSTempDisableLogin());
					pThis->m_nIntruderDetect = 0;
				} else {
					theApp.glob_prefs->SetWSIsEnabled(false);
		            pThis->m_bServerWorking = false; 
					StopSockets();
					AddLogLine(true,IDS_WEB_INTRWEBDISABLE);
					pThis->m_nIntruderDetect = 0;
				}
			}
		}else{//End Johnny-B -IntruderDetection
			AddLogLine(true,IDS_WEB_BADLOGINATTEMPT);
		}
		isUseGzip = false; // [Julien]
	}

	CString sSession; sSession.Format(_T("%ld"), lSession);

	if (_ParseURL(Data.sURL, _T("w")) == "logout") 
		_RemoveSession(Data, lSession);

	if(_IsLoggedIn(Data, lSession))
	{
		Out += _GetHeader(Data, lSession);
		CString sPage = _ParseURL(Data.sURL, _T("w"));
		if (sPage == "server") 
		{
			Out += _GetServerList(Data);
		}
		else
		if (sPage == "shared") 
		{ 
			Out += _GetSharedFilesList(Data);
		}
		else
		if (sPage == "transfer") 
		{
			Out += _GetTransferList(Data);
		}
		else
		if (sPage == "websearch") 
		{
			Out += _GetWebSearch(Data);
		}
		else
		if (sPage == "search") 
		{
			Out += _GetSearch(Data);
		}
		else
		if (sPage == "graphs")
		{
			Out += _GetGraphs(Data);
		}
		else
		if (sPage == "log") 
		{
			Out += _GetLog(Data);
		}
		if (sPage == "sinfo") 
		{
			Out += _GetServerInfo(Data);
		}
		if (sPage == "debuglog") 
		{
			Out += _GetDebugLog(Data);
		}
		if (sPage == "stats") 
		{
			Out += _GetStats(Data);
		}
		if (sPage == "options") 
		{
			isUseGzip = false;
			Out += _GetPreferences(Data);
		}
		Out += _GetFooter(Data);

		if (sPage == "")
			isUseGzip = false;

		if(isUseGzip)
		{
			bool bOk = false;
			try
			{
				uLongf destLen = Out.GetLength() + 1024;
				gzipOut = new TCHAR[destLen];
				if(_GzipCompress((Bytef*)gzipOut, &destLen, (const Bytef*)(TCHAR*)Out.GetBuffer(0), Out.GetLength(), Z_DEFAULT_COMPRESSION) == Z_OK)
				{
					bOk = true;
					gzipLen = destLen;
				}
			}
			catch(...)
			{
			}
			if(!bOk)
			{
				isUseGzip = false;
				if(gzipOut != NULL)
				{
					delete[] gzipOut;
					gzipOut = NULL;
				}
			}
		}
	}
	else 
	{
		isUseGzip = false;
		Out += _GetLoginScreen(Data);
	}

	// send answer ...
	if(!isUseGzip)
	{
		Data.pSocket->SendContent(HTTPInit, Out, Out.GetLength());
	}
	else
	{
		Data.pSocket->SendContent(HTTPInitGZ, gzipOut, gzipLen);
	}
	if(gzipOut != NULL)
		delete[] gzipOut;

	EMULE_CATCH
	CoUninitialize(); //(0.29b)

	EMULE_CATCH
}

CString CWebServer::_ParseURLArray(CString URL, CString fieldname) {
	EMULE_TRY

	CString res,temp;

	while (URL.GetLength()>0) {
		int pos=URL.MakeLower().Find(fieldname.MakeLower() + _T("="));
		if (pos>-1) {
			temp=_ParseURL(URL,fieldname);
			if (temp=="") break;
			res.Append(temp + _T("|"));
			URL.Delete(pos,10);
		} else break;
	}
	return res;

	EMULE_CATCH
	return _T("");
}

CString CWebServer::_ParseURL(CString URL, CString fieldname)
{
	EMULE_TRY

	CString value = _T("");
	CString Parameter = _T("");
	TCHAR fromReplace[4] = _T("");	// decode URL
	TCHAR toReplace[2] = _T("");		// decode URL
	int i = 0;
	int findPos = -1;
	int findLength = 0;

	if (URL.Find(_T("?")) > -1) {
		Parameter = URL.Mid(URL.Find(_T("?"))+1, URL.GetLength()-URL.Find(_T("?"))-1);

		// search the fieldname beginning / middle and strip the rest...
		if (Parameter.Find(fieldname + _T("=")) == 0) {
			findPos = 0;
			findLength = fieldname.GetLength() + 1;
		}
		if (Parameter.Find(_T("&") + fieldname + _T("=")) > -1) {
			findPos = Parameter.Find(_T("&") + fieldname + _T("="));
			findLength = fieldname.GetLength() + 2;
		}
		if (findPos > -1) {
			Parameter = Parameter.Mid(findPos + findLength, Parameter.GetLength());
			if (Parameter.Find(_T("&")) > -1) {
				Parameter = Parameter.Mid(0, Parameter.Find(_T("&")));
			}
	
			value = Parameter;

			// decode value ...
			value.Replace(_T("+"), _T(" "));
			for (i = 0 ; i <= 255 ; i++) {
				_stprintf(fromReplace, _T("%%%02x"), i);
				toReplace[0] = (char)i;
				toReplace[1] = NULL;
				value.Replace(fromReplace, toReplace);
				_stprintf(fromReplace, _T("%%%02X"), i);
				value.Replace(fromReplace, toReplace);
			}
		}
	}

	return value;

	EMULE_CATCH

	return _T("");
}

CString CWebServer::_GetHeader(ThreadData Data, long lSession)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return _T("");

	CString sSession; sSession.Format(_T("%ld"), lSession);

	CString Out = pThis->m_Templates.sHeader;

	Out.Replace(_T("[CharSet]"), _GetWebCharSet());

	if(theApp.glob_prefs->GetWebPageRefresh() != 0)
	{
		CString sPage = _ParseURL(Data.sURL, _T("w"));
		if ((sPage == "transfer") ||
			(sPage == "server") ||
			(sPage == "graphs") ||
			(sPage == "log") ||
			(sPage == "sinfo") ||
			(sPage == "debuglog") ||
			(sPage == "stats"))
		{
			CString sT = pThis->m_Templates.sHeaderMetaRefresh;
			CString sRefresh; sRefresh.Format(_T("%d"), theApp.glob_prefs->GetWebPageRefresh());
			sT.Replace(_T("[RefreshVal]"), sRefresh);
			sT.Replace(_T("[wCommand]"), _ParseURL(Data.sURL, _T("w")));
			Out.Replace(_T("[HeaderMeta]"), sT);
		}
	}
	Out.Replace(_T("[Session]"), sSession);
	Out.Replace(_T("[HeaderMeta]"), _T("")); // In case there are no meta
	Out.Replace(_T("[eMuleAppName]"), _GetPlainResString(IDS_EMULE_PLUS));
	Out.Replace(_T("[version]"), CURRENT_VERSION_LONG);
	Out.Replace(_T("[StyleSheet]"), pThis->m_Templates.sHeaderStylesheet);
	Out.Replace(_T("[WebControl]"), _GetPlainResString(IDS_WEB_CONTROL));
	Out.Replace(_T("[Transfer]"), _GetPlainResString(IDS_CD_TRANS));
	Out.Replace(_T("[Server]"), _GetPlainResString(IDS_SV_SERVERLIST));
	Out.Replace(_T("[Shared]"), _GetPlainResString(IDS_SHAREDFILES));
	Out.Replace(_T("[Graphs]"), _GetPlainResString(IDS_GRAPHS));
	Out.Replace(_T("[Log]"), _GetPlainResString(IDS_SV_LOG));
	Out.Replace(_T("[ServerInfo]"), _GetPlainResString(IDS_SV_SERVERINFO));
	Out.Replace(_T("[DebugLog]"), _GetPlainResString(IDS_SV_DEBUGLOG));
	Out.Replace(_T("[Stats]"), _GetPlainResString(IDS_SF_STATISTICS));
	Out.Replace(_T("[Options]"), _GetPlainResString(IDS_EM_PREFS));
	Out.Replace(_T("[Logout]"), _GetPlainResString(IDS_WEB_LOGOUT));
	Out.Replace(_T("[Search]"), _GetPlainResString(IDS_SW_SEARCHBOX));
	Out.Replace(_T("[Download]"), _GetPlainResString(IDS_SW_DOWNLOAD));
	Out.Replace(_T("[Ed2klink]"), _GetPlainResString(IDS_SW_LINK));
	Out.Replace(_T("[Start]"), _GetPlainResString(IDS_SW_START));

	Out.Replace(_T("[FileIsHashing]"), _GetPlainResString(IDS_HASHING, true));
	Out.Replace(_T("[FileIsErroneous]"), _GetPlainResString(IDS_ERRORLIKE, true));
	Out.Replace(_T("[FileIsCompleting]"), _GetPlainResString(IDS_COMPLETING, true));
	Out.Replace(_T("[FileDetails]"), _GetPlainResString(IDS_FD_TITLE, true));
	Out.Replace(_T("[Ed2klink]"), _GetPlainResString(IDS_SW_LINK, true));
	Out.Replace(_T("[ClearCompleted]"), _GetPlainResString(IDS_DL_CLEAR, true));
	Out.Replace(_T("[A4AF]"), _GetPlainResString(IDS_ALL_A4AF_TO_HERE, true));
	Out.Replace(_T("[AUTOA4AF]"), _GetPlainResString(IDS_ALL_A4AF_TO_HERE, true) + " (" + _GetPlainResString(IDS_PRIOAUTO, true) + ")"); // Purity: Auto-A4AF
	Out.Replace(_T("[DropSCB]"), _GetPlainResString(IDS_DROPSUPERCOMPRESSED, true));
	Out.Replace(_T("[NoNeededSources]"), _GetPlainResString(IDS_DROPNONEEDEDSRCS, true));
	Out.Replace(_T("[Resume]"), _GetPlainResString(IDS_DL_RESUME, true));
	Out.Replace(_T("[Stop]"), _GetPlainResString(IDS_DL_STOP, true));
	Out.Replace(_T("[ConfirmCancel]"), _GetPlainResString(IDS_Q_CANCELDL2, true));
	Out.Replace(_T("[Cancel]"), _GetPlainResString(IDS_MAIN_BTN_CANCEL, true));
	Out.Replace(_T("[PriorityUp]"), _GetPlainResString(IDS_PRIORITY_UP, true));
	Out.Replace(_T("[PriorityDown]"), _GetPlainResString(IDS_PRIORITY_DOWN, true));
	Out.Replace(_T("[Connect]"), _GetPlainResString(IDS_IRC_CONNECT, true));
	Out.Replace(_T("[ConfirmRemove]"), _GetPlainResString(IDS_WEB_CONFIRM_REMOVE_SERVER, true));
	Out.Replace(_T("[RemoveServer]"), _GetPlainResString(IDS_REMOVETHIS, true));
	Out.Replace(_T("[AddToStatic]"), _GetPlainResString(IDS_ADDTOSTATIC, true));
	Out.Replace(_T("[ConfirmJumpstart]"), _GetPlainResString(IDS_JS_DISABLE, true) + _T("\\n"));//+++
	Out.Replace(_T("[Jumpstart]"), _GetPlainResString(IDS_PRIOJUMPSTART, true));//+++

	TCHAR HTTPTempC[100] = _T("");
	CString sConnected = _T("");
#ifdef OLD_SOCKETS_ENABLED
	if (theApp.serverconnect->IsConnected()) 
	{
		if(!theApp.serverconnect->IsLowID())
			sConnected = _GetPlainResString(IDS_CONNECTED);
		else
			sConnected = _GetPlainResString(IDS_CONNECTED) + _T(" (") + _GetPlainResString(IDS_LOWID) + _T(")");

		CServer* cur_server = theApp.serverconnect->GetCurrentServer();
		sConnected += _T(": ") + CString(cur_server->GetListName());

		_stprintf(HTTPTempC, _T("%i "), cur_server->GetUsers());
		sConnected += _T(" [") + CString(HTTPTempC) + _GetPlainResString(IDS_LUSERS) + _T("]");

	} 
	else if (theApp.serverconnect->IsConnecting())
	{
		sConnected = _GetPlainResString(IDS_CONNECTING);
	}
	else
#endif //OLD_SOCKETS_ENABLED
	{
		sConnected = _GetPlainResString(IDS_DISCONNECTED);
		if (IsSessionAdmin(Data,sSession)) sConnected+=_T(" (<small><a href=\"/?ses=") + sSession + _T("&w=server&c=connect\">")+_GetPlainResString(IDS_CONNECTTOANYSERVER)+_T("</a></small>)");
	}
	Out.Replace(_T("[Connected]"), _T("<b>")+_GetPlainResString(IDS_PW_CONNECTION)+_T(":</b> ")+sConnected);
    _stprintf(HTTPTempC, _GetPlainResString(IDS_UPDOWN),(float)theApp.uploadqueue->GetDatarate()/1024,(float)theApp.downloadqueue->GetDatarate()/1024);
	CString sLimits;
	// EC 25-12-2002
	CString MaxUpload;
	CString MaxDownload;
	MaxUpload.Format(_T("%i"),theApp.glob_prefs->GetMaxUpload());
	MaxDownload.Format(_T("%i"),theApp.glob_prefs->GetMaxDownload());
	if (MaxUpload == "65535")  MaxUpload = GetResString(IDS_PW_UNLIMITED);
	if (MaxDownload == "65535") MaxDownload = GetResString(IDS_PW_UNLIMITED);
	sLimits.Format(_T("%s/%s"), MaxUpload, MaxDownload);
	// EC Ends
	Out.Replace(_T("[Speed]"), _T("<b>")+_GetPlainResString(IDS_DL_SPEED)+_T(":</b> ")+CString(HTTPTempC) + _T("<small> (") + _GetPlainResString(IDS_PW_CON_LIMIT) + _T(": ") + sLimits + _T(")</small>"));

	return Out;

	EMULE_CATCH

	return _T("");
}

CString CWebServer::_GetFooter(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return _T("");

	return pThis->m_Templates.sFooter;

	EMULE_CATCH

	return _T("");
}

CString CWebServer::_GetServerList(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return _T("");

	CString HTTPTemp;

	CString sSession = _ParseURL(Data.sURL, _T("ses"));
	bool bAdmin = IsSessionAdmin(Data,sSession);
	CString sAddServerBox = _T("");

	CString sCmd = _ParseURL(Data.sURL, _T("c"));
	CString sIP = _ParseURL(Data.sURL, _T("ip"));
	int nPort = _tstoi(_ParseURL(Data.sURL, _T("port")));
	if (bAdmin &&  sCmd.CompareNoCase(_T("connect")) == 0)
	{
		if(sIP.IsEmpty())
			theApp.emuledlg->SendMessage(WM_COMMAND, MP_CONNECT);
		else
			_ConnectToServer(sIP, nPort);
	}	
	else if (bAdmin && sCmd.CompareNoCase(_T("disconnect")) == 0) 
	{
		theApp.emuledlg->SendMessage(WM_COMMAND, MP_DISCONNECT);
	}	
	else if (bAdmin && sCmd.CompareNoCase(_T("remove")) == 0) 
	{
		if(!sIP.IsEmpty())
			_RemoveServer(sIP, nPort);
	}	
	else if (bAdmin && sCmd.CollateNoCase(_T("addtostatic")) == 0) 
	{
		if(!sIP.IsEmpty())
			_AddToStatic(sIP, nPort);
	}
	else if (bAdmin && sCmd.CompareNoCase(_T("removefromstatic")) == 0) 
	{
		if(!sIP.IsEmpty())
			_RemoveFromStatic(sIP, nPort);
	}
	else if (bAdmin && sCmd.CompareNoCase(_T("prioup")) == 0) ////
	{
		if(!sIP.IsEmpty())
			_ServerPrioUp(sIP, nPort);
	}
	else if (bAdmin && sCmd.CompareNoCase(_T("priodown")) == 0) ////
	{
		if(!sIP.IsEmpty())
			_ServerPrioDown(sIP, nPort);
	}
	else if (sCmd.CompareNoCase(_T("options")) == 0) 
	{
		sAddServerBox = _GetAddServerBox(Data);
	}

	CString sSort = _ParseURL(Data.sURL, _T("sort"));
	if (sSort != "") 
	{
		if(sSort == "name")
			pThis->m_Params.ServerSort = SERVER_SORT_NAME;
		else
		if(sSort == "description")
			pThis->m_Params.ServerSort = SERVER_SORT_DESCRIPTION;
		else
		if(sSort == "ip")
			pThis->m_Params.ServerSort = SERVER_SORT_IP;
		else
		if(sSort == "users")
			pThis->m_Params.ServerSort = SERVER_SORT_USERS;
		else
		if(sSort == "files")
			pThis->m_Params.ServerSort = SERVER_SORT_FILES;
		else
		if(sSort == "priority")
			pThis->m_Params.ServerSort = SERVER_SORT_PRIORITY;
	}
	sSort.SetString(_ParseURL(Data.sURL, _T("sortreverse")));
	if (sSort.IsEmpty()) {
		pThis->m_Params.bServerSortReverse = false;
	} else {
		pThis->m_Params.bServerSortReverse = (sSort.CompareNoCase(_T("true")) == 0);
	}

	CString sServerSortRev;
	if(pThis->m_Params.bServerSortReverse)
		sServerSortRev = "false";
	else
		sServerSortRev = "true";

	CString Out = pThis->m_Templates.sServerList;
	Out.Replace(_T("[ConnectedServerData]"), _GetConnectedServer(Data));
    Out.Replace(_T("[AddServerBox]"), sAddServerBox);
	Out.Replace(_T("[Session]"), sSession);
	if(pThis->m_Params.ServerSort == SERVER_SORT_NAME)
		Out.Replace(_T("[SortName]"), _T("&sortreverse=") + sServerSortRev);
	else
		Out.Replace(_T("[SortName]"), _T(""));
	if(pThis->m_Params.ServerSort == SERVER_SORT_DESCRIPTION)
		Out.Replace(_T("[SortDescription]"), _T("&sortreverse=") + sServerSortRev);
	else
		Out.Replace(_T("[SortDescription]"), _T(""));
	if(pThis->m_Params.ServerSort == SERVER_SORT_IP)
		Out.Replace(_T("[SortIP]"), _T("&sortreverse=") + sServerSortRev);
	else
		Out.Replace(_T("[SortIP]"), _T(""));
	if(pThis->m_Params.ServerSort == SERVER_SORT_USERS)
		Out.Replace(_T("[SortUsers]"), _T("&sortreverse=") + sServerSortRev);
	else
		Out.Replace(_T("[SortUsers]"), _T(""));
	if(pThis->m_Params.ServerSort == SERVER_SORT_FILES)
		Out.Replace(_T("[SortFiles]"), _T("&sortreverse=") + sServerSortRev);
	else
		Out.Replace(_T("[SortFiles]"), _T(""));
	if(pThis->m_Params.ServerSort == SERVER_SORT_PRIORITY)
		Out.Replace(_T("[SortPriority]"), _T("&sortreverse=") + sServerSortRev);
	else
		Out.Replace(_T("[SortPriority]"), _T(""));
	Out.Replace(_T("[ServerList]"), _GetPlainResString(IDS_SV_SERVERLIST));
	Out.Replace(_T("[Servername]"), _GetPlainResString(IDS_SL_SERVERNAME));
	Out.Replace(_T("[Description]"), _GetPlainResString(IDS_DESCRIPTION));
	Out.Replace(_T("[Address]"), _GetPlainResString(IDS_IP));
	Out.Replace(_T("[Users]"), _GetPlainResString(IDS_LUSERS));
	Out.Replace(_T("[Files]"), _GetPlainResString(IDS_LFILES));
	Out.Replace(_T("[Priority]"), _GetPlainResString(IDS_PRIORITY)); ////
	Out.Replace(_T("[Actions]"), _GetPlainResString(IDS_WEB_ACTIONS));

	CString OutE = pThis->m_Templates.sServerLine;

	CArray<ServerEntry, ServerEntry> ServerArray;

	CServer* tempServer;
	// Populating array
	for (uint32 sc=0;sc<theApp.serverlist->GetServerCount();sc++)
	{
		CServer* cur_file = theApp.serverlist->GetServerAt(sc);
		ServerEntry Entry;
		Entry.sServerName = _SpecialChars(cur_file->GetListName());
		Entry.sServerDescription = _SpecialChars(cur_file->GetDescription());
		Entry.nServerPort = cur_file->GetPort();
		Entry.sServerIP = cur_file->GetAddress();
        Entry.nServerUsers = cur_file->GetUsers();
		Entry.nServerMaxUsers = cur_file->GetMaxUsers();
		Entry.nServerFiles = cur_file->GetFiles();
		Entry.bServerStatic = cur_file->IsStaticMember();	// Purity (Add Remove from Static Server List in WebServer)

		int counter=0;
		CString temp="";
		CString newip="";
		for(int j=0; j<4; j++) {  
			temp = Entry.sServerIP.Tokenize(_T("."),counter); 
			if (temp.GetLength() == 1) newip.AppendFormat("00" + temp); 
			else if (temp.GetLength() == 2) newip.AppendFormat("0" + temp); 
			else if (temp.GetLength() == 3) newip.AppendFormat("" + temp);
        }
		Entry.sServerFullIP = newip;

		if(cur_file->GetPreferences()== CServer::ePR_HIGH) {
			Entry.sServerPriority = _GetPlainResString(IDS_PRIOHIGH);
			Entry.nServerPriority = 2;
		} else if(cur_file->GetPreferences()== CServer::ePR_NORMAL) {
			Entry.sServerPriority = _GetPlainResString(IDS_PRIONORMAL);
			Entry.nServerPriority = 1;
		} else if(cur_file->GetPreferences()== CServer::ePR_LOW) {
			Entry.sServerPriority = _GetPlainResString(IDS_PRIOLOW);
			Entry.nServerPriority = 0;
		}
		if (theApp.serverconnect->IsConnecting()) { 
			tempServer = theApp.serverconnect->GetConnectingServer();
			if (tempServer->GetFullIP() == cur_file->GetFullIP())
				Entry.sServerState = "connecting";
			else
				Entry.sServerState = "disconnected";
		}
		else if (theApp.serverconnect->IsConnected()) {
			tempServer = theApp.serverconnect->GetCurrentServer();
			if (tempServer->GetFullIP() == cur_file->GetFullIP()) {  
				if (!theApp.serverconnect->IsLowID())  
					Entry.sServerState = "high";  
				else  
					Entry.sServerState = "low";
			} else
				Entry.sServerState = "disconnected";
		} // end IsConnected
		else
			Entry.sServerState = "disconnected";

		ServerArray.Add(Entry);
	}
	// Sorting (simple bubble sort, we don't have tons of data here)
	bool bSorted = true;
	for(int nMax = 0;bSorted && nMax < ServerArray.GetCount()*2; nMax++)
	{
		bSorted = false;
		for(int i = 0; i < ServerArray.GetCount() - 1; i++)
		{
			bool bSwap = false;
			switch(pThis->m_Params.ServerSort)
			{
			case SERVER_SORT_NAME:
				bSwap = ServerArray[i].sServerName.CompareNoCase(ServerArray[i+1].sServerName) > 0;
				break;
			case SERVER_SORT_DESCRIPTION:
				bSwap = ServerArray[i].sServerDescription.CompareNoCase(ServerArray[i+1].sServerDescription) < 0;
				break;
			case SERVER_SORT_IP:
				bSwap = ServerArray[i].sServerFullIP < ServerArray[i+1].sServerFullIP;
				break;
			case SERVER_SORT_USERS:
				bSwap = ServerArray[i].nServerUsers < ServerArray[i+1].nServerUsers;
				break;
			case SERVER_SORT_FILES:
				bSwap = ServerArray[i].nServerFiles < ServerArray[i+1].nServerFiles;
				break;
			case SERVER_SORT_PRIORITY:
			bSwap = ServerArray[i].nServerPriority < ServerArray[i+1].nServerPriority;
				break;
			}
			if(pThis->m_Params.bServerSortReverse)
			{
				bSwap = !bSwap;
			}
			if(bSwap)
			{
				bSorted = true;
				ServerEntry TmpEntry = ServerArray[i];
				ServerArray[i] = ServerArray[i+1];
				ServerArray[i+1] = TmpEntry;
			}
		}
	}

	// Displaying
	CString sList = _T("");
	for(int i = 0; i < ServerArray.GetCount(); i++)
	{
		CString HTTPProcessData = OutE;	// Copy Entry Line to Temp

		CString admin = "";		//admin rights
		CString ed2k = "";		//ed2k
		CString session = "";	//session
		CString ip = "";		//ip
		CString port = "";		//port
		CString isstatic = "";	//serverstatic

		CString sServerPort;
		sServerPort.Format(_T("%d"), ServerArray[i].nServerPort);
		ed2k.Format(CString(_T("ed2k://|server|") + ServerArray[i].sServerIP + "|" + sServerPort + _T("|/")));

		if (ServerArray[i].sServerIP == _ParseURL(Data.sURL,_T("ip")) && sServerPort == _ParseURL(Data.sURL,_T("port")))
            HTTPProcessData.Replace(_T("[LastChangedDataset]"), "checked");
		else
            HTTPProcessData.Replace(_T("[LastChangedDataset]"), "checked_no");

		if (bAdmin)
			admin.Format(_T("admin"));

		session.Format(CString(sSession));
		ip.Format(CString(ServerArray[i].sServerIP));
		port.Format(CString(sServerPort));

		// DonGato: static server showing
		if (ServerArray[i].bServerStatic)
		{
			isstatic.Format(_T("staticsrv"));
			HTTPProcessData.Replace(_T("[ServerType]"), _T("static"));
		}
		else
			HTTPProcessData.Replace(_T("[ServerType]"), _T("none"));
		
		HTTPProcessData.Replace(_T("[admin]"), admin);
		HTTPProcessData.Replace(_T("[ed2k]"), ed2k);
		HTTPProcessData.Replace(_T("[session]"), session);
		HTTPProcessData.Replace(_T("[ip]"), ip);
		HTTPProcessData.Replace(_T("[port]"), port);
		HTTPProcessData.Replace(_T("[isstatic]"), isstatic);

		// DonGato: reduced large servernames or descriptions
		if(ServerArray[i].sServerName.GetLength() > SHORT_LENGTH)
			HTTPProcessData.Replace(_T("[1]"), _T("<acronym title=\"") + ServerArray[i].sServerName + _T("\">") + ServerArray[i].sServerName.Left(SHORT_LENGTH) + _T("...") + _T("</acronym>"));
		else
			HTTPProcessData.Replace(_T("[1]"), ServerArray[i].sServerName);
		if(ServerArray[i].sServerDescription.GetLength() > SHORT_LENGTH)
			HTTPProcessData.Replace(_T("[2]"), _T("<acronym title=\"") + ServerArray[i].sServerDescription + _T("\">") + ServerArray[i].sServerDescription.Left(SHORT_LENGTH) + _T("...") + _T("</acronym>"));
		else
			HTTPProcessData.Replace(_T("[2]"), ServerArray[i].sServerDescription);
		// DonGato
		CString sPort; sPort.Format(_T(":%d"), ServerArray[i].nServerPort);
		HTTPProcessData.Replace(_T("[3]"), ServerArray[i].sServerIP + sPort);

		CString sT;
		if(ServerArray[i].nServerUsers > 0)
		{
			if(ServerArray[i].nServerMaxUsers > 0)
				sT.Format(_T("%d (%d)"), ServerArray[i].nServerUsers, ServerArray[i].nServerMaxUsers);
			else
				sT.Format(_T("%d"), ServerArray[i].nServerUsers);
		}
		HTTPProcessData.Replace(_T("[4]"), sT);
		sT = _T("");
		if(ServerArray[i].nServerFiles > 0)
			sT.Format(_T("%d"), ServerArray[i].nServerFiles);
		HTTPProcessData.Replace(_T("[5]"), sT);
		HTTPProcessData.Replace(_T("[Priority]"), ServerArray[i].sServerPriority); ////
		HTTPProcessData.Replace(_T("[ServerState]"), ServerArray[i].sServerState);
		sList += HTTPProcessData;
	}
	Out.Replace(_T("[ServersList]"), sList);

	return Out;

	EMULE_CATCH

	return _T("");
}

CString CWebServer::_GetTransferList(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return _T("");

	CString HTTPTemp;

	CString sSession = _ParseURL(Data.sURL, _T("ses"));
	bool bAdmin = IsSessionAdmin(Data,sSession);	//SyruS cached (used often here)
	CString Out = _T("");

	//SyruS show completed files (0.28b) plus clear completed
	CString sClear(_ParseURL(Data.sURL, _T("clearcompleted")));
	if (bAdmin && !sClear.IsEmpty()) 
	{
		if (sClear.CompareNoCase(_T("all")) == 0) {
			theApp.emuledlg->transferwnd.downloadlistctrl.ClearCompleted();
		} else {
			uchar FileHash[16];
			if (_GetFileHash(sClear, FileHash))	//failsafe
				theApp.emuledlg->transferwnd.downloadlistctrl.ClearCompleted(FileHash);
		}
	}

	HTTPTemp = _ParseURL(Data.sURL, _T("c"));

	if (bAdmin && !HTTPTemp.IsEmpty()) 
		theApp.emuledlg->searchwnd.AddEd2kLinksToDownload(HTTPTemp);
	else 
		HTTPTemp.Empty();	//free for reusing

	CString sOp(_ParseURL(Data.sURL, _T("op")));
	CString sFile(_ParseURL(Data.sURL, _T("file")));
	uchar FileHash[16];
	if (!sOp.IsEmpty() && !sFile.IsEmpty() && /*failsafe*/ _GetFileHash(sFile, FileHash))
	{
 		//unsafe _GetFileHash(_ParseURL(Data.sURL, _T("file")), FileHash);
		CPartFile* found_file = theApp.downloadqueue->GetFileByID(FileHash);
		if(found_file)
		{	// SyruS all actions require a found file (removed double-check inside)
			if(bAdmin && sOp.CompareNoCase(_T("a4af")) == 0) // DonGato: added A4ADF action
			{
				found_file->DownloadAllA4AF();
			}
			else if(bAdmin && sOp.CompareNoCase(_T("autoa4af")) == 0)
			{
				theApp.downloadqueue->SetA4AFAutoFile(found_file); // Purity: Auto-A4AF
			}
			else if(bAdmin && sOp.CompareNoCase(_T("noautoa4af")) == 0)
			{
				theApp.downloadqueue->SetA4AFAutoFile(NULL); // Purity: Auto-A4AF
			}
			else if(bAdmin && sOp.CompareNoCase(_T("stop")) == 0)
			{
				found_file->StopFile();
			}
			else if(bAdmin && sOp.CompareNoCase(_T("resume")) == 0)
			{
				found_file->ResumeFile();
			}
			else if(bAdmin && sOp.CompareNoCase(_T("cancel")) == 0)
			{
				found_file->DeleteFile();
			}
			else if(bAdmin && sOp.CompareNoCase(_T("prioup")) == 0)
			{
				if (!found_file->IsAutoPrioritized())
					switch (found_file->GetPriority()) {
						case PR_LOW: found_file->SetAutoPriority(false); found_file->SetPriority(PR_NORMAL);break;
						case PR_NORMAL: found_file->SetAutoPriority(false); found_file->SetPriority(PR_HIGH);break;
						case PR_HIGH: found_file->SetAutoPriority(true); found_file->SetPriority(PR_HIGH);break;
					}
				else {found_file->SetAutoPriority(false); found_file->SetPriority(PR_LOW);}
			}
			else if(bAdmin && sOp.CompareNoCase(_T("priodown")) == 0)
			{
				if (!found_file->IsAutoPrioritized())
					switch (found_file->GetPriority()) {
						case PR_LOW: found_file->SetAutoPriority(true); found_file->SetPriority(PR_HIGH);break;
						case PR_NORMAL: found_file->SetAutoPriority(false); found_file->SetPriority(PR_LOW);break;
						case PR_HIGH: found_file->SetAutoPriority(false); found_file->SetPriority(PR_NORMAL);break;
					}
				else {found_file->SetAutoPriority(false); found_file->SetPriority(PR_HIGH);}
			}
			else if(bAdmin && sOp.CompareNoCase(_T("compressed")) == 0) // Purity: DropSCB
			{
				found_file->SetDiscardSuperCompressed( !found_file->GetDiscardSuperCompressed() );
			}
			else if(bAdmin && sOp.CompareNoCase(_T("noneededsources")) == 0) // Purity: DropNNS
			{
				found_file->RemoveNoNeededSources();
			}
		}	//if(found_file) pre-check
	}

	CString sSort(_ParseURL(Data.sURL, _T("sort")));
	if (!sSort.IsEmpty()) 
	{
		if(sSort.CompareNoCase(_T("name")) == 0)
			pThis->m_Params.DownloadSort = DOWN_SORT_NAME;
		else
		if(sSort.CompareNoCase(_T("size")) == 0)
			pThis->m_Params.DownloadSort = DOWN_SORT_SIZE;
		else
		if(sSort.CompareNoCase(_T("transferred")) == 0)
			pThis->m_Params.DownloadSort = DOWN_SORT_TRANSFERRED;
		else
		if(sSort.CompareNoCase(_T("speed")) == 0)
			pThis->m_Params.DownloadSort = DOWN_SORT_SPEED;
		else
		if(sSort.CompareNoCase(_T("progress")) == 0)
			pThis->m_Params.DownloadSort = DOWN_SORT_PROGRESS;		
		else
		if(sSort.CompareNoCase(_T("sources")) == 0)
			pThis->m_Params.DownloadSort = DOWN_SORT_SOURCES;
		else
		if(sSort.CompareNoCase(_T("priority")) == 0)
			pThis->m_Params.DownloadSort = DOWN_SORT_PRIORITY;

	}
	sSort.SetString(_ParseURL(Data.sURL, _T("sortreverse")));
	if (sSort.IsEmpty()) {
		pThis->m_Params.bDownloadSortReverse = false;
	} else {
		pThis->m_Params.bDownloadSortReverse = (sSort.CompareNoCase(_T("true")) == 0);
	}

	HTTPTemp.SetString(_ParseURL(Data.sURL, _T("showuploadqueue")));
	if (HTTPTemp.CompareNoCase(_T("true")) == 0)
		pThis->m_Params.bShowUploadQueue = true;
	else if (HTTPTemp.CompareNoCase(_T("false")) == 0)
		pThis->m_Params.bShowUploadQueue = false;

	HTTPTemp.SetString(_ParseURL(Data.sURL, _T("showuploadqueuebanned")));
	if (HTTPTemp.CompareNoCase(_T("true")) == 0)
		pThis->m_Params.bShowUploadQueueBanned = true;
	else if (HTTPTemp.CompareNoCase(_T("false")) == 0)
		pThis->m_Params.bShowUploadQueueBanned = false;

	HTTPTemp.SetString(_ParseURL(Data.sURL, _T("showuploadqueuefriend")));
	if (HTTPTemp.CompareNoCase(_T("true")) == 0)
		pThis->m_Params.bShowUploadQueueFriend = true;
	else if (HTTPTemp.CompareNoCase(_T("false")) == 0)
		pThis->m_Params.bShowUploadQueueFriend = false;

	HTTPTemp.SetString(_ParseURL(Data.sURL, _T("showuploadqueuecredit")));
	if (HTTPTemp.CompareNoCase(_T("true")) == 0)
		pThis->m_Params.bShowUploadQueueCredit = true;
	else if (HTTPTemp.CompareNoCase(_T("false")) == 0)
		pThis->m_Params.bShowUploadQueueCredit = false;

	CString sDownloadSortRev;
	if(pThis->m_Params.bDownloadSortReverse)
		sDownloadSortRev = "false";
	else
		sDownloadSortRev = "true";

	Out += pThis->m_Templates.sTransferImages;
	Out += pThis->m_Templates.sTransferList;
	Out.Replace(_T("[DownloadHeader]"), pThis->m_Templates.sTransferDownHeader);
	Out.Replace(_T("[DownloadFooter]"), pThis->m_Templates.sTransferDownFooter);
	Out.Replace(_T("[UploadHeader]"), pThis->m_Templates.sTransferUpHeader);
	Out.Replace(_T("[UploadFooter]"), pThis->m_Templates.sTransferUpFooter);
	Out.Replace(_T("[Session]"), sSession);
	if(pThis->m_Params.DownloadSort == DOWN_SORT_NAME)
		Out.Replace(_T("[SortName]"), _T("&sortreverse=") + sDownloadSortRev);
	else
		Out.Replace(_T("[SortName]"), _T(""));
	if(pThis->m_Params.DownloadSort == DOWN_SORT_SIZE)
		Out.Replace(_T("[SortSize]"), _T("&sortreverse=") + sDownloadSortRev);
	else
		Out.Replace(_T("[SortSize]"), _T(""));
	if(pThis->m_Params.DownloadSort == DOWN_SORT_TRANSFERRED)
		Out.Replace(_T("[SortTransferred]"), _T("&sortreverse=") + sDownloadSortRev);
	else
		Out.Replace(_T("[SortTransferred]"), _T(""));
	if(pThis->m_Params.DownloadSort == DOWN_SORT_SPEED)
		Out.Replace(_T("[SortSpeed]"), _T("&sortreverse=") + sDownloadSortRev);
	else
		Out.Replace(_T("[SortSpeed]"), _T(""));
	if(pThis->m_Params.DownloadSort == DOWN_SORT_PROGRESS)
		Out.Replace(_T("[SortProgress]"), _T("&sortreverse=") + sDownloadSortRev);
	else
		Out.Replace(_T("[SortProgress]"), _T(""));
	if(pThis->m_Params.DownloadSort == DOWN_SORT_SOURCES)
		Out.Replace(_T("[SortSources]"), _T("&sortreverse=") + sDownloadSortRev);
	else
		Out.Replace(_T("[SortSources]"), _T(""));
	if(pThis->m_Params.DownloadSort == DOWN_SORT_PRIORITY)
		Out.Replace(_T("[SortPriority]"), _T("&sortreverse=") + sDownloadSortRev);
	else
		Out.Replace(_T("[SortPriority]"), _T(""));

	Out.Replace(_T("[DownloadList]"), _GetPlainResString(IDS_ST_ACTIVEDOWNLOAD));
	Out.Replace(_T("[Filename]"), _GetPlainResString(IDS_DL_FILENAME));
	Out.Replace(_T("[Size]"), _GetPlainResString(IDS_DL_SIZE));
	Out.Replace(_T("[Transferred]"), _GetPlainResString(IDS_COMPLETE));
	Out.Replace(_T("[Progress]"), _GetPlainResString(IDS_DL_PROGRESS));
	Out.Replace(_T("[Speed]"), _GetPlainResString(IDS_DL_SPEED));
	Out.Replace(_T("[Sources]"), _GetPlainResString(IDS_DL_SOURCES));
	Out.Replace(_T("[Actions]"), _GetPlainResString(IDS_WEB_ACTIONS));
	Out.Replace(_T("[UploadList]"), _GetPlainResString(IDS_ST_ACTIVEUPLOAD));
	Out.Replace(_T("[User]"), _GetPlainResString(IDS_QL_USERNAME));
	Out.Replace(_T("[Version]"), _GetPlainResString(IDS_INFLST_USER_CLIENTVERSION));
	Out.Replace(_T("[TotalDown]"), _GetPlainResString(IDS_INFLST_USER_TOTALDOWNLOAD));
	Out.Replace(_T("[TotalUp]"), _GetPlainResString(IDS_INFLST_USER_TOTALUPLOAD));
	Out.Replace(_T("[Prio]"), _GetPlainResString(IDS_PRIORITY));

	CString OutE = pThis->m_Templates.sTransferDownLine;

	float fTotalSize = 0, fTotalTransferred = 0, fTotalSpeed = 0;
	CArray<DownloadFiles, DownloadFiles> FilesArray;
	//SyruS show completed files (0.28b)
	CArray<CPartFile*,CPartFile*> partlist;
	if (!theApp.emuledlg->transferwnd.downloadlistctrl.GetDisplayedFiles(&partlist))	//failsafe
	{
		partlist.RemoveAll();
		// Populating array
		for (POSITION pos=theApp.downloadqueue->filelist.GetHeadPosition(); pos != 0; theApp.downloadqueue->filelist.GetNext(pos))
		{
			CPartFile* cur_file = theApp.downloadqueue->filelist.GetAt(pos);
			partlist.Add(cur_file);
		}
	}
	for (int i=0; i<partlist.GetCount(); i++) {
		CPartFile* cur_file=partlist.GetAt(i);
		if (cur_file) {
			DownloadFiles dFile;
			dFile.sFileName = _SpecialChars(cur_file->GetFileName());
			dFile.sFileNameJS = _SpecialChars(cur_file->GetFileName(), true);	//for javascript
			dFile.lFileSize = cur_file->GetFileSize();
			dFile.lFileTransferred = cur_file->GetCompletedSize();
			dFile.fCompleted = cur_file->GetPercentCompleted();
			dFile.lFileSpeed = cur_file->GetDatarate();
			dFile.nFileStatus = cur_file->GetStatus();
			dFile.sFileStatus = cur_file->GetPartfileStatus();
			if (cur_file->IsAutoPrioritized())
			{
				cur_file->UpdateDownloadAutoPriority();
				dFile.nFilePrio = cur_file->GetPriority();
				dFile.nFilePrio+=10;
			}
			else
			{
				dFile.nFilePrio = cur_file->GetPriority();
			}
			dFile.sFileHash = EncodeBase16(cur_file->GetFileHash(), 16);
			dFile.lSourceCount = cur_file->GetSourceCount();
			dFile.lNotCurrentSourceCount = cur_file->GetNotCurrentSourcesCount();
			dFile.lTransferringSourceCount = cur_file->GetTransferingSrcCount();
			dFile.bCompress = cur_file->GetDiscardSuperCompressed(); // Purity: DropSCB
#ifdef OLD_SOCKETS_ENABLED
			if (theApp.serverconnect->IsConnected() && !theApp.serverconnect->IsLowID())
				dFile.sED2kLink = cur_file->CreateED2kSourceLink();
			else
#endif //OLD_SOCKETS_ENABLED
				dFile.sED2kLink = cur_file->CreateED2kLink();
			dFile.sFileInfo = _SpecialChars(cur_file->GetDownloadFileInfo());

			FilesArray.Add(dFile);
		}
	}

	// Sorting (simple bubble sort, we don't have tons of data here)
	bool bSorted = true;
	for(int nMax = 0;bSorted && nMax < FilesArray.GetCount()*2; nMax++)
	{
		bSorted = false;
		for(int i = 0; i < FilesArray.GetCount() - 1; i++)
		{
			bool bSwap = false;
			switch(pThis->m_Params.DownloadSort)
			{
			case DOWN_SORT_NAME:
				bSwap = FilesArray[i].sFileName.CompareNoCase(FilesArray[i+1].sFileName) > 0;
				break;
			case DOWN_SORT_SIZE:
				bSwap = FilesArray[i].lFileSize < FilesArray[i+1].lFileSize;
				break;
			case DOWN_SORT_TRANSFERRED:
				bSwap = FilesArray[i].lFileTransferred < FilesArray[i+1].lFileTransferred;
				break;
			case DOWN_SORT_SPEED:
				bSwap = FilesArray[i].lFileSpeed < FilesArray[i+1].lFileSpeed;
				break;
			case DOWN_SORT_PROGRESS:
				bSwap = FilesArray[i].fCompleted  < FilesArray[i+1].fCompleted ;
				break;
			case DOWN_SORT_SOURCES:
				bSwap = FilesArray[i].lSourceCount  < FilesArray[i+1].lSourceCount ;
				break;
			case DOWN_SORT_PRIORITY:
				bSwap = FilesArray[i].nFilePrio < FilesArray[i+1].nFilePrio ;
				break;
			}
			if(pThis->m_Params.bDownloadSortReverse)
			{
				bSwap = !bSwap;
			}
			if(bSwap)
			{
				bSorted = true;
				DownloadFiles TmpFile = FilesArray[i];
				FilesArray[i] = FilesArray[i+1];
				FilesArray[i+1] = TmpFile;
			}
		}
	}

	uchar FileHashA4AF[16]; // Purity: Auto-A4AF

	// Displaying
	CString sDownList = _T("");
	for(int i = 0; i < FilesArray.GetCount(); i++)
	{
		CString HTTPProcessData;
		HTTPProcessData = OutE;

		if (FilesArray[i].sFileHash == _ParseURL(Data.sURL,_T("file")) )
            HTTPProcessData.Replace(_T("[LastChangedDataset]"), "checked");
		else
            HTTPProcessData.Replace(_T("[LastChangedDataset]"), "checked_no");

		CString admin = "";		//admin rights
		CString finfo = "";		//fileinfo
		CString ed2k = "";		//ed2klink
		CString state = "";		//state
		CString scb = "";		//dropscb
		CString autoa4af = "";	//autoa4af
		CString fname = "";		//filename
		CString session = "";	//session
		CString filehash = "";	//filehash

		bool bIsComplete = false;

		// Purity: Auto-A4AF
		_GetFileHash(FilesArray[i].sFileHash, FileHashA4AF);
		CPartFile* found_file = theApp.downloadqueue->GetFileByID(FileHashA4AF);

		if(FilesArray[i].lFileSpeed > 0)
			state.Format(_T("downloading"));
		else
			state.Format(_T("waiting"));

		switch(FilesArray[i].nFileStatus)
		{
			case PS_HASHING: 
			case PS_WAITINGFORHASH:
				state.Format(_T("hashing"));
				break;
			case PS_ERROR:
				state.Format(_T("error"));
				break;
			case PS_COMPLETING:
				state.Format(_T("completing"));
				break;
			case PS_COMPLETE:
				bIsComplete = true;
				state.Format(_T("complete"));
			case PS_STOPPED:	
			case PS_PAUSED:
				if(!bIsComplete)
					state.Format(_T("paused"));
			default:
				{
					CString JSfileinfo=FilesArray[i].sFileInfo;
					JSfileinfo.Replace(_T("\n"),_T("\\n"));
					JSfileinfo.Replace(_T("%"),_T("%%"));
					JSfileinfo.Replace(_T("'"),_T("&#8217;")); // EC 22/5 - Chararacter ' causes a problem in javascript:alert()

					CString JSED2kLink=FilesArray[i].sED2kLink;
					JSED2kLink.Replace(_T("'"),_T("&#8217;"));

					finfo.Format(JSfileinfo);
					ed2k.Format(JSED2kLink);
					fname.Format(FilesArray[i].sFileNameJS);
					session.Format(CString(sSession));
					filehash.Format(FilesArray[i].sFileHash);

					if (FilesArray[i].bCompress && !bIsComplete)
						scb.Format(_T("scb"));
					
					if (theApp.downloadqueue->GetA4AFAutoFile() == found_file && !bIsComplete)
						autoa4af.Format(_T("autoa4af"));
				}
				break;
		}	//switch

		if (bAdmin)
			admin.Format(_T("admin"));

		HTTPProcessData.Replace(_T("[admin]"), admin);
		HTTPProcessData.Replace(_T("[finfo]"), finfo);
		HTTPProcessData.Replace(_T("[ed2k]"), ed2k);
		HTTPProcessData.Replace(_T("[DownState]"), state);
		HTTPProcessData.Replace(_T("[scb]"), scb);
		HTTPProcessData.Replace(_T("[autoa4af]"), autoa4af);
		HTTPProcessData.Replace(_T("[fname]"), fname);
		HTTPProcessData.Replace(_T("[session]"), session);
		HTTPProcessData.Replace(_T("[filehash]"), filehash);

		// Purity: showing status for files (A4AF & SCB)
		if (FilesArray[i].bCompress && !bIsComplete)
			HTTPProcessData.Replace(_T("[FileIsSCB]"), _T("scb"));
		else
			HTTPProcessData.Replace(_T("[FileIsSCB]"), _T("halfnone"));

		if (theApp.downloadqueue->GetA4AFAutoFile() == found_file && !bIsComplete)
			HTTPProcessData.Replace(_T("[FileIsA4AF]"), _T("a4af"));
		else
			HTTPProcessData.Replace(_T("[FileIsA4AF]"), _T("halfnone"));

		if(FilesArray[i].sFileName.GetLength() > SHORT_LENGTH_MAX)
			HTTPProcessData.Replace(_T("[ShortFileName]"), FilesArray[i].sFileName.Left(SHORT_LENGTH_MAX) + _T("..."));
		else
			HTTPProcessData.Replace(_T("[ShortFileName]"), FilesArray[i].sFileName);

		CString sTooltip = FilesArray[i].sFileInfo;
		sTooltip.Replace(_T("\n"), _T("<br>"));
		sTooltip.Replace(_T("'"), _T("&#8217;"));
		HTTPProcessData.Replace(_T("[FileInfo]"), sTooltip);

		fTotalSize += FilesArray[i].lFileSize;

		HTTPProcessData.Replace(_T("[2]"), CastItoXBytes(FilesArray[i].lFileSize));

		if(FilesArray[i].lFileTransferred > 0)
		{
			fTotalTransferred += FilesArray[i].lFileTransferred;
			HTTPProcessData.Replace(_T("[3]"), CastItoXBytes(FilesArray[i].lFileTransferred));
		}
		else
			HTTPProcessData.Replace(_T("[3]"), _T("-"));

		HTTPProcessData.Replace(_T("[DownloadBar]"), _GetDownloadGraph(Data,FilesArray[i].sFileHash));

		HTTPTemp = "";

		if(FilesArray[i].lFileSpeed > 0.0f)
		{
			fTotalSpeed += FilesArray[i].lFileSpeed;
			HTTPTemp.Format(_T("%8.2f %s"), FilesArray[i].lFileSpeed/1024.0 ,_GetPlainResString(IDS_KBYTESEC) );
			HTTPProcessData.Replace(_T("[4]"), HTTPTemp);
		}
		else
			HTTPProcessData.Replace(_T("[4]"), _T("-"));

		if(FilesArray[i].lSourceCount > 0)
		{
			HTTPTemp.Format(_T("%i&nbsp;/&nbsp;%8i&nbsp;(%i)"),
			    FilesArray[i].lSourceCount-FilesArray[i].lNotCurrentSourceCount,
			    FilesArray[i].lSourceCount,
			    FilesArray[i].lTransferringSourceCount);
			HTTPProcessData.Replace(_T("[5]"), HTTPTemp);
		}
		else
			HTTPProcessData.Replace(_T("[5]"), _T("-"));

		switch(FilesArray[i].nFilePrio) {
			case  0: HTTPTemp=GetResString(IDS_PRIOLOW); break;
			case 10: HTTPTemp=GetResString(IDS_PRIOAUTOLOW); break;
			case  1: HTTPTemp=GetResString(IDS_PRIONORMAL); break;
			case 11: HTTPTemp=GetResString(IDS_PRIOAUTONORMAL); break;
			case  2: HTTPTemp=GetResString(IDS_PRIOHIGH); break;
			case 12: HTTPTemp=GetResString(IDS_PRIOAUTOHIGH); break;
		}
		HTTPProcessData.Replace(_T("[PrioVal]"), HTTPTemp);

		sDownList += HTTPProcessData;
	}

	Out.Replace(_T("[DownloadFilesList]"), sDownList);
    Out.Replace(_T("[PriorityUp]"), _GetPlainResString(IDS_PRIORITY_UP));
    Out.Replace(_T("[PriorityDown]"), _GetPlainResString(IDS_PRIORITY_DOWN));
	Out.Replace(_T("[TotalDownSize]"), CastItoXBytes(fTotalSize));

	Out.Replace(_T("[TotalDownTransferred]"), CastItoXBytes(fTotalTransferred));
	
	HTTPTemp.Format(_T("%8.2f %s"), fTotalSpeed/1024.0,_GetPlainResString(IDS_KBYTESEC));
	Out.Replace(_T("[TotalDownSpeed]"), HTTPTemp);
	OutE = pThis->m_Templates.sTransferUpLine;

	HTTPTemp.Format(_T("%i"),pThis->m_Templates.iProgressbarWidth);
	Out.Replace(_T("[PROGRESSBARWIDTHVAL]"),HTTPTemp);

	fTotalSize = 0;
	fTotalTransferred = 0;
	fTotalSpeed = 0;

	CString sUpList = _T("");
	for (POSITION pos = theApp.uploadqueue->uploadinglist.GetHeadPosition(); pos != 0;theApp.uploadqueue->uploadinglist.GetNext(pos))
	{
		CUpDownClient* cur_client = theApp.uploadqueue->uploadinglist.GetAt(pos);
		CString HTTPProcessData;
		HTTPProcessData = OutE;

		if (cur_client->GetDatarate() > 0)
			HTTPProcessData.Replace(_T("[UpState]"), "downloading");
		else
			HTTPProcessData.Replace(_T("[UpState]"), "waiting");

		if(cur_client->GetUserName().GetLength() > SHORT_LENGTH_MIN)
            HTTPProcessData.Replace(_T("[1]"), _SpecialChars(cur_client->GetUserName().Left(SHORT_LENGTH_MIN)) + _T("..."));
		else
			HTTPProcessData.Replace(_T("[1]"), _SpecialChars(cur_client->GetUserName()));

		CString sUpTooltip = _SpecialChars(cur_client->GetUploadFileInfo());
		sUpTooltip.Replace(_T("\n"), _T("<br>"));
		sUpTooltip.Replace(_T("'"), _T("&#8217;"));
		HTTPProcessData.Replace(_T("[FileInfo]"), sUpTooltip);

		CString sClientSoft; sClientSoft.Format(_T("%u"), cur_client->GetClientSoft());
		HTTPProcessData.Replace(_T("[ClientSoft]"), sClientSoft);
		HTTPProcessData.Replace(_T("[ClientSoftV]"), GetClientNameAndVersionString(cur_client));

		if (cur_client->IsBanned())
			HTTPProcessData.Replace(_T("[ClientExtra]"), _T("banned"));
		else if (cur_client->IsFriend())
			HTTPProcessData.Replace(_T("[ClientExtra]"), _T("friend"));
		else if (cur_client->credits->GetScoreRatio() > 1)
			HTTPProcessData.Replace(_T("[ClientExtra]"), _T("credit"));
		else
			HTTPProcessData.Replace(_T("[ClientExtra]"), _T("none"));

		CKnownFile* file = theApp.sharedfiles->GetFileByID(cur_client->reqfileid);
		if (file)
			HTTPProcessData.Replace(_T("[2]"), _SpecialChars(CString(file->GetFileName())));
		else
			HTTPProcessData.Replace(_T("[2]"), _GetPlainResString(IDS_REQ_UNKNOWNFILE));

		fTotalSize += cur_client->GetTransferedDown();
		fTotalTransferred += cur_client->GetTransferedUp();
		HTTPTemp.Format(_T("%s / %s"), CastItoXBytes(cur_client->GetTransferedDown()),CastItoXBytes(cur_client->GetTransferedUp()));
		HTTPProcessData.Replace(_T("[3]"), HTTPTemp);

		fTotalSpeed += cur_client->GetDatarate();
		HTTPTemp.Format(_T("%8.2f ") + _GetPlainResString(IDS_KBYTESEC), cur_client->GetDatarate()/1024.0);
		HTTPProcessData.Replace(_T("[4]"), HTTPTemp);

		sUpList += HTTPProcessData;
	}
	Out.Replace(_T("[UploadFilesList]"), sUpList);
	HTTPTemp.Format(_T("%s / %s"), CastItoXBytes(fTotalSize), CastItoXBytes(fTotalTransferred));
	Out.Replace(_T("[TotalUpTransferred]"), HTTPTemp);
	HTTPTemp.Format(_T("%8.2f ") + _GetPlainResString(IDS_KBYTESEC), fTotalSpeed/1024.0);
	Out.Replace(_T("[TotalUpSpeed]"), HTTPTemp);

	int nCountQueue = 0;
	int nCountQueueBanned = 0;
	int nCountQueueFriend = 0;
	int nCountQueueCredit = 0;

	for (POSITION pos = theApp.uploadqueue->waitinglist.GetHeadPosition(); pos != 0;theApp.uploadqueue->waitinglist.GetNext(pos)){
		CUpDownClient* cur_client = theApp.uploadqueue->waitinglist.GetAt(pos);
		if (cur_client->IsBanned())
			nCountQueueBanned++;
		else if (cur_client->IsFriend())
			nCountQueueFriend++;
		else if (cur_client->credits->GetScoreRatio() > 1)
			nCountQueueCredit++;
		else
			nCountQueue++;
	}

	if(pThis->m_Params.bShowUploadQueue) 
	{
		Out.Replace(_T("[UploadQueue]"), pThis->m_Templates.sTransferUpQueueShow);
		Out.Replace(_T("[UploadQueueList]"), _GetPlainResString(IDS_ONQUEUE));
		CString OutE = pThis->m_Templates.sTransferUpQueueLine;
		CString sQueue = _T("");
		for (POSITION pos = theApp.uploadqueue->waitinglist.GetHeadPosition(); pos != 0;theApp.uploadqueue->waitinglist.GetNext(pos)){
			CUpDownClient* cur_client = theApp.uploadqueue->waitinglist.GetAt(pos);
			CString HTTPProcessData;
            TCHAR HTTPTempC[100] = _T("");
			if (!(cur_client->IsBanned())&&!(cur_client->IsFriend())&&!(cur_client->credits->GetScoreRatio() > 1))
				HTTPProcessData = OutE;
			if(cur_client->GetUserName().GetLength() > SHORT_LENGTH)
				HTTPProcessData.Replace(_T("[UserName]"), _SpecialChars(cur_client->GetUserName().Left(SHORT_LENGTH)) + _T("..."));
			else
				HTTPProcessData.Replace(_T("[UserName]"), _SpecialChars(cur_client->GetUserName()));
			CKnownFile* file = theApp.sharedfiles->GetFileByID(cur_client->reqfileid);
			if (file)
				HTTPProcessData.Replace(_T("[FileName]"), _SpecialChars(file->GetFileName()));
			else
				HTTPProcessData.Replace(_T("[FileName]"), _T("?"));
			_stprintf(HTTPTempC, _T("%i") , cur_client->GetScore(false));
			HTTPProcessData.Replace(_T("[Score]"), HTTPTempC);
			CString sClientSoft; sClientSoft.Format(_T("%u"), cur_client->GetClientSoft());
			HTTPProcessData.Replace(_T("[ClientSoft]"), sClientSoft);
			HTTPProcessData.Replace(_T("[ClientSoftV]"), GetClientNameAndVersionString(cur_client));
			HTTPProcessData.Replace(_T("[ClientExtra]"), _T("none"));
			sQueue += HTTPProcessData;
		}
		Out.Replace(_T("[QueueList]"), sQueue);
	}
	else {
		Out.Replace(_T("[UploadQueue]"), pThis->m_Templates.sTransferUpQueueHide);
	}

	if(pThis->m_Params.bShowUploadQueueBanned) 
	{
		Out.Replace(_T("[UploadQueueBanned]"), pThis->m_Templates.sTransferUpQueueBannedShow);
		Out.Replace(_T("[UploadQueueBannedList]"), _GetPlainResString(IDS_ONQUEUEBANNED));
		CString OutE2 = pThis->m_Templates.sTransferUpQueueBannedLine;
		CString sQueueBanned = _T("");
		for (POSITION pos = theApp.uploadqueue->waitinglist.GetHeadPosition(); pos != 0;theApp.uploadqueue->waitinglist.GetNext(pos)){
			CUpDownClient* cur_client = theApp.uploadqueue->waitinglist.GetAt(pos);
			CString HTTPProcessData;
            TCHAR HTTPTempC[100] = _T("");
			if (cur_client->IsBanned())
				HTTPProcessData = OutE2;
			if(cur_client->GetUserName().GetLength() > SHORT_LENGTH)
				HTTPProcessData.Replace(_T("[UserName]"), _SpecialChars(cur_client->GetUserName().Left(SHORT_LENGTH)) + _T("..."));
			else
				HTTPProcessData.Replace(_T("[UserName]"), _SpecialChars(cur_client->GetUserName()));
			CKnownFile* file = theApp.sharedfiles->GetFileByID(cur_client->reqfileid);
			if (file)
				HTTPProcessData.Replace(_T("[FileName]"), _SpecialChars(file->GetFileName()));
			else
				HTTPProcessData.Replace(_T("[FileName]"), _T("?"));
			_stprintf(HTTPTempC, _T("%i") , cur_client->GetScore(false));
			HTTPProcessData.Replace(_T("[Score]"), HTTPTempC);
			CString sClientSoft; sClientSoft.Format(_T("%u"), cur_client->GetClientSoft());
			HTTPProcessData.Replace(_T("[ClientSoft]"), sClientSoft);
			HTTPProcessData.Replace(_T("[ClientSoftV]"), GetClientNameAndVersionString(cur_client));
			HTTPProcessData.Replace(_T("[ClientExtra]"), _T("banned"));
			sQueueBanned += HTTPProcessData;
		}
		Out.Replace(_T("[QueueListBanned]"), sQueueBanned);
	}
	else {
		Out.Replace(_T("[UploadQueueBanned]"), pThis->m_Templates.sTransferUpQueueBannedHide);
	}

	if(pThis->m_Params.bShowUploadQueueFriend) 
	{
		Out.Replace(_T("[UploadQueueFriend]"), pThis->m_Templates.sTransferUpQueueFriendShow);
		Out.Replace(_T("[UploadQueueFriendList]"), _GetPlainResString(IDS_ONQUEUEFRIEND));
		CString OutE3 = pThis->m_Templates.sTransferUpQueueFriendLine;
		CString sQueueFriend = _T("");
		for (POSITION pos = theApp.uploadqueue->waitinglist.GetHeadPosition(); pos != 0;theApp.uploadqueue->waitinglist.GetNext(pos)){
			CUpDownClient* cur_client = theApp.uploadqueue->waitinglist.GetAt(pos);
			CString HTTPProcessData;
            TCHAR HTTPTempC[100] = _T("");
			if ((cur_client->IsFriend())&&!(cur_client->IsBanned()))
				HTTPProcessData = OutE3;
			if(cur_client->GetUserName().GetLength() > SHORT_LENGTH)
				HTTPProcessData.Replace(_T("[UserName]"), _SpecialChars(cur_client->GetUserName().Left(SHORT_LENGTH)) + _T("..."));
			else
				HTTPProcessData.Replace(_T("[UserName]"), _SpecialChars(cur_client->GetUserName()));
			CKnownFile* file = theApp.sharedfiles->GetFileByID(cur_client->reqfileid);
			if (file)
				HTTPProcessData.Replace(_T("[FileName]"), _SpecialChars(file->GetFileName()));
			else
				HTTPProcessData.Replace(_T("[FileName]"), _T("?"));
			_stprintf(HTTPTempC, _T("%i") , cur_client->GetScore(false));
			HTTPProcessData.Replace(_T("[Score]"), HTTPTempC);
			CString sClientSoft; sClientSoft.Format(_T("%u"), cur_client->GetClientSoft());
			HTTPProcessData.Replace(_T("[ClientSoft]"), sClientSoft);
			HTTPProcessData.Replace(_T("[ClientSoftV]"), GetClientNameAndVersionString(cur_client));
			HTTPProcessData.Replace(_T("[ClientExtra]"), _T("friend"));
			sQueueFriend += HTTPProcessData;
		}
		Out.Replace(_T("[QueueListFriend]"), sQueueFriend);
	}
	else {
		Out.Replace(_T("[UploadQueueFriend]"), pThis->m_Templates.sTransferUpQueueFriendHide);
	}

	if(pThis->m_Params.bShowUploadQueueCredit) 
	{
		Out.Replace(_T("[UploadQueueCredit]"), pThis->m_Templates.sTransferUpQueueCreditShow);
		Out.Replace(_T("[UploadQueueCreditList]"), _GetPlainResString(IDS_ONQUEUECREDIT));
		CString OutE4 = pThis->m_Templates.sTransferUpQueueCreditLine;
		CString sQueueCredit = _T("");
		for (POSITION pos = theApp.uploadqueue->waitinglist.GetHeadPosition(); pos != 0;theApp.uploadqueue->waitinglist.GetNext(pos)){
			CUpDownClient* cur_client = theApp.uploadqueue->waitinglist.GetAt(pos);
			CString HTTPProcessData;
            TCHAR HTTPTempC[100] = _T("");
			if ((cur_client->credits->GetScoreRatio() > 1)&&!(cur_client->IsBanned())&&!(cur_client->IsFriend()))
				HTTPProcessData = OutE4;
			if(cur_client->GetUserName().GetLength() > SHORT_LENGTH)
				HTTPProcessData.Replace(_T("[UserName]"), _SpecialChars(cur_client->GetUserName().Left(SHORT_LENGTH)) + _T("..."));
			else
				HTTPProcessData.Replace(_T("[UserName]"), _SpecialChars(cur_client->GetUserName()));
			CKnownFile* file = theApp.sharedfiles->GetFileByID(cur_client->reqfileid);
			if (file)
				HTTPProcessData.Replace(_T("[FileName]"), _SpecialChars(file->GetFileName()));
			else
				HTTPProcessData.Replace(_T("[FileName]"), _T("?"));
			_stprintf(HTTPTempC, _T("%i") , cur_client->GetScore(false));
			HTTPProcessData.Replace(_T("[Score]"), HTTPTempC);
			CString sClientSoft; sClientSoft.Format(_T("%u"), cur_client->GetClientSoft());
			HTTPProcessData.Replace(_T("[ClientSoft]"), sClientSoft);
			HTTPProcessData.Replace(_T("[ClientSoftV]"), GetClientNameAndVersionString(cur_client));
			HTTPProcessData.Replace(_T("[ClientExtra]"), _T("credit"));
			sQueueCredit += HTTPProcessData;
		}
		Out.Replace(_T("[QueueListCredit]"), sQueueCredit);
	}
	else {
		Out.Replace(_T("[UploadQueueCredit]"), pThis->m_Templates.sTransferUpQueueCreditHide);
	}
	CString mCountQueue;
	mCountQueue.Format(_T("%i"),nCountQueue);
	Out.Replace(_T("[CounterQueue]"), mCountQueue);
	CString mCountQueueBanned;
	mCountQueueBanned.Format(_T("%i"),nCountQueueBanned);
	Out.Replace(_T("[CounterQueueBanned]"), mCountQueueBanned);
	CString mCountQueueFriend;
	mCountQueueFriend.Format(_T("%i"),nCountQueueFriend);
	Out.Replace(_T("[CounterQueueFriend]"), mCountQueueFriend);
	CString mCountQueueCredit;
	mCountQueueCredit.Format(_T("%i"),nCountQueueCredit);
	Out.Replace(_T("[CounterQueueCredit]"), mCountQueueCredit);
	CString mCountQueueAll;
	mCountQueueAll.Format(_T("%i"),nCountQueue+nCountQueueBanned+nCountQueueFriend+nCountQueueCredit);
	Out.Replace(_T("[CounterAll]"), mCountQueueAll);
	Out.Replace(_T("[ShowUploadQueue]"), _GetPlainResString(IDS_VIEWQUEUE));
	Out.Replace(_T("[ShowUploadQueueList]"), _GetPlainResString(IDS_WEB_SHOW_UPLOAD_QUEUE));
	Out.Replace(_T("[ShowUploadQueueListBanned]"), _GetPlainResString(IDS_WEB_SHOW_UPLOAD_QUEUE_BANNED));
	Out.Replace(_T("[ShowUploadQueueListFriend]"), _GetPlainResString(IDS_WEB_SHOW_UPLOAD_QUEUE_FRIEND));
	Out.Replace(_T("[ShowUploadQueueListCredit]"), _GetPlainResString(IDS_WEB_SHOW_UPLOAD_QUEUE_CREDIT));
	Out.Replace(_T("[UserNameTitle]"), _GetPlainResString(IDS_QL_USERNAME));
	Out.Replace(_T("[Version]"), _GetPlainResString(IDS_INFLST_USER_CLIENTVERSION));
	Out.Replace(_T("[FileNameTitle]"), _GetPlainResString(IDS_DL_FILENAME));
	Out.Replace(_T("[ScoreTitle]"), _GetPlainResString(IDS_SCORE));
	Out.Replace(_T("[BannedTitle]"), _GetPlainResString(IDS_BANNED));
	Out.Replace(_T("[Session]"), sSession);

	return Out;

	EMULE_CATCH

	return _T("");
}

CString CWebServer::_GetSharedFilesList(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return _T("");

	CString HTTPTemp;

	CString sSession = _ParseURL(Data.sURL, _T("ses"));
	bool bAdmin = IsSessionAdmin(Data,sSession);

	if (_ParseURL(Data.sURL, _T("sort")) != "") 
	{
		if(_ParseURL(Data.sURL, _T("sort")) == "name")
			pThis->m_Params.SharedSort = SHARED_SORT_NAME;
		else
		if(_ParseURL(Data.sURL, _T("sort")) == "size")
			pThis->m_Params.SharedSort = SHARED_SORT_SIZE;
		else
		if(_ParseURL(Data.sURL, _T("sort")) == "transferred")
			pThis->m_Params.SharedSort = SHARED_SORT_TRANSFERRED;
		else
		if(_ParseURL(Data.sURL, _T("sort")) == "alltimetransferred")
			pThis->m_Params.SharedSort = SHARED_SORT_ALL_TIME_TRANSFERRED;
		else
		if(_ParseURL(Data.sURL, _T("sort")) == "requests")
			pThis->m_Params.SharedSort = SHARED_SORT_REQUESTS;
		else
		if(_ParseURL(Data.sURL, _T("sort")) == "alltimerequests")
			pThis->m_Params.SharedSort = SHARED_SORT_ALL_TIME_REQUESTS;
		else
		if(_ParseURL(Data.sURL, _T("sort")) == "accepts")
			pThis->m_Params.SharedSort = SHARED_SORT_ACCEPTS;
		else
		if(_ParseURL(Data.sURL, _T("sort")) == "alltimeaccepts")
			pThis->m_Params.SharedSort = SHARED_SORT_ALL_TIME_ACCEPTS;
		else
		if(_ParseURL(Data.sURL, _T("sort")) == "priority")
			pThis->m_Params.SharedSort = SHARED_SORT_PRIORITY;

		if(_ParseURL(Data.sURL, _T("sortreverse")) == "")
			pThis->m_Params.bSharedSortReverse = false;
	}
	if (_ParseURL(Data.sURL, _T("sortreverse")) != "") 
		pThis->m_Params.bSharedSortReverse = (_ParseURL(Data.sURL, _T("sortreverse")) == "true");

	if(_ParseURL(Data.sURL, _T("hash")) != "" && _ParseURL(Data.sURL, _T("setpriority")) != "" && IsSessionAdmin(Data,sSession)) 
		_SetSharedFilePriority(_ParseURL(Data.sURL, _T("hash")),_tstoi(_ParseURL(Data.sURL, _T("setpriority"))));

	//+++
	CString sCmd = _ParseURL(Data.sURL, _T(""));
	if (bAdmin && _ParseURL(Data.sURL, _T("jumpstart")) == "true")
		_SetSharedFileJumpstart(_ParseURL(Data.sURL, _T("hash")));
	else if (bAdmin &&  _ParseURL(Data.sURL, _T("jumpstart")) == "false")
		_SetSharedFileNoJumpstart(_ParseURL(Data.sURL, _T("hash")));

	if(_ParseURL(Data.sURL, _T("reload")) == "true")
		theApp.emuledlg->SendMessage(WEB_SHARED_FILES_RELOAD);

	CString sSharedSortRev;
	if(pThis->m_Params.bSharedSortReverse)
		sSharedSortRev = "false";
	else
		sSharedSortRev = "true";
    //Name sorting link
	CString Out = pThis->m_Templates.sSharedList;
	if(pThis->m_Params.SharedSort == SHARED_SORT_NAME)
		Out.Replace(_T("[SortName]"), _T("sort=name&sortreverse=") + sSharedSortRev);
	else
		Out.Replace(_T("[SortName]"), _T("sort=name"));
	//Size sorting Link
    if(pThis->m_Params.SharedSort == SHARED_SORT_SIZE)
		Out.Replace(_T("[SortSize]"), _T("sort=size&sortreverse=") + sSharedSortRev);
	else
		Out.Replace(_T("[SortSize]"), _T("sort=size"));
	//Priority sorting Link
    if(pThis->m_Params.SharedSort == SHARED_SORT_PRIORITY)
		Out.Replace(_T("[SortPriority]"), _T("sort=priority&sortreverse=") + sSharedSortRev);
	else
		Out.Replace(_T("[SortPriority]"), _T("sort=priority"));
    //Transferred sorting link
	if(pThis->m_Params.SharedSort == SHARED_SORT_TRANSFERRED)
	{
		if(pThis->m_Params.bSharedSortReverse)
            Out.Replace(_T("[SortTransferred]"), _T("sort=alltimetransferred&sortreverse=") + sSharedSortRev);
		else
			Out.Replace(_T("[SortTransferred]"), _T("sort=transferred&sortreverse=") + sSharedSortRev);
	}
	else
	if(pThis->m_Params.SharedSort == SHARED_SORT_ALL_TIME_TRANSFERRED)
	{
		if(pThis->m_Params.bSharedSortReverse)
            Out.Replace(_T("[SortTransferred]"), _T("sort=transferred&sortreverse=") + sSharedSortRev);
		else
			Out.Replace(_T("[SortTransferred]"), _T("sort=alltimetransferred&sortreverse=") + sSharedSortRev);
	}
	else
        Out.Replace(_T("[SortTransferred]"), _T("&sort=transferred&sortreverse=false"));
    //Request sorting link
	if(pThis->m_Params.SharedSort == SHARED_SORT_REQUESTS)
	{
		if(pThis->m_Params.bSharedSortReverse)
            Out.Replace(_T("[SortRequests]"), _T("sort=alltimerequests&sortreverse=") + sSharedSortRev);
		else
			Out.Replace(_T("[SortRequests]"), _T("sort=requests&sortreverse=") + sSharedSortRev);
	}
	else
	if(pThis->m_Params.SharedSort == SHARED_SORT_ALL_TIME_REQUESTS)
	{
		if(pThis->m_Params.bSharedSortReverse)
            Out.Replace(_T("[SortRequests]"), _T("sort=requests&sortreverse=") + sSharedSortRev);
		else
			Out.Replace(_T("[SortRequests]"), _T("sort=alltimerequests&sortreverse=") + sSharedSortRev);
	}
	else
        Out.Replace(_T("[SortRequests]"), _T("&sort=requests&sortreverse=false"));
    //Accepts sorting link
	if(pThis->m_Params.SharedSort == SHARED_SORT_ACCEPTS)
	{
		if(pThis->m_Params.bSharedSortReverse)
            Out.Replace(_T("[SortAccepts]"), _T("sort=alltimeaccepts&sortreverse=") + sSharedSortRev);
		else
			Out.Replace(_T("[SortAccepts]"), _T("sort=accepts&sortreverse=") + sSharedSortRev);
	}
	else
	if(pThis->m_Params.SharedSort == SHARED_SORT_ALL_TIME_ACCEPTS)
	{
		if(pThis->m_Params.bSharedSortReverse)
            Out.Replace(_T("[SortAccepts]"), _T("sort=accepts&sortreverse=") + sSharedSortRev);
		else
			Out.Replace(_T("[SortAccepts]"), _T("sort=alltimeaccepts&sortreverse=") + sSharedSortRev);
	}
	else
        Out.Replace(_T("[SortAccepts]"), _T("&sort=accepts&sortreverse=false"));

	if(_ParseURL(Data.sURL, _T("reload")) == "true")
	{
		CString resultlog = _SpecialChars(theApp.emuledlg->logtext);	//Pick-up last line of the log
		resultlog = resultlog.TrimRight('\n');
		resultlog = resultlog.Mid(resultlog.ReverseFind('\n'));
		Out.Replace(_T("[Message]"),resultlog);
	}
	else
        Out.Replace(_T("[Message]"),_T(""));

	Out.Replace(_T("[Filename]"), _GetPlainResString(IDS_DL_FILENAME));
	Out.Replace(_T("[Priority]"),  _GetPlainResString(IDS_PRIORITY));
    Out.Replace(_T("[FileTransferred]"),  _GetPlainResString(IDS_SF_TRANSFERRED));
    Out.Replace(_T("[FileRequests]"),  _GetPlainResString(IDS_SF_REQUESTS));
    Out.Replace(_T("[FileAccepts]"),  _GetPlainResString(IDS_SF_ACCEPTS));
	Out.Replace(_T("[Size]"), _GetPlainResString(IDS_DL_SIZE));
	Out.Replace(_T("[Actions]"), _GetPlainResString(IDS_WEB_ACTIONS));
	Out.Replace(_T("[Reload]"), _GetPlainResString(IDS_SF_RELOAD));
	Out.Replace(_T("[Session]"), sSession);

	CString OutE = pThis->m_Templates.sSharedLine; 

	CArray<SharedFiles, SharedFiles> SharedArray;
	// Populating array
	for (POSITION pos = theApp.sharedfiles->m_Files_map.GetStartPosition();pos != 0;)
	{
		CCKey bufKey;
		CKnownFile* cur_file;
		CString buffer;
		
		theApp.sharedfiles->m_Files_map.GetNextAssoc(pos,bufKey,cur_file);

		SharedFiles dFile;
		//dFile.sFileName = _SpecialChars(cur_file->GetFileName());
		dFile.bIsPartFile = cur_file->IsPartFile();
		dFile.sFileName = cur_file->GetFileName();
		dFile.lFileSize = cur_file->GetFileSize();
#ifdef OLD_SOCKETS_ENABLED
		if (theApp.serverconnect->IsConnected()) 
			dFile.sED2kLink = cur_file->CreateED2kSourceLink();
		else
#endif //OLD_SOCKETS_ENABLED
            dFile.sED2kLink = cur_file->CreateED2kLink();

		dFile.nFileTransferred = cur_file->statistic.GetTransfered();
		dFile.nFileAllTimeTransferred = cur_file->statistic.GetAllTimeTransfered();
		dFile.nFileRequests = cur_file->statistic.GetRequests();
		dFile.nFileAllTimeRequests = cur_file->statistic.GetAllTimeRequests();
		dFile.nFileAccepts = cur_file->statistic.GetAccepts();
		dFile.nFileAllTimeAccepts = cur_file->statistic.GetAllTimeAccepts();

		dFile.sFileHash = EncodeBase16(cur_file->GetFileHash(), 16);
		if (cur_file->IsAutoPrioritized())
		{
			if (cur_file->GetPriority() == PR_NORMAL)
				dFile.sFilePriority = _GetPlainResString(IDS_PRIOAUTONORMAL);
			else if (cur_file->GetPriority() == PR_HIGH)
				dFile.sFilePriority = _GetPlainResString(IDS_PRIOAUTOHIGH);
			else if (cur_file->GetPriority() == PR_VERYHIGH)
				dFile.sFilePriority = _GetPlainResString(IDS_PRIOAUTORELEASE);
		}
		else
		{
			// unused  CString sPriority;
			if (cur_file->GetPriority() == PR_VERYLOW)
				dFile.sFilePriority = _GetPlainResString(IDS_PRIOVERYLOW);
			else if (cur_file->GetPriority() == PR_LOW)
				dFile.sFilePriority = _GetPlainResString(IDS_PRIOLOW);
			else if (cur_file->GetPriority() == PR_NORMAL)
				dFile.sFilePriority = _GetPlainResString(IDS_PRIONORMAL);
			else if (cur_file->GetPriority() == PR_HIGH)
				dFile.sFilePriority = _GetPlainResString(IDS_PRIOHIGH);
			else if (cur_file->GetPriority() == PR_VERYHIGH)
				dFile.sFilePriority = _GetPlainResString(IDS_PRIORELEASE);
#if JUMPSTART
			if (cur_file->GetJumpstartEnabled()) {
				buffer.Format(_T("%s[%s]"), _GetPlainResString(IDS_PRIOJUMPSTART),dFile.sFilePriority);
				dFile.sFilePriority = buffer;
			}
#endif
			// removed: dFile.sFilePriority = buffer;
		}
		dFile.nFilePriority = cur_file->GetPriority();
		dFile.bFileAutoPriority = cur_file->IsAutoPrioritized();
		SharedArray.Add(dFile);
	}

	// Sorting (simple bubble sort, we don't have tons of data here)
	bool bSorted = true;
	
	for(int nMax = 0;bSorted && nMax < SharedArray.GetCount()*2; nMax++)
	{
		bSorted = false;
		for(int i = 0; i < SharedArray.GetCount() - 1; i++)
		{
			bool bSwap = false;
			switch(pThis->m_Params.SharedSort)
			{
			case SHARED_SORT_NAME:
				bSwap = SharedArray[i].sFileName.CompareNoCase(SharedArray[i+1].sFileName) > 0;
				break;
			case SHARED_SORT_SIZE:
				bSwap = SharedArray[i].lFileSize < SharedArray[i+1].lFileSize;
				break;
			case SHARED_SORT_TRANSFERRED:
				bSwap = SharedArray[i].nFileTransferred < SharedArray[i+1].nFileTransferred;
				break;
			case SHARED_SORT_ALL_TIME_TRANSFERRED:
				bSwap = SharedArray[i].nFileAllTimeTransferred < SharedArray[i+1].nFileAllTimeTransferred;
				break;
			case SHARED_SORT_REQUESTS:
				bSwap = SharedArray[i].nFileRequests < SharedArray[i+1].nFileRequests;
				break;
			case SHARED_SORT_ALL_TIME_REQUESTS:
				bSwap = SharedArray[i].nFileAllTimeRequests < SharedArray[i+1].nFileAllTimeRequests;
				break;
			case SHARED_SORT_ACCEPTS:
				bSwap = SharedArray[i].nFileAccepts < SharedArray[i+1].nFileAccepts;
				break;
			case SHARED_SORT_ALL_TIME_ACCEPTS:
				bSwap = SharedArray[i].nFileAllTimeAccepts < SharedArray[i+1].nFileAllTimeAccepts;
				break;
			case SHARED_SORT_PRIORITY:
				//Very low priority is define equal to 4 ! Must adapte sorting code
				if(SharedArray[i].nFilePriority == 4)
				{
					if(SharedArray[i+1].nFilePriority == 4)
						bSwap = false;
					else
						bSwap = true;
				}        
				else
					if(SharedArray[i+1].nFilePriority == 4)
					{
						if(SharedArray[i].nFilePriority == 4)
							bSwap = true;
						else
							bSwap = false;
					}
					else
						bSwap = SharedArray[i].nFilePriority < SharedArray[i+1].nFilePriority;
				break;
			}
			if(pThis->m_Params.bSharedSortReverse)
			{
				bSwap = !bSwap;
			}
			if(bSwap)
			{
				bSorted = true;
				SharedFiles TmpFile = SharedArray[i];
				SharedArray[i] = SharedArray[i+1];
				SharedArray[i+1] = TmpFile;
			}
		}
	}
	// Displaying
	CString sSharedList = _T("");
	for(int i = 0; i < SharedArray.GetCount(); i++)
	{
		TCHAR HTTPTempC[100] = _T("");
		CString HTTPProcessData;
        HTTPProcessData = OutE;

		if (SharedArray[i].sFileHash == _ParseURL(Data.sURL,_T("hash")) )
            HTTPProcessData.Replace(_T("[LastChangedDataset]"), "checked");
		else
            HTTPProcessData.Replace(_T("[LastChangedDataset]"), "checked_no");

		CString admin = "";			//admin rights
		CString ed2k = "";			//ed2klink
		CString session = "";		//ed2klink
		CString hash = "";			//hash
		CString fname = "";			//filename
		CString upprio = "";		//prioup
		CString lowprio = "";		//priodown
		CString isjumpstart = "";	//jumpstart

		uint8 upperpriority=0, lesserpriority=0;
		if(SharedArray[i].nFilePriority == 4) {
			upperpriority = 0;	lesserpriority = 4;
		} else if(SharedArray[i].nFilePriority == 0) {
			upperpriority = 1;	lesserpriority = 4;
		} else if(SharedArray[i].nFilePriority == 1) {
			upperpriority = 2;	lesserpriority = 0;
		} else if(SharedArray[i].nFilePriority == 2) {
			upperpriority = 3;	lesserpriority = 1;
		} else if(SharedArray[i].nFilePriority == 3) {
			upperpriority = 5;	lesserpriority = 2;
		} else if(SharedArray[i].nFilePriority == 5) {
			upperpriority = 5;	lesserpriority = 3;
		}
		if(SharedArray[i].bFileAutoPriority) {
			upperpriority = 5;	lesserpriority = 3;
		}

		if (bAdmin)
			admin.Format(_T("admin"));

		CString JSED2kLink=SharedArray[i].sED2kLink;
		JSED2kLink.Replace(_T("'"),_T("&#8217;"));

		ed2k.Format(JSED2kLink);
		session.Format(CString(sSession));
		hash.Format(CString(SharedArray[i].sFileHash));
		fname.Format(CString(SharedArray[i].sFileName));
		fname.Replace(_T("'"),_T("&#8217;"));

		_stprintf(HTTPTempC, _T("%i"), upperpriority);
		upprio.Format(CString(HTTPTempC));
		_stprintf(HTTPTempC, _T("%i"), lesserpriority);
		lowprio.Format(CString(HTTPTempC));

		// Purity: showing shared files priority for Release and JS
		CKnownFile* cur_file;
		uchar fileid[16];
		DecodeBase16(hash.GetBuffer(),hash.GetLength(),fileid);
		cur_file=theApp.sharedfiles->GetFileByID(fileid);

		if (cur_file->GetJumpstartEnabled())
		{
			isjumpstart.Format(_T("jumpstart"));//+++
			HTTPProcessData.Replace(_T("[FileIsPriority]"), "jumpstart");
		}
		else if (cur_file->GetPriority() == PR_VERYHIGH)
			HTTPProcessData.Replace(_T("[FileIsPriority]"), "release");
		else
			HTTPProcessData.Replace(_T("[FileIsPriority]"), "none");

		HTTPProcessData.Replace(_T("[admin]"), admin);
		HTTPProcessData.Replace(_T("[ed2k]"), ed2k);
		HTTPProcessData.Replace(_T("[hash]"), hash);
		HTTPProcessData.Replace(_T("[fname]"), fname);
		HTTPProcessData.Replace(_T("[session]"), session);
		HTTPProcessData.Replace(_T("[upprio]"), upprio);
		HTTPProcessData.Replace(_T("[lowprio]"), lowprio);
		HTTPProcessData.Replace(_T("[isjumpstart]"), isjumpstart);//+++

		if(SharedArray[i].bIsPartFile)
			HTTPProcessData.Replace(_T("[FileState]"), "filedown");
		else
			HTTPProcessData.Replace(_T("[FileState]"), "file");

		HTTPProcessData.Replace(_T("[FileName]"), _SpecialChars(SharedArray[i].sFileName));
		if(SharedArray[i].sFileName.GetLength() > SHORT_LENGTH)
			HTTPProcessData.Replace(_T("[ShortFileName]"), _SpecialChars(SharedArray[i].sFileName.Left(SHORT_LENGTH)) + _T("..."));
		else
			HTTPProcessData.Replace(_T("[ShortFileName]"), _SpecialChars(SharedArray[i].sFileName));

		_stprintf(HTTPTempC, _T("%s"),CastItoXBytes(SharedArray[i].lFileSize));
		HTTPProcessData.Replace(_T("[FileSize]"), CString(HTTPTempC));

		_stprintf(HTTPTempC, _T("%s"),CastItoXBytes(SharedArray[i].nFileTransferred));
		HTTPProcessData.Replace(_T("[FileTransferred]"), CString(HTTPTempC));

		_stprintf(HTTPTempC, _T("%s"),CastItoXBytes(SharedArray[i].nFileAllTimeTransferred));
		HTTPProcessData.Replace(_T("[FileAllTimeTransferred]"), CString(HTTPTempC));

		_stprintf(HTTPTempC, _T("%i"), SharedArray[i].nFileRequests);
		HTTPProcessData.Replace(_T("[FileRequests]"), CString(HTTPTempC));

		_stprintf(HTTPTempC, _T("%i"), SharedArray[i].nFileAllTimeRequests);
		HTTPProcessData.Replace(_T("[FileAllTimeRequests]"), CString(HTTPTempC));

		_stprintf(HTTPTempC, _T("%i"), SharedArray[i].nFileAccepts);
		HTTPProcessData.Replace(_T("[FileAccepts]"), CString(HTTPTempC));

		_stprintf(HTTPTempC, _T("%i"), SharedArray[i].nFileAllTimeAccepts);
		HTTPProcessData.Replace(_T("[FileAllTimeAccepts]"), CString(HTTPTempC));
		HTTPProcessData.Replace(_T("[Priority]"), SharedArray[i].sFilePriority);

		HTTPProcessData.Replace(_T("[FileHash]"), SharedArray[i].sFileHash);

		sSharedList += HTTPProcessData;
	}
	Out.Replace(_T("[SharedFilesList]"), sSharedList);
	Out.Replace(_T("[Session]"), sSession);

	return Out;

	EMULE_CATCH

	return _T("");
}

CString CWebServer::_GetGraphs(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return _T("");

	CString Out = pThis->m_Templates.sGraphs;

	CString sGraphDownload = _T(""), sGraphUpload = _T(""), sGraphCons = _T("");
	CString sTmp = _T("");
	int nPrevValUp = 0, nPrevValDown = 0;
	for(int i = 0; i < WEB_GRAPH_WIDTH; i++)
	{
		if(i < pThis->m_Params.PointsForWeb.GetCount())
		{
			if(i != 0)
			{
				sGraphDownload += _T(",");
				sGraphUpload += _T(",");
				sGraphCons += _T(",");
			}

			// download
			sTmp.Format(_T("%d") , (uint32) (pThis->m_Params.PointsForWeb[i].download*1024));
			sGraphDownload += sTmp;

			// upload
			sTmp.Format(_T("%d") , (uint32) (pThis->m_Params.PointsForWeb[i].upload*1024));
			sGraphUpload += sTmp;

			// connections
			sTmp.Format(_T("%d") , (uint32) (pThis->m_Params.PointsForWeb[i].connections));
			sGraphCons += sTmp;
		}
	}

	Out.Replace(_T("[GraphDownload]"), sGraphDownload);
	Out.Replace(_T("[GraphUpload]"), sGraphUpload);
	Out.Replace(_T("[GraphConnections]"), sGraphCons);

	Out.Replace(_T("[TxtDownload]"), _GetPlainResString(IDS_ST_ACTIVEDOWNLOAD));
	Out.Replace(_T("[TxtUpload]"), _GetPlainResString(IDS_ST_ACTIVEUPLOAD));
	Out.Replace(_T("[TxtTime]"), _GetPlainResString(IDS_TIME));
	Out.Replace(_T("[KByteSec]"), _GetPlainResString(IDS_KBYTESEC));
	Out.Replace(_T("[TxtConnections]"), _GetPlainResString(IDS_ST_ACTIVECONNECTIONS));

	CString sScale;
	sScale.Format(_T("%s"), CastSecondsToHM(theApp.glob_prefs->GetTrafficOMeterInterval() * WEB_GRAPH_WIDTH) );

	CString s1, s2, s3;
	s1.Format(_T("%d"), theApp.glob_prefs->GetMaxGraphDownloadRate() + 4);
	s2.Format(_T("%d"), theApp.glob_prefs->GetMaxGraphUploadRate() + 4);
	s3.Format(_T("%d"), theApp.glob_prefs->GetMaxConnections());

	Out.Replace(_T("[ScaleTime]"), sScale);
	Out.Replace(_T("[MaxDownload]"), s1);
	Out.Replace(_T("[MaxUpload]"), s2);
	Out.Replace(_T("[MaxConnections]"), s3);

	return Out;

	EMULE_CATCH

	return _T("");
}

CString CWebServer::_GetAddServerBox(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return _T("");

	CString sSession = _ParseURL(Data.sURL, _T("ses"));

	if (!IsSessionAdmin(Data,sSession)) return _T("");

	CString Out = pThis->m_Templates.sAddServerBox;
	if(_ParseURL(Data.sURL, _T("addserver")) == "true")
	{
		theApp.emuledlg->serverwnd.AddServer(_ParseURL(Data.sURL, _T("serveraddr")),_ParseURL(Data.sURL, _T("serverport")),_ParseURL(Data.sURL, _T("servername")));
		CString resultlog = _SpecialChars(theApp.emuledlg->logtext);	//Pick-up last line of the log
		resultlog = resultlog.TrimRight('\n');
		resultlog = resultlog.Mid(resultlog.ReverseFind('\n'));
		Out.Replace(_T("[Message]"),resultlog);
	}
	else
		if(_ParseURL(Data.sURL, _T("updateservermetfromurl")) == "true")
		{
			theApp.emuledlg->serverwnd.UpdateServerMetFromURL(_ParseURL(Data.sURL, _T("servermeturl")));
			CString resultlog = _SpecialChars(theApp.emuledlg->logtext);
			resultlog = resultlog.TrimRight('\n');
			resultlog = resultlog.Mid(resultlog.ReverseFind('\n'));
			Out.Replace(_T("[Message]"),resultlog);
		}
		else
		Out.Replace(_T("[Message]"), _T(""));
    Out.Replace(_T("[AddServer]"), _GetPlainResString(IDS_SV_NEWSERVER));
	Out.Replace(_T("[IP]"), _GetPlainResString(IDS_SV_ADDRESS));
	Out.Replace(_T("[Port]"), _GetPlainResString(IDS_SV_PORT));
	Out.Replace(_T("[Name]"), _GetPlainResString(IDS_SW_NAME));
	Out.Replace(_T("[Add]"), _GetPlainResString(IDS_SV_ADD));
	Out.Replace(_T("[UpdateServerMetFromURL]"), _GetPlainResString(IDS_SV_MET));
	Out.Replace(_T("[URL]"), _GetPlainResString(IDS_SV_URL));
	Out.Replace(_T("[Apply]"), _GetPlainResString(IDS_PW_APPLY));

	return Out;

	EMULE_CATCH

	return _T("");
}
CString	CWebServer::_GetWebSearch(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return _T("");

	CString sSession = _ParseURL(Data.sURL, _T("ses"));
    
	CString Out = pThis->m_Templates.sWebSearch;
	if(_ParseURL(Data.sURL, _T("tosearch")) != "")
	{
		CString query;
		CString tosearch = _ParseURL(Data.sURL, _T("tosearch"));
		query = _T("http://www.filedonkey.com/fdsearch/index.php?media=");
		query += _ParseURL(Data.sURL, _T("media"));
		tosearch = URLEncode(tosearch);
		tosearch.Replace(_T("%20"),_T("+"));
		query += _T("&pattern=");
		query += _ParseURL(Data.sURL, _T("tosearch"));
		query += _T("&action=search&name=FD-Search&op=modload&file=index&requestby=emule");
		Out += _T("\n<script language=\"javascript\">");
		Out += _T("\n searchwindow=window.open('")+ query + _T("','searchwindow');");
		Out += _T("\n</script>");
	}
	Out.Replace(_T("[Session]"), sSession);
	Out.Replace(_T("[Name]"), _GetPlainResString(IDS_SW_NAME));
	Out.Replace(_T("[Type]"), _GetPlainResString(IDS_TYPE));
	Out.Replace(_T("[Any]"), _GetPlainResString(IDS_SEARCH_ANY));
	Out.Replace(_T("[Audio]"), _GetPlainResString(IDS_SEARCH_AUDIO));
	Out.Replace(_T("[Video]"), _GetPlainResString(IDS_SEARCH_VIDEO));
	Out.Replace(_T("[Document]"), _GetPlainResString(IDS_SEARCH_DOC));
	Out.Replace(_T("[Other]"), _T("Other"));
	Out.Replace(_T("[Search]"), _GetPlainResString(IDS_SW_START));
	Out.Replace(_T("[WebSearch]"), _GetPlainResString(IDS_SW_WEBBASED));
	
	return Out;

	EMULE_CATCH

	return _T("");
}

CString CWebServer::_GetLog(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return _T("");

	CString sSession = _ParseURL(Data.sURL, _T("ses"));

	CString Out = pThis->m_Templates.sLog;

	if (_ParseURL(Data.sURL, _T("clear")) == "yes" && IsSessionAdmin(Data,sSession))
	{
		theApp.emuledlg->ResetLog();
	}
	Out.Replace(_T("[Clear]"), _GetPlainResString(IDS_PW_RESET));
	Out.Replace(_T("[Log]"), _SpecialChars(theApp.emuledlg->logtext));
	Out.Replace(_T("[Session]"), sSession);

	return Out;

	EMULE_CATCH

	return _T("");
}

CString CWebServer::_GetServerInfo(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return _T("");

	CString sSession = _ParseURL(Data.sURL, _T("ses"));

	CString Out = pThis->m_Templates.sServerInfo;

	if (_ParseURL(Data.sURL, _T("clear")) == "yes" && IsSessionAdmin(Data,sSession))
	{
		theApp.emuledlg->serverwnd.servermsgbox.SetWindowText(_T(""));
	}
	Out.Replace(_T("[Clear]"), _GetPlainResString(IDS_PW_RESET));
	Out.Replace(_T("[ServerInfo]"), _SpecialChars(theApp.emuledlg->serverwnd.servermsgbox.GetHyperText()->GetText()));
	Out.Replace(_T("[Session]"), sSession);

	return Out;

	EMULE_CATCH

	return _T("");
}

CString CWebServer::_GetDebugLog(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return _T("");

	CString sSession = _ParseURL(Data.sURL, _T("ses"));

	CString Out = pThis->m_Templates.sDebugLog;

	if (_ParseURL(Data.sURL, _T("clear")) == "yes" && IsSessionAdmin(Data,sSession))
	{
		theApp.emuledlg->ResetDebugLog();
	}
	Out.Replace(_T("[Clear]"), _GetPlainResString(IDS_PW_RESET));
	Out.Replace(_T("[DebugLog]"), _SpecialChars(theApp.emuledlg->debuglog));
	Out.Replace(_T("[Session]"), sSession);

	return Out;

	EMULE_CATCH

	return _T("");
}

CString CWebServer::_GetStats(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return _T("");

	CString sSession = _ParseURL(Data.sURL, _T("ses"));

	// refresh statistics
	theApp.emuledlg->ShowStatistics();

	CString Out = pThis->m_Templates.sStats;
	Out.Replace(_T("[Stats]"), theApp.emuledlg->statisticswnd.m_Stats.ExportHTML());

	return Out;

	EMULE_CATCH

	return _T("");
}

CString CWebServer::_GetPreferences(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return _T("");

	CString sSession = _ParseURL(Data.sURL, _T("ses"));

	CString Out = pThis->m_Templates.sPreferences;
	Out.Replace(_T("[Session]"), sSession);

	if ((_ParseURL(Data.sURL, _T("saveprefs")) == "true") && IsSessionAdmin(Data,sSession) ) {
		if(_ParseURL(Data.sURL, _T("gzip")) == "true" || _ParseURL(Data.sURL, _T("gzip")).MakeLower() == "on")
	    {
		    theApp.glob_prefs->SetWebUseGzip(true);
	    }
		if(_ParseURL(Data.sURL, _T("gzip")) == "false" || _ParseURL(Data.sURL, _T("gzip")) == "")
	    {
		    theApp.glob_prefs->SetWebUseGzip(false);
	    }
		if(_ParseURL(Data.sURL, _T("showuploadqueue")) == "true" || _ParseURL(Data.sURL, _T("showuploadqueue")).MakeLower() == "on" )
	    {
		    pThis->m_Params.bShowUploadQueue = true;
	    }
		if(_ParseURL(Data.sURL, _T("showuploadqueuebanned")) == "true" || _ParseURL(Data.sURL, _T("showuploadqueuebanned")).MakeLower() == "on" )
	    {
		    pThis->m_Params.bShowUploadQueueBanned = true;
	    }
		if(_ParseURL(Data.sURL, _T("showuploadqueuefriend")) == "true" || _ParseURL(Data.sURL, _T("showuploadqueuefriend")).MakeLower() == "on" )
	    {
		    pThis->m_Params.bShowUploadQueueFriend = true;
	    }
		if(_ParseURL(Data.sURL, _T("showuploadqueuecredit")) == "true" || _ParseURL(Data.sURL, _T("showuploadqueuecredit")).MakeLower() == "on" )
	    {
		    pThis->m_Params.bShowUploadQueueCredit = true;
	    }
	    if(_ParseURL(Data.sURL, _T("refresh")) != "")
	    {
		    theApp.glob_prefs->SetWebPageRefresh(_tstoi(_ParseURL(Data.sURL, _T("refresh"))));
	    }
	    if(_ParseURL(Data.sURL, _T("maxdown")) != "")
	    {
		    if(_tstoi(_ParseURL(Data.sURL, _T("maxdown")))>0)
			    theApp.glob_prefs->SetMaxDownload(_tstoi(_ParseURL(Data.sURL, _T("maxdown"))));
//		    else
//			    theApp.glob_prefs->SetMaxDownload(65535);
	    }
	    if(_ParseURL(Data.sURL, _T("maxup")) != "")
	    {
		    if(_tstoi(_ParseURL(Data.sURL, _T("maxup")))>0)
			    theApp.glob_prefs->SetMaxUpload(_tstoi(_ParseURL(Data.sURL, _T("maxup"))));
//		    else
//			    theApp.glob_prefs->SetMaxUpload(65535);
	    }
	    if(_ParseURL(Data.sURL, _T("maxcapdown")) != "")
	    {
		    theApp.glob_prefs->SetMaxGraphDownloadRate(_tstoi(_ParseURL(Data.sURL, _T("maxcapdown"))));
	    }
	    if(_ParseURL(Data.sURL, _T("maxcapup")) != "")
	    {
		    theApp.glob_prefs->SetMaxGraphUploadRate(_tstoi(_ParseURL(Data.sURL, _T("maxcapup"))));
	    }

		if(_ParseURL(Data.sURL, _T("maxsources")) != "")
		{
			theApp.glob_prefs->SetMaxSourcePerFile(_tstoi(_ParseURL(Data.sURL, _T("maxsources"))));
		}
		if(_ParseURL(Data.sURL, _T("maxconnections")) != "")
		{
			theApp.glob_prefs->SetMaxConnections(_tstoi(_ParseURL(Data.sURL, _T("maxconnections"))));
		}
		if(_ParseURL(Data.sURL, _T("maxconnectionsperfive")) != "")
		{
			theApp.glob_prefs->SetMaxDownloadConperFive(_tstoi(_ParseURL(Data.sURL, _T("maxconnectionsperfive"))));
		}
	}

	// Fill form
	if(theApp.glob_prefs->GetWebUseGzip())
	{
		Out.Replace(_T("[UseGzipVal]"), _T("checked"));
	}
	else
	{
		Out.Replace(_T("[UseGzipVal]"), _T(""));
	}
    if(pThis->m_Params.bShowUploadQueue)
	{
		Out.Replace(_T("[ShowUploadQueueVal]"), _T("checked"));
	}
	else
	{
		Out.Replace(_T("[ShowUploadQueueVal]"), _T(""));
	}
    if(pThis->m_Params.bShowUploadQueueBanned)
	{
		Out.Replace(_T("[ShowUploadQueueBannedVal]"), _T("checked"));
	}
	else
	{
		Out.Replace(_T("[ShowUploadQueueBannedVal]"), _T(""));
	}
    if(pThis->m_Params.bShowUploadQueueFriend)
	{
		Out.Replace(_T("[ShowUploadQueueFriendVal]"), _T("checked"));
	}
	else
	{
		Out.Replace(_T("[ShowUploadQueueFriendVal]"), _T(""));
	}
    if(pThis->m_Params.bShowUploadQueueCredit)
	{
		Out.Replace(_T("[ShowUploadQueueCreditVal]"), _T("checked"));
	}
	else
	{
		Out.Replace(_T("[ShowUploadQueueCreditVal]"), _T(""));
	}
	CString sRefresh;
	
	sRefresh.Format(_T("%d"), theApp.glob_prefs->GetWebPageRefresh());
	Out.Replace(_T("[RefreshVal]"), sRefresh);

	sRefresh.Format(_T("%d"), theApp.glob_prefs->GetMaxSourcePerFile());
	Out.Replace(_T("[MaxSourcesVal]"), sRefresh);

	sRefresh.Format(_T("%d"), theApp.glob_prefs->GetMaxConnections());
	Out.Replace(_T("[MaxConnectionsVal]"), sRefresh);

	sRefresh.Format(_T("%d"), theApp.glob_prefs->GetMaxConperFive());
	Out.Replace(_T("[MaxConnectionsPer5Val]"), sRefresh);

	Out.Replace(_T("[KBS]"), _GetPlainResString(IDS_KBYTESEC)+_T(":"));
	Out.Replace(_T("[FileSettings]"), CString(_GetPlainResString(IDS_WEB_FILESETTINGS)+_T(":")));
	Out.Replace(_T("[LimitForm]"), _GetPlainResString(IDS_WEB_CONLIMITS)+_T(":"));
	Out.Replace(_T("[MaxSources]"), _GetPlainResString(IDS_PW_MAXSOURCES)+_T(":"));
	Out.Replace(_T("[MaxConnections]"), _GetPlainResString(IDS_PW_MAXC)+_T(":"));
	Out.Replace(_T("[MaxConnectionsPer5]"), _GetPlainResString(IDS_MAXCON5_TEXT)+_T(":"));
	Out.Replace(_T("[UseGzipForm]"), _GetPlainResString(IDS_WEB_GZIP_COMPRESSION));
	Out.Replace(_T("[UseGzipComment]"), _GetPlainResString(IDS_WEB_GZIP_COMMENT));
	Out.Replace(_T("[ShowUploadQueueForm]"), _GetPlainResString(IDS_WEB_SHOW_UPLOAD_QUEUE));
	Out.Replace(_T("[ShowUploadQueueBannedForm]"), _GetPlainResString(IDS_WEB_SHOW_UPLOAD_QUEUE_BANNED));
	Out.Replace(_T("[ShowUploadQueueFriendForm]"), _GetPlainResString(IDS_WEB_SHOW_UPLOAD_QUEUE_FRIEND));
	Out.Replace(_T("[ShowUploadQueueCreditForm]"), _GetPlainResString(IDS_WEB_SHOW_UPLOAD_QUEUE_CREDIT));
	Out.Replace(_T("[ShowUploadQueueComment]"), _GetPlainResString(IDS_WEB_UPLOAD_QUEUE_COMMENT));
	Out.Replace(_T("[ShowUploadQueueBannedComment]"), _GetPlainResString(IDS_WEB_UPLOAD_QUEUE_BANNED_COMMENT));
	Out.Replace(_T("[ShowUploadQueueFriendComment]"), _GetPlainResString(IDS_WEB_UPLOAD_QUEUE_FRIEND_COMMENT));
	Out.Replace(_T("[ShowUploadQueueCreditComment]"), _GetPlainResString(IDS_WEB_UPLOAD_QUEUE_CREDIT_COMMENT));
	Out.Replace(_T("[ShowQueue]"), _GetPlainResString(IDS_SHOW));
	Out.Replace(_T("[HideQueue]"), _GetPlainResString(IDS_HIDE));
	Out.Replace(_T("[ShowQueueBanned]"), _GetPlainResString(IDS_SHOW));
	Out.Replace(_T("[HideQueueBanned]"), _GetPlainResString(IDS_HIDE));
	Out.Replace(_T("[ShowQueueFriend]"), _GetPlainResString(IDS_SHOW));
	Out.Replace(_T("[HideQueueFriend]"), _GetPlainResString(IDS_HIDE));
	Out.Replace(_T("[ShowQueueCredit]"), _GetPlainResString(IDS_SHOW));
	Out.Replace(_T("[HideQueueCredit]"), _GetPlainResString(IDS_HIDE));
	Out.Replace(_T("[RefreshTimeForm]"), _GetPlainResString(IDS_WEB_REFRESH_TIME));
	Out.Replace(_T("[RefreshTimeComment]"), _GetPlainResString(IDS_WEB_REFRESH_COMMENT));
	Out.Replace(_T("[SpeedForm]"), _GetPlainResString(IDS_SPEED_LIMITS));
	Out.Replace(_T("[MaxDown]"), _GetPlainResString(IDS_ST_ACTIVEDOWNLOAD));
	Out.Replace(_T("[MaxUp]"), _GetPlainResString(IDS_ST_ACTIVEUPLOAD));
	Out.Replace(_T("[SpeedCapForm]"), _GetPlainResString(IDS_CAPACITY_LIMITS));
	Out.Replace(_T("[MaxCapDown]"), _GetPlainResString(IDS_ST_ACTIVEDOWNLOAD));
	Out.Replace(_T("[MaxCapUp]"), _GetPlainResString(IDS_ST_ACTIVEUPLOAD));
	Out.Replace(_T("[WebControl]"), _GetPlainResString(IDS_WEB_CONTROL));
	Out.Replace(_T("[eMuleAppName]"), _GetPlainResString(IDS_EMULE_PLUS));
	Out.Replace(_T("[Apply]"), _GetPlainResString(IDS_PW_APPLY));

	CString sT;
	int n = (int)theApp.glob_prefs->GetMaxDownload();
	if(n < 0 || n == 65535) n = 0;
	sT.Format(_T("%d"), n);
	Out.Replace(_T("[MaxDownVal]"), sT);
	n = (int)theApp.glob_prefs->GetMaxUpload();
	if(n < 0 || n == 65535) n = 0;
	sT.Format(_T("%d"), n);
	Out.Replace(_T("[MaxUpVal]"), sT);
	sT.Format(_T("%d"), theApp.glob_prefs->GetMaxGraphDownloadRate());
	Out.Replace(_T("[MaxCapDownVal]"), sT);
	sT.Format(_T("%d"), theApp.glob_prefs->GetMaxGraphUploadRate());
	Out.Replace(_T("[MaxCapUpVal]"), sT);

	return Out;

	EMULE_CATCH

	return _T("");
}

CString CWebServer::_GetLoginScreen(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return _T("");

	CString sSession = _ParseURL(Data.sURL, _T("ses"));

	CString Out = _T("");

	Out += pThis->m_Templates.sLogin;

	Out.Replace(_T("[CharSet]"), _GetWebCharSet());
	Out.Replace(_T("[eMuleAppName]"), _GetPlainResString(IDS_EMULE_PLUS));
	Out.Replace(_T("[version]"), CURRENT_VERSION_LONG);
	Out.Replace(_T("[Login]"), _GetPlainResString(IDS_WEB_LOGIN));
	Out.Replace(_T("[EnterPassword]"), _GetPlainResString(IDS_WEB_ENTER_PASSWORD));
	Out.Replace(_T("[LoginNow]"), _GetPlainResString(IDS_WEB_LOGIN_NOW));
	Out.Replace(_T("[WebControl]"), _GetPlainResString(IDS_WEB_CONTROL));

	if(pThis->m_nIntruderDetect >= 1)
		Out.Replace(_T("[FailedLogin]"), _T("<p class=\"failed\">") + _GetPlainResString(IDS_WEB_BADLOGINATTEMPT) + _T("</p>"));
	else
		Out.Replace(_T("[FailedLogin]"), "&nbsp;");

	return Out;

	EMULE_CATCH

	return _T("");
}

CString CWebServer::_GetConnectedServer(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return _T("");

	CString sSession = _ParseURL(Data.sURL, _T("ses"));

	CString HTTPTemp = _T("");
	TCHAR	HTTPTempC[100] = _T("");
	CString OutS = pThis->m_Templates.sConnectedServer;
	OutS.Replace(_T("[ConnectedServer]"), _GetPlainResString(IDS_PW_SERVER));
	OutS.Replace(_T("[Servername]"), _GetPlainResString(IDS_SL_SERVERNAME));
	OutS.Replace(_T("[Status]"), _GetPlainResString(IDS_STATUS));
	OutS.Replace(_T("[Usercount]"), _GetPlainResString(IDS_LUSERS));
	OutS.Replace(_T("[Action]"), _GetPlainResString(IDS_CONNECTING));
	OutS.Replace(_T("[URL_Disconnect]"), IsSessionAdmin(Data,sSession)?CString(_T("/?ses=") + sSession + _T("&w=server&c=disconnect")):GetPermissionDenied());
	OutS.Replace(_T("[URL_Connect]"), IsSessionAdmin(Data,sSession)?CString(_T("/?ses=") + sSession + _T("&w=server&c=connect")):GetPermissionDenied());
	OutS.Replace(_T("[Disconnect]"), _GetPlainResString(IDS_IRC_DISCONNECT));
	OutS.Replace(_T("[Connect]"), _GetPlainResString(IDS_CONNECTTOANYSERVER));
	OutS.Replace(_T("[URL_ServerOptions]"), IsSessionAdmin(Data,sSession)?CString(_T("/?ses=") + sSession + _T("&w=server&c=options")):GetPermissionDenied());
	OutS.Replace(_T("[ServerOptions]"), CString(_GetPlainResString(IDS_SERVER)+_GetPlainResString(IDS_EM_PREFS)));
	OutS.Replace(_T("[WebSearch]"), _GetPlainResString(IDS_SW_WEBBASED));

#ifdef OLD_SOCKETS_ENABLED
	if (theApp.serverconnect->IsConnected()) {
		if(!theApp.serverconnect->IsLowID())
			OutS.Replace(_T("[1]"), _GetPlainResString(IDS_CONNECTED));
		else
			OutS.Replace(_T("[1]"), _GetPlainResString(IDS_CONNECTED) + _T(" (") + _GetPlainResString(IDS_LOWID) + _T(")"));

		CServer* cur_server = theApp.serverconnect->GetCurrentServer();
		OutS.Replace(_T("[2]"), CString(cur_server->GetListName()));

		_stprintf(HTTPTempC, _T("%10i"), cur_server->GetUsers());
		HTTPTemp = HTTPTempC;												
		OutS.Replace(_T("[3]"), HTTPTemp);

	} else if (theApp.serverconnect->IsConnecting()) {
		OutS.Replace(_T("[1]"), _GetPlainResString(IDS_CONNECTING));
		OutS.Replace(_T("[2]"), _T(""));
		OutS.Replace(_T("[3]"), _T(""));
	} else 
#endif //OLD_SOCKETS_ENABLED
	{
		OutS.Replace(_T("[1]"), _GetPlainResString(IDS_DISCONNECTED));
		OutS.Replace(_T("[2]"), _T(""));
		OutS.Replace(_T("[3]"), _T(""));
	}
	return OutS;

	EMULE_CATCH

	return _T("");
}

// We have to add gz-header and some other stuff
// to standard zlib functions
// in order to use gzip in web pages
int CWebServer::_GzipCompress(Bytef *dest, uLongf *destLen, const Bytef *source, uLong sourceLen, int level)
{ 
	EMULE_TRY

	const static int gz_magic[2] = {0x1f, 0x8b}; // gzip magic header
	int err;
	uLong crc;
	z_stream stream = {0};
	stream.zalloc = (alloc_func)0;
	stream.zfree = (free_func)0;
	stream.opaque = (voidpf)0;
	crc = crc32(0L, Z_NULL, 0);
	// init Zlib stream
	// NOTE windowBits is passed < 0 to suppress zlib header
	err = deflateInit2(&stream, level, Z_DEFLATED, -MAX_WBITS, MAX_MEM_LEVEL, Z_DEFAULT_STRATEGY);
	if (err != Z_OK)
		return err;

	sprintf((char*)dest , "%c%c%c%c%c%c%c%c%c%c", gz_magic[0], gz_magic[1],
		Z_DEFLATED, 0 /*flags*/, 0,0,0,0 /*time*/, 0 /*xflags*/, 255);
	// wire buffers
	stream.next_in = (Bytef*) source ;
	stream.avail_in = (uInt)sourceLen;
	stream.next_out = ((Bytef*) dest) + 10;
	stream.avail_out = *destLen - 18;
	// doit
	err = deflate(&stream, Z_FINISH);
	if (err != Z_STREAM_END)
	{
		deflateEnd(&stream);
		return err;
	}
	err = deflateEnd(&stream);
	crc = crc32(crc, (const Bytef *) source ,  sourceLen );
	//CRC
	*(((Bytef*) dest)+10+stream.total_out) = (Bytef)(crc & 0xFF);
	*(((Bytef*) dest)+10+stream.total_out+1) = (Bytef)((crc>>8) & 0xFF);
	*(((Bytef*) dest)+10+stream.total_out+2) = (Bytef)((crc>>16) & 0xFF);
	*(((Bytef*) dest)+10+stream.total_out+3) = (Bytef)((crc>>24) & 0xFF);
	// Length
	*(((Bytef*) dest)+10+stream.total_out+4) = (Bytef)( sourceLen  & 0xFF);
	*(((Bytef*) dest)+10+stream.total_out+5) = (Bytef)(( sourceLen >>8) & 0xFF);
	*(((Bytef*) dest)+10+stream.total_out+6) = (Bytef)(( sourceLen >>16) &	0xFF);
	*(((Bytef*) dest)+10+stream.total_out+7) = (Bytef)(( sourceLen >>24) &	0xFF);
	// return  destLength
	*destLen = 10 + stream.total_out + 8;
	return err;

	EMULE_CATCH

	return -1;
}

bool CWebServer::_IsLoggedIn(ThreadData Data, long lSession)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return "";

	_RemoveTimeOuts(Data,lSession);

	// find our session
	// i should have used CMap there, but i like CArray more ;-)
	for(int i = 0; i < pThis->m_Params.Sessions.GetSize(); i++)
	{
		if(pThis->m_Params.Sessions[i].lSession == lSession && lSession != 0)
		{
			// if found, also reset expiration time
			pThis->m_Params.Sessions[i].startTime = CTime::GetCurrentTime();
			return true;
		}
	}

	EMULE_CATCH

	return false;
}

void CWebServer::_RemoveTimeOuts(ThreadData Data, long lSession) {
	EMULE_TRY

	// remove expired sessions
	CWebServer *pThis = (CWebServer *)Data.pThis;
	pThis->UpdateSessionCount();

	EMULE_CATCH
}

bool CWebServer::_RemoveSession(ThreadData Data, long lSession)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return "";

	// find our session
	for(int i = 0; i < pThis->m_Params.Sessions.GetSize(); i++)
	{
		if(pThis->m_Params.Sessions[i].lSession == lSession && lSession != 0)
		{
			pThis->m_Params.Sessions.RemoveAt(i);
			CString t_ulCurIP;
			t_ulCurIP.Format(_T("%i.%i.%i.%i"),(uint8)pThis->m_ulCurIP,(uint8)(pThis->m_ulCurIP>>8),(uint8)(pThis->m_ulCurIP>>16),(uint8)(pThis->m_ulCurIP>>24));
			theApp.emuledlg->AddLogLine(true,IDS_WEB_SESSIONEND,t_ulCurIP);
			return true;
		}
	}

	EMULE_CATCH

	return false;
}

Session CWebServer::GetSessionByID(ThreadData Data,long sessionID) {
	EMULE_TRY
	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis != NULL) {
		for(int i = 0; i < pThis->m_Params.Sessions.GetSize(); i++)
		{
			if(pThis->m_Params.Sessions[i].lSession == sessionID && sessionID != 0)
				return pThis->m_Params.Sessions.GetAt(i);
		}
	}

	EMULE_CATCH

	Session ses;
	ses.admin=false;
	ses.startTime = 0;

	return ses;
}

bool CWebServer::IsSessionAdmin(ThreadData Data,CString SsessionID){
	EMULE_TRY
	long sessionID=_tstoi64(SsessionID);
	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis != NULL) {
		for(int i = 0; i < pThis->m_Params.Sessions.GetSize(); i++)
		{
			if(pThis->m_Params.Sessions[i].lSession == sessionID && sessionID != 0)
				return pThis->m_Params.Sessions[i].admin;
		}
	}
	EMULE_CATCH
	return false;
}

CString CWebServer::GetPermissionDenied() {
	return _T("javascript:alert(\'")+_GetPlainResString(IDS_ACCESSDENIED)+_T("\')");
}

bool CWebServer::_GetFileHash(CString sHash, uchar *FileHash)
{
	EMULE_TRY

	if (sHash.GetLength() != 32)
		return false;
	char hex_byte[3];
	int byte;
	hex_byte[2] = '\0';
	for (int i = 0; i < 16; i++) 
	{
		hex_byte[0] = sHash.GetAt(i*2);
		hex_byte[1] = sHash.GetAt((i*2 + 1));
		if (!_istxdigit(hex_byte[0]) || !_istxdigit(hex_byte[1])) {
			md4clr(FileHash);
			return false;
		} else {
			sscanf(hex_byte, "%02x", &byte);
			FileHash[i] = (uchar)byte;
		}
	}
	return true;

	EMULE_CATCH

	md4clr(FileHash);
	return false;
}

CString	CWebServer::_GetPlainResString(UINT nID, bool noquote)
{
	EMULE_TRY

	CString sRet = GetResString(nID);
	sRet.Replace(_T("&"), _T(""));
	if(noquote)
	{
		sRet.Replace(_T("'"), _T("&#8217;"));
		sRet.Replace(_T("\n"), _T("\\n"));
	}
	return sRet;

	EMULE_CATCH

	return _T("");
}

// EC + kuchin
CString	CWebServer::_GetWebCharSet()
{
	EMULE_TRY

	switch (theApp.glob_prefs->GetLanguageID())
	{
		case MAKELANGID(LANG_POLISH,SUBLANG_DEFAULT):				return _T("windows-1250");
		case MAKELANGID(LANG_RUSSIAN,SUBLANG_DEFAULT):				return _T("windows-1251");
		case MAKELANGID(LANG_GREEK,SUBLANG_DEFAULT):				return _T("ISO-8859-7");
		case MAKELANGID(LANG_HEBREW,SUBLANG_DEFAULT):				return _T("ISO-8859-8");
		case MAKELANGID(LANG_KOREAN,SUBLANG_DEFAULT):				return _T("EUC-KR");
		case MAKELANGID(LANG_CHINESE,SUBLANG_CHINESE_SIMPLIFIED):	return _T("GB2312");
		case MAKELANGID(LANG_CHINESE,SUBLANG_CHINESE_TRADITIONAL):	return _T("Big5");
		case MAKELANGID(LANG_LITHUANIAN,SUBLANG_DEFAULT):			return _T("windows-1257");
		case MAKELANGID(LANG_ROMANIAN,SUBLANG_DEFAULT):				return _T("windows-1250");
		case MAKELANGID(LANG_TURKISH,SUBLANG_DEFAULT):				return _T("windows-1254");
	}

	EMULE_CATCH

	// Western (Latin) includes Catalan, Danish, Dutch, English, Faeroese, Finnish, French,
	// German, Galician, Irish, Icelandic, Italian, Norwegian, Portuguese, Spanish and Swedish
	return _T("ISO-8859-1");
}

// Ornis: creating the progressbar. colored if ressources are given/available
CString CWebServer::_GetDownloadGraph(ThreadData Data,CString filehash)
{
	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return _T("");

	CPartFile* cur_file;
	uchar fileid[16];
	DecodeBase16(filehash.GetBuffer(),filehash.GetLength(),fileid);

	CString Out = _T("");
	CString temp = _T("");

	// cool style
	CString progresscolor[12];	//Ju1i3n - merged with eMulePlus by janes bong
	progresscolor[0]=_T("green.gif");
	progresscolor[1]=_T("black.gif");
	progresscolor[2]=_T("yellow.gif");
	progresscolor[3]=_T("red.gif");
	//Ju1i3n start - merged with eMulePlus by janes bong
	progresscolor[4]=_T("blue1.gif");
	progresscolor[5]=_T("blue2.gif");
	progresscolor[6]=_T("blue3.gif");
	progresscolor[7]=_T("blue4.gif");
	progresscolor[8]=_T("blue5.gif");
	progresscolor[9]=_T("blue6.gif");
	progresscolor[10]=_T("greenpercent.gif");
	progresscolor[11]=_T("transparent.gif");
	//Ju1i3n end

	//SyruS show completed files (0.28b)
	//CKnownFile* known_file=theApp.knownfiles->FindKnownFileByID(fileid);
	//if (known_file && !known_file->IsPartFile()) {
	cur_file=theApp.downloadqueue->GetFileByID(fileid); // DonGato: solve non completed files shown as complete
	if (cur_file==0 || !cur_file->IsPartFile()) {
		temp.Format(pThis->m_Templates.sProgressbarImgsPercent+_T("<br>"),progresscolor[10],pThis->m_Templates.iProgressbarWidth);
		Out+=temp;
		temp.Format(pThis->m_Templates.sProgressbarImgs,progresscolor[0],pThis->m_Templates.iProgressbarWidth);
		Out+=temp;
	}
	else {
		//cur_file=theApp.downloadqueue->GetFileByID(fileid);
		CString s_ChunkBar=cur_file->GetProgressString(pThis->m_Templates.iProgressbarWidth);

		// and now make a graph out of the array - need to be in a progressive way
		uint8 lastcolor=1;
		uint16 lastindex=0;
		for (uint16 i=0;i<pThis->m_Templates.iProgressbarWidth;i++) {
			if (lastcolor!= _tstoi(s_ChunkBar.Mid(i,1))  ) {
				if (i>lastindex) {
					temp.Format(pThis->m_Templates.sProgressbarImgs ,progresscolor[lastcolor],i-lastindex);

					Out+=temp;
				}
				lastcolor=_tstoi(s_ChunkBar.Mid(i,1));
				lastindex=i;
			}
		}
		temp.Format(pThis->m_Templates.sProgressbarImgs,progresscolor[lastcolor],pThis->m_Templates.iProgressbarWidth-lastindex);
		Out+=temp;
		int compl=(int)((pThis->m_Templates.iProgressbarWidth/100)*cur_file->GetPercentCompleted());
		(compl>0)?temp.Format(pThis->m_Templates.sProgressbarImgsPercent+_T("<br>"),progresscolor[10],compl) :temp.Format(pThis->m_Templates.sProgressbarImgsPercent+_T("<br>"),progresscolor[11],5);	//Ju1i3n - merged with eMulePlus by janes bong
		Out=temp+Out;
	}
	return Out;
}


CString	CWebServer::_GetSearch(ThreadData Data)
{
	EMULE_TRY

	CWebServer *pThis = (CWebServer *)Data.pThis;
	if(pThis == NULL)
		return _T("");

	CString sSession = _ParseURL(Data.sURL, _T("ses"));
	CString Out = pThis->m_Templates.sSearch;

	if (_ParseURL(Data.sURL, _T("downloads")) != "" && IsSessionAdmin(Data,sSession) ) {
		CString downloads=_ParseURLArray(Data.sURL,_T("downloads"));
				
		CString resToken;
		int curPos=0;
		resToken= downloads.Tokenize(_T("|"),curPos);
		while (resToken != "")
		{
			uchar fileid[16];
			DecodeBase16(resToken.GetBuffer(),resToken.GetLength(),fileid);
			theApp.searchlist->AddFileToDownloadByHash(fileid);
			resToken= downloads.Tokenize(_T("|"),curPos);
		}
	}

	if(_ParseURL(Data.sURL, _T("tosearch")) != "" && IsSessionAdmin(Data,sSession) )
	{
		// perform search
		theApp.emuledlg->searchwnd.DeleteAllSearchs();
		theApp.emuledlg->searchwnd.DoNewSearch(
			_ParseURL(Data.sURL, _T("tosearch")),
			_ParseURL(Data.sURL, _T("type")),
			_tstol(_ParseURL(Data.sURL, _T("min")))*1048576,
			_tstol(_ParseURL(Data.sURL, _T("max")))*1048576,
			_tstoi(_ParseURL(Data.sURL, _T("avail"))),
			_ParseURL(Data.sURL, _T("ext")),
			(_ParseURL(Data.sURL, _T("global"))=="on")
			);
		Out.Replace(_T("[Message]"),_GetPlainResString(IDS_SW_SEARCHINGINFO));
	}
	else if(_ParseURL(Data.sURL, _T("tosearch")) != "" && !IsSessionAdmin(Data,sSession) ) {
		Out.Replace(_T("[Message]"),_GetPlainResString(IDS_ACCESSDENIED));
	}
	else Out.Replace(_T("[Message]"),_T(""));

	CString result=pThis->m_Templates.sSearchHeader +theApp.searchlist->GetWebList(pThis->m_Templates.sSearchResultLine);

	Out.Replace(_T("[SEARCHINFOMSG]"),_T(""));
	Out.Replace(_T("[RESULTLIST]"), result);
	Out.Replace(_T("[Result]"), GetResString(IDS_SW_RESULT) );
	Out.Replace(_T("[Session]"), sSession);
	Out.Replace(_T("[WebSearch]"), _GetPlainResString(IDS_SW_WEBBASED));
	Out.Replace(_T("[Name]"), _GetPlainResString(IDS_SW_NAME));
	Out.Replace(_T("[Type]"), _GetPlainResString(IDS_TYPE));
	Out.Replace(_T("[Any]"), _GetPlainResString(IDS_SEARCH_ANY));
	Out.Replace(_T("[Audio]"), _GetPlainResString(IDS_SEARCH_AUDIO));
	Out.Replace(_T("[Image]"), _GetPlainResString(IDS_SEARCH_PICS));
	Out.Replace(_T("[Video]"), _GetPlainResString(IDS_SEARCH_VIDEO));
	Out.Replace(_T("[Document]"), _GetPlainResString(IDS_SEARCH_DOC));
	Out.Replace(_T("[Other]"), _T("Other"));
	Out.Replace(_T("[Search]"), _GetPlainResString(IDS_SW_SEARCHBOX));
	Out.Replace(_T("[RefetchResults]"), _GetPlainResString(IDS_SW_REFETCHRES));
	Out.Replace(_T("[Download]"), _GetPlainResString(IDS_DOWNLOAD));
	Out.Replace(_T("[Filesize]"), _GetPlainResString(IDS_DL_SIZE));
	Out.Replace(_T("[Sources]"), _GetPlainResString(IDS_DL_SOURCES));
	Out.Replace(_T("[Filehash]"), _GetPlainResString(IDS_FILEHASH));
	Out.Replace(_T("[Filename]"), _GetPlainResString(IDS_DL_FILENAME));
	Out.Replace(_T("[WebSearch]"), _GetPlainResString(IDS_SW_WEBBASED));

	Out.Replace(_T("[SizeMin]"), _GetPlainResString(IDS_SEARCHMINSIZE));
	Out.Replace(_T("[SizeMax]"), _GetPlainResString(IDS_SEARCHMAXSIZE));
	Out.Replace(_T("[Availabl]"), _GetPlainResString(IDS_SEARCHAVAIL));
	Out.Replace(_T("[Extention]"), _GetPlainResString(IDS_SEARCHEXTENSION_LBL));
	Out.Replace(_T("[Global]"), _GetPlainResString(IDS_SW_GLOBAL));
	Out.Replace(_T("[MB]"), _GetPlainResString(IDS_MBYTES));
	
	return Out;

	EMULE_CATCH
	return _T("");
}

int CWebServer::UpdateSessionCount() {
	EMULE_TRY

	int oldvalue=m_Params.Sessions.GetSize();
	for(int i = 0; i < m_Params.Sessions.GetSize();)
	{
		CTimeSpan ts = CTime::GetCurrentTime() - m_Params.Sessions[i].startTime;
		if(ts.GetTotalSeconds() > SESSION_TIMEOUT_SECS) {
			m_Params.Sessions.RemoveAt(i);
		}
		else
			i++;
	}

	return m_Params.Sessions.GetCount();

	EMULE_CATCH
	return 0;
}
